@@ -486,12 +486,14 @@ BEGIN
 				FROM information_schema.columns 
 				 WHERE table_name = param_target_table
 					AND table_schema = var_data_schema 
-					AND column_name <> ALL(param_columns_exclude) ), ',') || ') SELECT ' 
+					AND column_name <> ALL(param_columns_exclude) 
+                    ORDER BY ordinal_position ), ',') || ') SELECT ' 
 					|| array_to_string(ARRAY(SELECT quote_ident(column_name::text) 
 				FROM information_schema.columns 
 				 WHERE table_name = param_staging_table
 					AND table_schema = var_staging_schema 
-					AND column_name <> ALL( param_columns_exclude) ), ',') ||' FROM ' 
+					AND column_name <> ALL( param_columns_exclude) 
+                    ORDER BY ordinal_position ), ',') ||' FROM ' 
 					|| var_staging_schema || '.' || param_staging_table || ';';
 	RAISE NOTICE '%', var_sql;
 	EXECUTE (var_sql);
