@@ -1,10 +1,5 @@
 -- This function determines the levenshtein distance irespective of case.
 CREATE OR REPLACE FUNCTION levenshtein_ignore_case(VARCHAR, VARCHAR) RETURNS INTEGER
 AS $_$
-DECLARE
-  result INTEGER;
-BEGIN
-  result := levenshtein(upper($1), upper($2));
-  RETURN result;
-END
-$_$ LANGUAGE plpgsql;
+  SELECT levenshtein(COALESCE(upper($1),''), COALESCE(upper($2),''));
+$_$ LANGUAGE sql IMMUTABLE;
