@@ -1,5 +1,4 @@
 /**********************************************************************
- * $Id: cu_libgeom.c 13698 2015-06-24 20:21:57Z pramsey $
  *
  * PostGIS - Spatial Types for PostgreSQL
  * http://postgis.net
@@ -216,8 +215,8 @@ static void test_lwgeom_calculate_gbox(void)
 	lwgeom_free(g);
 	
 	/* Geometry with NaN 0101000020E8640000000000000000F8FF000000000000F8FF */
-	/* NaN should show up in bbox */
-	g = lwgeom_from_hexwkb("0101000020E8640000000000000000F8FF000000000000F8FF", LW_PARSER_CHECK_NONE);
+	/* NaN should show up in bbox for "SRID=4326;POINT(0 NaN)" */
+	g = lwgeom_from_hexwkb("0101000020E86400000000000000000000000000000000F8FF", LW_PARSER_CHECK_NONE);
 	lwgeom_calculate_gbox_cartesian(g, &b);
 	CU_ASSERT(isnan(b.ymax));
 	lwgeom_free(g);	
@@ -241,7 +240,6 @@ static void test_gbox_serialized_size(void)
 
 
 
-
 static void test_lwgeom_from_gserialized(void)
 {
 	LWGEOM *geom;
@@ -250,7 +248,7 @@ static void test_lwgeom_from_gserialized(void)
 	char *out_ewkt;
 	int i = 0;
 
-	char ewkt[][512] =
+	char *ewkt[] =
 	{
 		"POINT EMPTY",
 		"POINT(0 0.2)",
@@ -271,13 +269,13 @@ static void test_lwgeom_from_gserialized(void)
 		"SRID=4326;MULTIPOLYGON EMPTY",
 		"SRID=4326;GEOMETRYCOLLECTION(POINT(0 1),POLYGON((-1 -1,-1 2.5,2 2,2 -1,-1 -1),(0 0,0 1,1 1,1 0,0 0)),MULTIPOLYGON(((-1 -1,-1 2.5,2 2,2 -1,-1 -1),(0 0,0 1,1 1,1 0,0 0),(-0.5 -0.5,-0.5 -0.4,-0.4 -0.4,-0.4 -0.5,-0.5 -0.5))))",
 		"SRID=4326;GEOMETRYCOLLECTION EMPTY",
-		"SRID=4326;GEOMETRYCOLLECTION(POINT EMPTY, MULTIPOLYGON EMPTY)",
+		"SRID=4326;GEOMETRYCOLLECTION(POINT EMPTY,MULTIPOLYGON EMPTY)",
 		"MULTICURVE((5 5 1 3,3 5 2 2,3 3 3 1,0 3 1 1),CIRCULARSTRING(0 0 0 0,0.26794 1 3 -2,0.5857864 1.414213 1 2))",
 		"MULTISURFACE(CURVEPOLYGON(CIRCULARSTRING(-2 0,-1 -1,0 0,1 -1,2 0,0 2,-2 0),(-1 0,0 0.5,1 0,0 1,-1 0)),((7 8,10 10,6 14,4 11,7 8)))",
 		"MULTISURFACE(CURVEPOLYGON(CIRCULARSTRING EMPTY))",
 	};
 
-	for ( i = 0; i < 13; i++ )
+	for ( i = 0; i < (sizeof ewkt/sizeof(char*)); i++ )
 	{
 		LWGEOM* geom2;
 
@@ -641,8 +639,8 @@ static void test_lwgeom_flip_coordinates(void)
 
 
 	/*
-	     * Srid
-	     */
+	* Srid
+	*/
 
 	do_lwgeom_flip_coordinates(
 	    "SRID=4326;POINT(1 2)",
@@ -934,6 +932,61 @@ static void test_lwgeom_same(void)
 
 }
 
+/*
+ * Test lwgeom_force_curve
+ */
+static void test_lwgeom_as_curve(void)
+{
+	LWGEOM *geom;
+	LWGEOM *geom2;
+	char *in_ewkt, *out_ewkt;
+
+	geom = lwgeom_from_wkt("LINESTRING(0 0, 10 0)", LW_PARSER_CHECK_NONE);
+	geom2 = lwgeom_as_curve(geom);
+	in_ewkt = "COMPOUNDCURVE((0 0,10 0))";
+	out_ewkt = lwgeom_to_ewkt(geom2);
+	if (strcmp(in_ewkt, out_ewkt))
+		fprintf(stderr, "\nExp:   %s\nObt:  %s\n", in_ewkt, out_ewkt);
+	CU_ASSERT_STRING_EQUAL(in_ewkt, out_ewkt);
+	lwfree(out_ewkt);
+	lwgeom_free(geom);
+	lwgeom_free(geom2);
+
+	geom = lwgeom_from_wkt("MULTILINESTRING((0 0, 10 0))", LW_PARSER_CHECK_NONE);
+	geom2 = lwgeom_as_curve(geom);
+	in_ewkt = "MULTICURVE((0 0,10 0))";
+	out_ewkt = lwgeom_to_ewkt(geom2);
+	if (strcmp(in_ewkt, out_ewkt))
+		fprintf(stderr, "\nExp:   %s\nObt:  %s\n", in_ewkt, out_ewkt);
+	CU_ASSERT_STRING_EQUAL(in_ewkt, out_ewkt);
+	lwfree(out_ewkt);
+	lwgeom_free(geom);
+	lwgeom_free(geom2);
+
+	geom = lwgeom_from_wkt("POLYGON((0 0, 10 0, 10 10, 0 10, 0 0))", LW_PARSER_CHECK_NONE);
+	geom2 = lwgeom_as_curve(geom);
+	in_ewkt = "CURVEPOLYGON((0 0,10 0,10 10,0 10,0 0))";
+	out_ewkt = lwgeom_to_ewkt(geom2);
+	if (strcmp(in_ewkt, out_ewkt))
+		fprintf(stderr, "\nExp:   %s\nObt:  %s\n", in_ewkt, out_ewkt);
+	CU_ASSERT_STRING_EQUAL(in_ewkt, out_ewkt);
+	lwfree(out_ewkt);
+	lwgeom_free(geom);
+	lwgeom_free(geom2);
+
+	geom = lwgeom_from_wkt("MULTIPOLYGON(((0 0, 10 0, 10 10, 0 10, 0 0)))", LW_PARSER_CHECK_NONE);
+	geom2 = lwgeom_as_curve(geom);
+	in_ewkt = "MULTISURFACE(((0 0,10 0,10 10,0 10,0 0)))";
+	out_ewkt = lwgeom_to_ewkt(geom2);
+	if (strcmp(in_ewkt, out_ewkt))
+		fprintf(stderr, "\nExp:   %s\nObt:  %s\n", in_ewkt, out_ewkt);
+	CU_ASSERT_STRING_EQUAL(in_ewkt, out_ewkt);
+	lwfree(out_ewkt);
+	lwgeom_free(geom);
+	lwgeom_free(geom2);
+
+}
+
 static void test_lwline_from_lwmpoint(void)
 {
 	LWLINE *line;
@@ -951,13 +1004,81 @@ static void test_lwline_from_lwmpoint(void)
 	lwmpoint_free(mpoint);
 }
 
+/*
+ * Test lwgeom_scale
+ */
+static void test_lwgeom_scale(void)
+{
+	LWGEOM *geom;
+	POINT4D factor;
+	char *out_ewkt;
+	GBOX *box;
+
+	geom = lwgeom_from_wkt("SRID=4326;GEOMETRYCOLLECTION(POINT(0 1 2 3),POLYGON((-1 -1 0 1,-1 2.5 0 1,2 2 0 1,2 -1 0 1,-1 -1 0 1),(0 0 1 2,0 1 1 2,1 1 1 2,1 0 2 3,0 0 1 2)),LINESTRING(0 0 0 0, 1 2 3 4))", LW_PARSER_CHECK_NONE);
+	factor.x = 2; factor.y = 3; factor.z = 4; factor.m = 5;
+	lwgeom_scale(geom, &factor);
+	out_ewkt = lwgeom_to_ewkt(geom);
+	ASSERT_STRING_EQUAL(out_ewkt, "SRID=4326;GEOMETRYCOLLECTION(POINT(0 3 8 15),POLYGON((-2 -3 0 5,-2 7.5 0 5,4 6 0 5,4 -3 0 5,-2 -3 0 5),(0 0 4 10,0 3 4 10,2 3 4 10,2 0 8 15,0 0 4 10)),LINESTRING(0 0 0 0,2 6 12 20))");
+	lwgeom_free(geom);
+	lwfree(out_ewkt);
+
+	geom = lwgeom_from_wkt("POINT(1 1 1 1)", LW_PARSER_CHECK_NONE);
+	lwgeom_add_bbox(geom);
+	factor.x = 2; factor.y = 3; factor.z = 4; factor.m = 5;
+	lwgeom_scale(geom, &factor);
+	box = geom->bbox;
+	ASSERT_DOUBLE_EQUAL(box->xmin, 2);
+	ASSERT_DOUBLE_EQUAL(box->xmax, 2);
+	ASSERT_DOUBLE_EQUAL(box->ymin, 3);
+	ASSERT_DOUBLE_EQUAL(box->ymax, 3);
+	ASSERT_DOUBLE_EQUAL(box->zmin, 4);
+	ASSERT_DOUBLE_EQUAL(box->zmax, 4);
+	ASSERT_DOUBLE_EQUAL(box->mmin, 5);
+	ASSERT_DOUBLE_EQUAL(box->mmax, 5);
+	lwgeom_free(geom);
+}
+
+void test_gbox_same_2d(void);
+void test_gbox_same_2d(void)
+{
+	LWGEOM* g1 = lwgeom_from_wkt("LINESTRING(0 0, 1 1)", LW_PARSER_CHECK_NONE);
+    LWGEOM* g2 = lwgeom_from_wkt("LINESTRING(0 0, 0 1, 1 1)", LW_PARSER_CHECK_NONE);
+    LWGEOM* g3 = lwgeom_from_wkt("LINESTRING(0 0, 1 1.000000000001)", LW_PARSER_CHECK_NONE);
+
+    lwgeom_add_bbox(g1);
+    lwgeom_add_bbox(g2);
+    lwgeom_add_bbox(g3);
+
+    CU_ASSERT_TRUE(gbox_same_2d(g1->bbox, g2->bbox));
+    CU_ASSERT_FALSE(gbox_same_2d(g1->bbox, g3->bbox));
+    
+    /* Serializing a GBOX with precise coordinates renders the boxes not strictly equal,
+     * but still equal according to gbox_same_2d_float.
+     */
+    GSERIALIZED* s3 = gserialized_from_lwgeom(g3, LW_FALSE, NULL);
+    GBOX s3box;
+    gserialized_read_gbox_p(s3, &s3box);
+
+    CU_ASSERT_FALSE(gbox_same_2d(g3->bbox, &s3box));
+    CU_ASSERT_TRUE(gbox_same_2d_float(g3->bbox, &s3box));
+
+    /* The serialized box equals itself by either the exact or closest-float compares */
+    CU_ASSERT_TRUE(gbox_same_2d(&s3box, &s3box));
+    CU_ASSERT_TRUE(gbox_same_2d_float(&s3box, &s3box));
+
+    lwgeom_free(g1);
+    lwgeom_free(g2);
+    lwgeom_free(g3);
+    lwfree(s3);
+}
+
 /*
 ** Used by test harness to register the tests in this file.
 */
 void libgeom_suite_setup(void);
 void libgeom_suite_setup(void)
 {
-	CU_pSuite suite = CU_add_suite("Serialization/Deserialization", NULL, NULL);
+	CU_pSuite suite = CU_add_suite("serialization/deserialization", NULL, NULL);
 	PG_ADD_TEST(suite, test_typmod_macros);
 	PG_ADD_TEST(suite, test_flags_macros);
 	PG_ADD_TEST(suite, test_serialized_srid);
@@ -977,5 +1098,8 @@ void libgeom_suite_setup(void)
 	PG_ADD_TEST(suite, test_lwgeom_is_empty);
 	PG_ADD_TEST(suite, test_lwgeom_same);
 	PG_ADD_TEST(suite, test_lwline_from_lwmpoint);
+	PG_ADD_TEST(suite, test_lwgeom_as_curve);
+	PG_ADD_TEST(suite, test_lwgeom_scale);
 	PG_ADD_TEST(suite, test_gserialized_is_empty);
+    PG_ADD_TEST(suite, test_gbox_same_2d);
 }
