@@ -1,5 +1,4 @@
 /**********************************************************************
- * $Id: cu_print.c 6160 2010-11-01 01:28:12Z pramsey $
  *
  * PostGIS - Spatial Types for PostgreSQL
  * http://postgis.net
@@ -49,7 +48,7 @@ static void test_misc_simplify(void)
 	char *wkt_out;
 
 	geom = lwgeom_from_wkt("LINESTRING(0 0,0 10,0 51,50 20,30 20,7 32)", LW_PARSER_CHECK_NONE);
-	geom2d = lwgeom_simplify(geom,2);
+	geom2d = lwgeom_simplify(geom, 2, LW_FALSE);
 	wkt_out = lwgeom_to_ewkt(geom2d);
 	CU_ASSERT_STRING_EQUAL("LINESTRING(0 0,0 51,50 20,30 20,7 32)",wkt_out);
 	lwgeom_free(geom);
@@ -57,7 +56,7 @@ static void test_misc_simplify(void)
 	lwfree(wkt_out);
 
 	geom = lwgeom_from_wkt("MULTILINESTRING((0 0,0 10,0 51,50 20,30 20,7 32))", LW_PARSER_CHECK_NONE);
-	geom2d = lwgeom_simplify(geom,2);
+	geom2d = lwgeom_simplify(geom, 2, LW_FALSE);
 	wkt_out = lwgeom_to_ewkt(geom2d);
 	CU_ASSERT_STRING_EQUAL("MULTILINESTRING((0 0,0 51,50 20,30 20,7 32))",wkt_out);
 	lwgeom_free(geom);
@@ -119,16 +118,38 @@ static void test_misc_wkb(void)
 		
 }
 
+
+static void test_grid(void)
+{
+	gridspec grid;
+	static char *wkt = "MULTIPOLYGON(((0 0, 10 0, 10 10, 0 10, 0 0)))";
+	LWGEOM *geom = lwgeom_from_wkt(wkt, LW_PARSER_CHECK_ALL);
+	LWGEOM *geomgrid;
+	char *str;
+	
+	grid.ipx = grid.ipy = 0;
+	grid.xsize = grid.ysize = 20;
+
+	geomgrid = lwgeom_grid(geom, &grid);
+	str = lwgeom_to_ewkt(geomgrid);
+	CU_ASSERT_STRING_EQUAL(str, "MULTIPOLYGON EMPTY");
+	lwfree(str);
+	lwgeom_free(geom);
+	lwgeom_free(geomgrid);		
+}
+
+
 /*
 ** Used by the test harness to register the tests in this file.
 */
 void misc_suite_setup(void);
 void misc_suite_setup(void)
 {
-	CU_pSuite suite = CU_add_suite("Miscellaneous", NULL, NULL);
+	CU_pSuite suite = CU_add_suite("miscellaneous", NULL, NULL);
 	PG_ADD_TEST(suite, test_misc_force_2d);
 	PG_ADD_TEST(suite, test_misc_simplify);
 	PG_ADD_TEST(suite, test_misc_count_vertices);
 	PG_ADD_TEST(suite, test_misc_area);
 	PG_ADD_TEST(suite, test_misc_wkb);
+	PG_ADD_TEST(suite, test_grid);
 }
