@@ -1,5 +1,4 @@
 /**********************************************************************
- * $Id: cu_print.c 6160 2010-11-01 01:28:12Z pramsey $
  *
  * PostGIS - Spatial Types for PostgreSQL
  * http://postgis.net
@@ -46,12 +45,12 @@ static void test_ptarray_append_point(void)
 	p.y = 1;
 	ptarray_append_point(line->points, &p, LW_TRUE);
 	wkt = lwgeom_to_text(lwline_as_lwgeom(line));
-	CU_ASSERT_STRING_EQUAL(wkt,"LINESTRING(0 0,1 1,1 1)");
+	ASSERT_STRING_EQUAL(wkt,"LINESTRING(0 0,1 1,1 1)");
 	lwfree(wkt);
 
 	ptarray_append_point(line->points, &p, LW_FALSE);
 	wkt = lwgeom_to_text(lwline_as_lwgeom(line));
-	CU_ASSERT_STRING_EQUAL(wkt,"LINESTRING(0 0,1 1,1 1)");
+	ASSERT_STRING_EQUAL(wkt,"LINESTRING(0 0,1 1,1 1)");
 	lwfree(wkt);
 
 	lwline_free(line);
@@ -68,35 +67,35 @@ static void test_ptarray_insert_point(void)
 	p.y = 1;
 	ptarray_insert_point(line->points, &p, 0);
 	wkt = lwgeom_to_text(lwline_as_lwgeom(line));
-	CU_ASSERT_STRING_EQUAL(wkt,"LINESTRING(1 1)");
+	ASSERT_STRING_EQUAL(wkt,"LINESTRING(1 1)");
 	lwfree(wkt);
 
 	p.x = 2; 
 	p.y = 20;
 	ptarray_insert_point(line->points, &p, 0);
 	wkt = lwgeom_to_text(lwline_as_lwgeom(line));
-	CU_ASSERT_STRING_EQUAL(wkt,"LINESTRING(2 20,1 1)");
+	ASSERT_STRING_EQUAL(wkt,"LINESTRING(2 20,1 1)");
 	lwfree(wkt);
 
 	p.x = 3; 
 	p.y = 30;
 	ptarray_insert_point(line->points, &p, 1);
 	wkt = lwgeom_to_text(lwline_as_lwgeom(line));
-	CU_ASSERT_STRING_EQUAL(wkt,"LINESTRING(2 20,3 30,1 1)");
+	ASSERT_STRING_EQUAL(wkt,"LINESTRING(2 20,3 30,1 1)");
 	lwfree(wkt);
 
 	p.x = 4; 
 	p.y = 40;
 	ptarray_insert_point(line->points, &p, 0);
 	wkt = lwgeom_to_text(lwline_as_lwgeom(line));
-	CU_ASSERT_STRING_EQUAL(wkt,"LINESTRING(4 40,2 20,3 30,1 1)");
+	ASSERT_STRING_EQUAL(wkt,"LINESTRING(4 40,2 20,3 30,1 1)");
 	lwfree(wkt);
 
 	p.x = 5; 
 	p.y = 50;
 	ptarray_insert_point(line->points, &p, 4);
 	wkt = lwgeom_to_text(lwline_as_lwgeom(line));
-	CU_ASSERT_STRING_EQUAL(wkt,"LINESTRING(4 40,2 20,3 30,1 1,5 50)");
+	ASSERT_STRING_EQUAL(wkt,"LINESTRING(4 40,2 20,3 30,1 1,5 50)");
 	lwfree(wkt);
 
 	lwline_free(line);
@@ -114,7 +113,7 @@ static void test_ptarray_append_ptarray(void)
 	ret = ptarray_append_ptarray(line1->points, line2->points, -1);
 	CU_ASSERT(ret == LW_SUCCESS);
 	wkt = lwgeom_to_text(lwline_as_lwgeom(line1));
-	CU_ASSERT_STRING_EQUAL(wkt, "LINESTRING(0 0,0 10,5 5)");
+	ASSERT_STRING_EQUAL(wkt, "LINESTRING(0 0,0 10,5 5)");
 	lwfree(wkt);
 	lwline_free(line2);
 	lwline_free(line1);
@@ -125,7 +124,7 @@ static void test_ptarray_append_ptarray(void)
 	ret = ptarray_append_ptarray(line1->points, line2->points, -1);
 	CU_ASSERT(ret == LW_SUCCESS);
 	wkt = lwgeom_to_text(lwline_as_lwgeom(line1));
-	CU_ASSERT_STRING_EQUAL(wkt, "LINESTRING(0 0,5 5,6 3)");
+	ASSERT_STRING_EQUAL(wkt, "LINESTRING(0 0,5 5,6 3)");
 	lwfree(wkt);
 	lwline_free(line2);
 	lwline_free(line1);
@@ -136,7 +135,7 @@ static void test_ptarray_append_ptarray(void)
 	ret = ptarray_append_ptarray(line1->points, line2->points, -1);
 	CU_ASSERT(ret == LW_SUCCESS);
 	wkt = lwgeom_to_text(lwline_as_lwgeom(line1));
-	CU_ASSERT_STRING_EQUAL(wkt, "LINESTRING EMPTY");
+	ASSERT_STRING_EQUAL(wkt, "LINESTRING EMPTY");
 	lwfree(wkt);
 	lwline_free(line2);
 	lwline_free(line1);
@@ -147,7 +146,7 @@ static void test_ptarray_append_ptarray(void)
 	ret = ptarray_append_ptarray(line1->points, line2->points, 0);
 	CU_ASSERT(ret == LW_SUCCESS);
 	wkt = lwgeom_to_text(lwline_as_lwgeom(line1));
-	CU_ASSERT_STRING_EQUAL(wkt, "LINESTRING(10 4,0 0,5 7,12 43,42 15)");
+	ASSERT_STRING_EQUAL(wkt, "LINESTRING(10 4,0 0,5 7,12 43,42 15)");
 	lwfree(wkt);
 	lwline_free(line2);
 	lwline_free(line1);
@@ -166,7 +165,7 @@ static void test_ptarray_append_ptarray(void)
 	ret = ptarray_append_ptarray(line1->points, line2->points, .7);
 	CU_ASSERT(ret == LW_SUCCESS);
 	wkt = lwgeom_to_text(lwline_as_lwgeom(line1));
-	CU_ASSERT_STRING_EQUAL(wkt, "LINESTRING(10 4,0 0,5 7,5.5 7,12 43,42 15)");
+	ASSERT_STRING_EQUAL(wkt, "LINESTRING(10 4,0 0,5 7,5.5 7,12 43,42 15)");
 	lwfree(wkt);
 	lwline_free(line2);
 	lwline_free(line1);
@@ -177,7 +176,7 @@ static void test_ptarray_append_ptarray(void)
 	ret = ptarray_append_ptarray(line1->points, line2->points, -1);
 	CU_ASSERT(ret == LW_SUCCESS);
 	wkt = lwgeom_to_text(lwline_as_lwgeom(line1));
-	CU_ASSERT_STRING_EQUAL(wkt, "LINESTRING(0 0,0 10,0 0,0 10)");
+	ASSERT_STRING_EQUAL(wkt, "LINESTRING(0 0,0 10,0 0,0 10)");
 	lwfree(wkt);
 	lwline_free(line2);
 	lwline_free(line1);
@@ -197,7 +196,7 @@ static void test_ptarray_append_ptarray(void)
 	ret = ptarray_append_ptarray(line1->points, line2->points, -1);
 	CU_ASSERT(ret == LW_SUCCESS);
 	wkt = lwgeom_to_text(lwline_as_lwgeom(line1));
-	CU_ASSERT_STRING_EQUAL(wkt, "LINESTRING(0 10,10 0,11 0)");
+	ASSERT_STRING_EQUAL(wkt, "LINESTRING(0 10,10 0,11 0)");
 	lwfree(wkt);
 	FLAGS_SET_READONLY(line2->points->flags, 0); /* for lwline_free */
 	lwline_free(line2);
@@ -343,7 +342,7 @@ static void test_ptarray_signed_area()
 
 
 
-static void test_ptarray_desegmentize() 
+static void test_ptarray_unstroke() 
 {
 	LWGEOM *in, *out;
 	char *str;
@@ -352,98 +351,98 @@ static void test_ptarray_desegmentize()
 	   but it looks like the intersection itself is too sloppy in generating the derived point
 	   to accurately reconstruct the circles.
 	in = lwgeom_from_text("POLYGON((0.5 0,0.471177920604846 -0.292635483024192,0.38581929876693 -0.574025148547634,0.247204418453818 -0.833355349529403,0.0606601717798223 -1.06066017177982,-5.44089437167602e-17 -1.11044268820754,-0.0606601717798188 -1.06066017177982,-0.247204418453816 -0.833355349529406,-0.385819298766929 -0.574025148547639,-0.471177920604845 -0.292635483024197,-0.5 -4.84663372329776e-15,-0.471177920604847 0.292635483024187,-0.385819298766932 0.57402514854763,-0.247204418453821 0.833355349529398,-0.0606601717798256 1.06066017177982,3.45538806345173e-16 1.11044268820754,0.0606601717798183 1.06066017177982,0.247204418453816 0.833355349529407,0.385819298766929 0.574025148547638,0.471177920604845 0.292635483024196,0.5 0))");
-	out = lwgeom_desegmentize(in);
+	out = lwgeom_unstroke(in);
 	str = lwgeom_to_wkt(out, WKT_ISO, 8, NULL);
 	printf("%s\n", str);
-	CU_ASSERT_STRING_EQUAL(str, "CIRCULARSTRING(-1 0,0 1,0 -1)");
+	ASSERT_STRING_EQUAL(str, "CIRCULARSTRING(-1 0,0 1,0 -1)");
 	lwgeom_free(in);
 	lwgeom_free(out);
 	lwfree(str);	
 	*/
 	
 	in = lwgeom_from_text("CIRCULARSTRING(-1 0,0 1,0 -1)");
-	out = lwgeom_segmentize(in,8);
+	out = lwgeom_stroke(in,8);
 	lwgeom_free(in);
-  in = out;
-	out = lwgeom_desegmentize(in);
+	in = out;
+	out = lwgeom_unstroke(in);
 	str = lwgeom_to_wkt(out, WKT_ISO, 8, NULL);
-	//printf("%s\n", str);
-	CU_ASSERT_STRING_EQUAL(str, "CIRCULARSTRING(-1 0,0.70710678 0.70710678,0 -1)");
+	// printf("%s\n", str);
+	ASSERT_STRING_EQUAL(str, "CIRCULARSTRING(-1 0,0.70710678 0.70710678,0 -1)");
 	lwgeom_free(in);
 	lwgeom_free(out);
 	lwfree(str);	
 
 	in = lwgeom_from_text("COMPOUNDCURVE(CIRCULARSTRING(-1 0,0 1,0 -1),(0 -1,-1 -1))");
-	out = lwgeom_segmentize(in,8);
+	out = lwgeom_stroke(in,8);
 	lwgeom_free(in);
-  in = out;
-	out = lwgeom_desegmentize(in);
+	in = out;
+	out = lwgeom_unstroke(in);
 	str = lwgeom_to_wkt(out, WKT_ISO, 8, NULL);
-	//printf("%s\n", str);
-	CU_ASSERT_STRING_EQUAL(str, "COMPOUNDCURVE(CIRCULARSTRING(-1 0,0.70710678 0.70710678,0 -1),(0 -1,-1 -1))");
+	// printf("%s\n", str);
+	ASSERT_STRING_EQUAL(str, "COMPOUNDCURVE(CIRCULARSTRING(-1 0,0.70710678 0.70710678,0 -1),(0 -1,-1 -1))");
 	lwgeom_free(in);
 	lwgeom_free(out);
 	lwfree(str);	
 
 	in = lwgeom_from_text("COMPOUNDCURVE((-3 -3,-1 0),CIRCULARSTRING(-1 0,0 1,0 -1),(0 -1,0 -1.5,0 -2),CIRCULARSTRING(0 -2,-1 -3,1 -3),(1 -3,5 5))");
-	out = lwgeom_segmentize(in,8);
+	out = lwgeom_stroke(in,8);
 	lwgeom_free(in);
-  in = out;
-	out = lwgeom_desegmentize(in);
+	in = out;
+	out = lwgeom_unstroke(in);
 	str = lwgeom_to_wkt(out, WKT_ISO, 8, NULL);
-	//printf("%s\n", str);
-	CU_ASSERT_STRING_EQUAL(str, "COMPOUNDCURVE((-3 -3,-1 0),CIRCULARSTRING(-1 0,0.70710678 0.70710678,0 -1),(0 -1,0 -1.5,0 -2),CIRCULARSTRING(0 -2,-0.70710678 -3.7071068,1 -3),(1 -3,5 5))");
+	// printf("%s\n", str);
+	ASSERT_STRING_EQUAL(str, "COMPOUNDCURVE((-3 -3,-1 0),CIRCULARSTRING(-1 0,0.70710678 0.70710678,0 -1),(0 -1,0 -1.5,0 -2),CIRCULARSTRING(0 -2,-0.70710678 -3.7071068,1 -3),(1 -3,5 5))");
 	lwgeom_free(in);
 	lwgeom_free(out);
 	lwfree(str);	
 
 	in = lwgeom_from_text("COMPOUNDCURVE(CIRCULARSTRING(-1 0,0 1,0 -1),CIRCULARSTRING(0 -1,-1 -2,1 -2))");
-	out = lwgeom_segmentize(in,8);
+	out = lwgeom_stroke(in,8);
 	lwgeom_free(in);
-  in = out;
-	out = lwgeom_desegmentize(in);
+	in = out;
+	out = lwgeom_unstroke(in);
 	str = lwgeom_to_wkt(out, WKT_ISO, 8, NULL);
-	//printf("%s\n", str);
-	CU_ASSERT_STRING_EQUAL(str, "COMPOUNDCURVE(CIRCULARSTRING(-1 0,0.70710678 0.70710678,0 -1),CIRCULARSTRING(0 -1,-0.70710678 -2.7071068,1 -2))");
+	// printf("%s\n", str);
+	ASSERT_STRING_EQUAL(str, "COMPOUNDCURVE(CIRCULARSTRING(-1 0,0.70710678 0.70710678,0 -1),CIRCULARSTRING(0 -1,-0.70710678 -2.7071068,1 -2))");
 	lwgeom_free(in);
 	lwgeom_free(out);
 	lwfree(str);	
 	
 	in = lwgeom_from_text("COMPOUNDCURVE((0 0, 1 1), CIRCULARSTRING(1 1, 2 2, 3 1), (3 1, 4 4))");
-	out = lwgeom_segmentize(in,8);
+	out = lwgeom_stroke(in,8);
 	lwgeom_free(in);
-  in = out;
-	out = lwgeom_desegmentize(in);
+	in = out;
+	out = lwgeom_unstroke(in);
 	str = lwgeom_to_wkt(out, WKT_ISO, 8, NULL);
-	CU_ASSERT_STRING_EQUAL(str, "COMPOUNDCURVE((0 0,1 1),CIRCULARSTRING(1 1,2 2,3 1),(3 1,4 4))");
+	ASSERT_STRING_EQUAL(str, "COMPOUNDCURVE((0 0,1 1),CIRCULARSTRING(1 1,2 2,3 1),(3 1,4 4))");
 	lwgeom_free(in);
 	lwgeom_free(out);
-//	printf("%s\n", str);
+	// printf("%s\n", str);
 	lwfree(str);		
 	
 	// See http://trac.osgeo.org/postgis/ticket/2425 
 	// and http://trac.osgeo.org/postgis/ticket/2420 
 	in = lwgeom_from_text("LINESTRING(0 0,10 0,10 10,0 10,0 0)");
-	out = lwgeom_desegmentize(in);
+	out = lwgeom_unstroke(in);
 	str = lwgeom_to_wkt(out, WKT_ISO, 8, NULL);
-	CU_ASSERT_STRING_EQUAL(str, "LINESTRING(0 0,10 0,10 10,0 10,0 0)");
+	ASSERT_STRING_EQUAL(str, "LINESTRING(0 0,10 0,10 10,0 10,0 0)");
 	lwgeom_free(in);
 	lwgeom_free(out);
 	lwfree(str);	
 
 	in = lwgeom_from_text("LINESTRING(10 10,0 10,0 0,10 0)");
-	out = lwgeom_desegmentize(in);
+	out = lwgeom_unstroke(in);
 	str = lwgeom_to_wkt(out, WKT_ISO, 8, NULL);
-	CU_ASSERT_STRING_EQUAL(str, "LINESTRING(10 10,0 10,0 0,10 0)");
+	ASSERT_STRING_EQUAL(str, "LINESTRING(10 10,0 10,0 0,10 0)");
 	// printf("%s\n", str);
 	lwgeom_free(in);
 	lwgeom_free(out);
 	lwfree(str);
 
 	in = lwgeom_from_text("LINESTRING(0 0,10 0,10 10,0 10)");
-	out = lwgeom_desegmentize(in);
+	out = lwgeom_unstroke(in);
 	str = lwgeom_to_wkt(out, WKT_ISO, 8, NULL);
-	CU_ASSERT_STRING_EQUAL(str, "LINESTRING(0 0,10 0,10 10,0 10)");
+	ASSERT_STRING_EQUAL(str, "LINESTRING(0 0,10 0,10 10,0 10)");
 	// printf("%s\n", str);
 	lwgeom_free(in);
 	lwgeom_free(out);
@@ -451,13 +450,14 @@ static void test_ptarray_desegmentize()
 
 	// See http://trac.osgeo.org/postgis/ticket/2412
 	in = lwgeom_from_text("LINESTRING(0 0, 1 1)");
-	out = lwgeom_desegmentize(in);
+	out = lwgeom_unstroke(in);
 	str = lwgeom_to_wkt(out, WKT_ISO, 8, NULL);
-//printf("%s\n", str);
-	CU_ASSERT_STRING_EQUAL(str, "LINESTRING(0 0,1 1)");
+	// printf("%s\n", str);
+	ASSERT_STRING_EQUAL(str, "LINESTRING(0 0,1 1)");
 	lwgeom_free(in);
 	lwgeom_free(out);
 	lwfree(str);		
+
 }
 
 static void test_ptarray_contains_point() 
@@ -674,7 +674,7 @@ static void test_ptarrayarc_contains_point()
 	cu_error_msg_reset();
 	rv = ptarrayarc_contains_point(pa, &pt);
 	//printf("%s\n", cu_error_msg);
-	CU_ASSERT_STRING_EQUAL("ptarrayarc_contains_point called with even number of points", cu_error_msg);
+	ASSERT_STRING_EQUAL("ptarrayarc_contains_point called with even number of points", cu_error_msg);
 
 	/*** Unclosed ring ***/
 	lwline_free(lwline);
@@ -682,11 +682,60 @@ static void test_ptarrayarc_contains_point()
 	pa = lwline->points;
 	cu_error_msg_reset();
 	rv = ptarrayarc_contains_point(pa, &pt);
-	CU_ASSERT_STRING_EQUAL("ptarrayarc_contains_point called on unclosed ring", cu_error_msg);
+	ASSERT_STRING_EQUAL("ptarrayarc_contains_point called on unclosed ring", cu_error_msg);
 
 	lwline_free(lwline);
 }
 
+static void test_ptarray_scale() 
+{
+  LWLINE *line;
+  POINTARRAY *pa;
+  POINT4D factor;
+  const char *wkt;
+  char *wktout;
+
+  wkt = "LINESTRING ZM (0 1 2 3,1 2 3 0,-2 -3 0 -1,-3 0 -1 -2)";
+  line = lwgeom_as_lwline(lwgeom_from_text(wkt));
+  pa = line->points;
+
+  factor.x = factor.y = factor.z = factor.m = 1;
+  ptarray_scale(pa, &factor);
+  wktout = lwgeom_to_text(lwline_as_lwgeom(line));
+  ASSERT_STRING_EQUAL(wktout, wkt);
+  lwfree(wktout);
+  
+  factor.x = 2;
+  wkt = "LINESTRING ZM (0 1 2 3,2 2 3 0,-4 -3 0 -1,-6 0 -1 -2)";
+  ptarray_scale(pa, &factor);
+  wktout = lwgeom_to_text(lwline_as_lwgeom(line));
+  ASSERT_STRING_EQUAL(wktout, wkt);
+  lwfree(wktout);
+  
+  factor.x = 1; factor.y = 3;
+  wkt = "LINESTRING ZM (0 3 2 3,2 6 3 0,-4 -9 0 -1,-6 0 -1 -2)";
+  ptarray_scale(pa, &factor);
+  wktout = lwgeom_to_text(lwline_as_lwgeom(line));
+  ASSERT_STRING_EQUAL(wktout, wkt);
+  lwfree(wktout);
+  
+  factor.x = 1; factor.y = 1; factor.z = -2;
+  wkt = "LINESTRING ZM (0 3 -4 3,2 6 -6 0,-4 -9 -0 -1,-6 0 2 -2)";
+  ptarray_scale(pa, &factor);
+  wktout = lwgeom_to_text(lwline_as_lwgeom(line));
+  ASSERT_STRING_EQUAL(wktout, wkt);
+  lwfree(wktout);
+  
+  factor.x = 1; factor.y = 1; factor.z = 1; factor.m = 2;
+  wkt = "LINESTRING ZM (0 3 -4 6,2 6 -6 0,-4 -9 -0 -2,-6 0 2 -4)";
+  ptarray_scale(pa, &factor);
+  wktout = lwgeom_to_text(lwline_as_lwgeom(line));
+  ASSERT_STRING_EQUAL(wktout, wkt);
+  lwfree(wktout);
+
+  lwline_free(line);
+}
+
 
 /*
 ** Used by the test harness to register the tests in this file.
@@ -694,14 +743,15 @@ static void test_ptarrayarc_contains_point()
 void ptarray_suite_setup(void);
 void ptarray_suite_setup(void)
 {
-	CU_pSuite suite = CU_add_suite("Point Array", NULL, NULL);
+	CU_pSuite suite = CU_add_suite("ptarray", NULL, NULL);
 	PG_ADD_TEST(suite, test_ptarray_append_point);
 	PG_ADD_TEST(suite, test_ptarray_append_ptarray);
 	PG_ADD_TEST(suite, test_ptarray_locate_point);
 	PG_ADD_TEST(suite, test_ptarray_isccw);
 	PG_ADD_TEST(suite, test_ptarray_signed_area);
-	PG_ADD_TEST(suite, test_ptarray_desegmentize);
+	PG_ADD_TEST(suite, test_ptarray_unstroke);
 	PG_ADD_TEST(suite, test_ptarray_insert_point);
 	PG_ADD_TEST(suite, test_ptarray_contains_point);
 	PG_ADD_TEST(suite, test_ptarrayarc_contains_point);
+	PG_ADD_TEST(suite, test_ptarray_scale);
 }
