@@ -249,6 +249,32 @@ static void test_lwgeom_split(void)
 	lwgeom_free(ret);
 	lwgeom_free(geom);
 	lwgeom_free(blade);
+
+  /* See #3401 -- robustness issue */
+  geom = lwgeom_from_wkt("LINESTRING(-180 0,0 0)", LW_PARSER_CHECK_NONE);
+  CU_ASSERT(geom != NULL);
+  blade = lwgeom_from_wkt("POINT(-20 0)", LW_PARSER_CHECK_NONE);
+  ret = lwgeom_split(geom, blade);
+  CU_ASSERT(ret != NULL);
+	{
+		LWCOLLECTION *split = lwgeom_as_lwcollection(ret);
+		LWLINE *l1, *l2;
+		POINT2D pt;
+		CU_ASSERT(split != NULL);
+		l1 = lwgeom_as_lwline(split->geoms[0]);
+		CU_ASSERT(l1 != NULL);
+		getPoint2d_p(l1->points, 1, &pt);
+		ASSERT_DOUBLE_EQUAL(pt.x, -20);
+		ASSERT_DOUBLE_EQUAL(pt.y, 0);
+		l2 = lwgeom_as_lwline(split->geoms[1]);
+		CU_ASSERT(l2 != NULL);
+		getPoint2d_p(l2->points, 0, &pt);
+		ASSERT_DOUBLE_EQUAL(pt.x, -20);
+		ASSERT_DOUBLE_EQUAL(pt.y, 0);
+	}
+	lwgeom_free(ret);
+	lwgeom_free(geom);
+	lwgeom_free(blade);
 }
 
 
