@@ -1,5 +1,4 @@
 /**********************************************************************
- * $Id: g_box.c 12296 2014-03-06 05:53:39Z pramsey $
  *
  * PostGIS - Spatial Types for PostgreSQL
  * Copyright 2009 Paul Ramsey <pramsey@cleverelephant.ca>
@@ -9,8 +8,7 @@
  *
  **********************************************************************/
 
-#ifndef _GNU_SOURCE
-#define _GNU_SOURCE
+#if !HAVE_ISFINITE
 #endif
 
 #include "liblwgeom_internal.h"
@@ -18,6 +16,15 @@
 #include <stdlib.h>
 #include <math.h>
 
+/* Fall back to older finite() if necessary */
+#ifndef HAVE_ISFINITE
+# ifdef HAVE_GNU_ISFINITE
+#  define _GNU_SOURCE
+# else
+#  define isfinite finite
+# endif
+#endif
+
 GBOX* gbox_new(uint8_t flags)
 {
 	GBOX *g = (GBOX*)lwalloc(sizeof(GBOX));
@@ -136,8 +143,7 @@ int gbox_same(const GBOX *g1, const GBOX *g2)
 	if (FLAGS_GET_ZM(g1->flags) != FLAGS_GET_ZM(g2->flags))
 		return LW_FALSE;
 
-	if ( g1->xmin != g2->xmin || g1->ymin != g2->ymin ||
-	     g1->xmax != g2->xmax || g1->ymax != g2->ymax ) return LW_FALSE;
+	if (!gbox_same_2d(g1, g2)) return LW_FALSE;
 
 	if (FLAGS_GET_Z(g1->flags) && (g1->zmin != g2->zmin || g1->zmax != g2->zmax))
 		return LW_FALSE;
@@ -147,6 +153,24 @@ int gbox_same(const GBOX *g1, const GBOX *g2)
 	return LW_TRUE;
 }
 
+int gbox_same_2d(const GBOX *g1, const GBOX *g2)
+{
+    if (g1->xmin == g2->xmin && g1->ymin == g2->ymin &&
+        g1->xmax == g2->xmax && g1->ymax == g2->ymax)
+		return LW_TRUE;
+	return LW_FALSE;
+}
+
+int gbox_same_2d_float(const GBOX *g1, const GBOX *g2)
+{
+  if  ((g1->xmax == g2->xmax || next_float_up(g1->xmax)   == next_float_up(g2->xmax))   &&
+       (g1->ymax == g2->ymax || next_float_up(g1->ymax)   == next_float_up(g2->ymax))   &&
+       (g1->xmin == g2->xmin || next_float_down(g1->xmin) == next_float_down(g1->xmin)) &&
+       (g1->ymin == g2->ymin || next_float_down(g2->ymin) == next_float_down(g2->ymin)))
+      return LW_TRUE;
+  return LW_FALSE;
+}
+
 int gbox_is_valid(const GBOX *gbox)
 {
 	/* X */
@@ -288,6 +312,28 @@ gbox_overlaps_2d(const GBOX *g1, const GBOX *g2)
 	return LW_TRUE;
 }
 
+int 
+gbox_contains_2d(const GBOX *g1, const GBOX *g2)
+{
+	if ( ( g2->xmin < g1->xmin ) || ( g2->xmax > g1->xmax ) ||
+	     ( g2->ymin < g1->ymin ) || ( g2->ymax > g1->ymax ) )
+	{
+		return LW_FALSE;
+	}
+	return LW_TRUE;
+}
+
+int 
+gbox_contains_point2d(const GBOX *g, const POINT2D *p)
+{
+	if ( ( g->xmin <= p->x ) && ( g->xmax >= p->x ) &&
+	     ( g->ymin <= p->y ) && ( g->ymax >= p->y ) )
+	{
+		return LW_TRUE;
+	}
+	return LW_FALSE;
+}
+
 /**
 * Warning, this function is only good for x/y/z boxes, used
 * in unit testing of geodetic box generation.
@@ -520,8 +566,8 @@ static int lwcircstring_calculate_gbox_cartesian(LWCIRCSTRING *curve, GBOX *gbox
 	tmp.flags = flags;
 
 	/* Initialize */
-	gbox->xmin = gbox->ymin = gbox->zmin = gbox->mmin = MAXFLOAT;
-	gbox->xmax = gbox->ymax = gbox->zmax = gbox->mmax = -1 * MAXFLOAT;
+	gbox->xmin = gbox->ymin = gbox->zmin = gbox->mmin = FLT_MAX;
+	gbox->xmax = gbox->ymax = gbox->zmax = gbox->mmax = -1*FLT_MAX;
 
 	for ( i = 2; i < curve->points->npoints; i += 2 )
 	{
