@@ -1,7 +1,7 @@
 /**********************************************************************
  *
  * PostGIS - Spatial Types for PostgreSQL
- * http://postgis.refractions.net
+ * http://postgis.net
  *
  * Copyright (C) 2001-2006 Refractions Research Inc.
  *
@@ -59,7 +59,7 @@ double lwcompound_length(const LWCOMPOUND *comp)
 	LWLINE *line;
 	if ( lwgeom_is_empty((LWGEOM*)comp) )
 		return 0.0;
-	line = lwcompound_segmentize(comp, 32);
+	line = lwcompound_stroke(comp, 32);
 	length = lwline_length(line);
 	lwline_free(line);
 	return length;
@@ -71,7 +71,7 @@ double lwcompound_length_2d(const LWCOMPOUND *comp)
 	LWLINE *line;
 	if ( lwgeom_is_empty((LWGEOM*)comp) )
 		return 0.0;
-	line = lwcompound_segmentize(comp, 32);
+	line = lwcompound_stroke(comp, 32);
 	length = lwline_length_2d(line);
 	lwline_free(line);
 	return length;
@@ -114,8 +114,8 @@ int lwcompound_add_lwgeom(LWCOMPOUND *comp, LWGEOM *geom)
 LWCOMPOUND *
 lwcompound_construct_empty(int srid, char hasz, char hasm)
 {
-        LWCOMPOUND *ret = (LWCOMPOUND*)lwcollection_construct_empty(COMPOUNDTYPE, srid, hasz, hasm);
-        return ret;
+	LWCOMPOUND *ret = (LWCOMPOUND*)lwcollection_construct_empty(COMPOUNDTYPE, srid, hasz, hasm);
+	return ret;
 }
 
 int lwgeom_contains_point(const LWGEOM *geom, const POINT2D *pt)
@@ -191,3 +191,73 @@ lwcompound_contains_point(const LWCOMPOUND *comp, const POINT2D *pt)
 	/* Inside */
 	return LW_INSIDE;
 }	
+
+LWCOMPOUND *
+lwcompound_construct_from_lwline(const LWLINE *lwline)
+{
+  LWCOMPOUND* ogeom = lwcompound_construct_empty(lwline->srid, FLAGS_GET_Z(lwline->flags), FLAGS_GET_M(lwline->flags));
+  lwcompound_add_lwgeom(ogeom, lwgeom_clone((LWGEOM*)lwline));
+	/* ogeom->bbox = lwline->bbox; */
+  return ogeom;
+}
+
+LWPOINT* 
+lwcompound_get_lwpoint(const LWCOMPOUND *lwcmp, int where)
+{
+	int i;
+	int count = 0;
+	int npoints = 0;
+	if ( lwgeom_is_empty((LWGEOM*)lwcmp) )
+		return NULL;
+	
+	npoints = lwgeom_count_vertices((LWGEOM*)lwcmp);
+	if ( where < 0 || where >= npoints )
+	{
+		lwerror("%s: index %d is not in range of number of vertices (%d) in input", __func__, where, npoints);
+		return NULL;
+	}
+	
+	for ( i = 0; i < lwcmp->ngeoms; i++ )
+	{
+		LWGEOM* part = lwcmp->geoms[i];
+		int npoints_part = lwgeom_count_vertices(part);
+		if ( where >= count && where < count + npoints_part )
+		{
+			return lwline_get_lwpoint((LWLINE*)part, where - count);
+		}
+		else
+		{
+			count += npoints_part;
+		}
+	}
+
+	return NULL;	
+}
+
+
+
+LWPOINT *
+lwcompound_get_startpoint(const LWCOMPOUND *lwcmp)
+{
+	return lwcompound_get_lwpoint(lwcmp, 0);
+}
+
+LWPOINT *
+lwcompound_get_endpoint(const LWCOMPOUND *lwcmp)
+{
+	LWLINE *lwline;
+	if ( lwcmp->ngeoms < 1 )
+	{
+		return NULL;
+	}
+	
+	lwline = (LWLINE*)(lwcmp->geoms[lwcmp->ngeoms-1]);
+
+	if ( (!lwline) || (!lwline->points) || (lwline->points->npoints < 1) )
+	{
+		return NULL;
+	}
+	
+	return lwline_get_lwpoint(lwline, lwline->points->npoints-1);
+}
+
