@@ -1,5 +1,4 @@
 /**********************************************************************
- * $Id: lwgeodetic.h 12343 2014-03-25 08:17:56Z pramsey $
  *
  * PostGIS - Spatial Types for PostgreSQL
  * Copyright 2009 Paul Ramsey <pramsey@cleverelephant.ca>
@@ -57,8 +56,8 @@ typedef struct
 /**
 * Conversion functions
 */
-#define deg2rad(d) (PI * (d) / 180.0)
-#define rad2deg(r) (180.0 * (r) / PI)
+#define deg2rad(d) (M_PI * (d) / 180.0)
+#define rad2deg(r) (180.0 * (r) / M_PI)
 
 /**
 * Ape a java function
@@ -143,4 +142,6 @@ int spheroid_project(const GEOGRAPHIC_POINT *r, const SPHEROID *spheroid, double
 * Maintain consistent units (radians?) throughout all calculations
 * Put an index pointer onto LWGEOM itself, and cache the indexed LWGEOM instead of a bare tree
 * only primitive objects should get a tree
-*/
\ No newline at end of file
+*/
+
+
