@@ -81,10 +81,10 @@ circ_node_leaf_new(const POINTARRAY* pa, int i)
 	node->nodes = NULL;
 	node->edge_num = i;
     
-    /* Zero out metadata */
-    node->pt_outside.x = 0.0;
-    node->pt_outside.y = 0.0;
-    node->geom_type = 0;
+	/* Zero out metadata */
+	node->pt_outside.x = 0.0;
+	node->pt_outside.y = 0.0;
+	node->geom_type = 0;
 	
 	return node;
 }
@@ -102,9 +102,9 @@ circ_node_leaf_point_new(const POINTARRAY* pa)
 	tree->nodes = NULL;
 	tree->num_nodes = 0;
 	tree->edge_num = 0;
-    tree->geom_type = POINTTYPE;
-    tree->pt_outside.x = 0.0;
-    tree->pt_outside.y = 0.0;
+	tree->geom_type = POINTTYPE;
+	tree->pt_outside.x = 0.0;
+	tree->pt_outside.y = 0.0;
 	return tree;
 }
 
@@ -315,9 +315,9 @@ circ_node_internal_new(CIRC_NODE** c, int num_nodes)
 	node->num_nodes = num_nodes;
 	node->nodes = c;
 	node->edge_num = -1;
-    node->geom_type = new_geom_type;
-    node->pt_outside.x = 0.0;
-    node->pt_outside.y = 0.0;
+	node->geom_type = new_geom_type;
+	node->pt_outside.x = 0.0;
+	node->pt_outside.y = 0.0;
 	return node;
 }
 
@@ -388,8 +388,9 @@ circ_nodes_merge(CIRC_NODE** nodes, int num_nodes)
 	int num_parents = 0;
 	int j;
 
-	// TODO, roll geom_type *up* as tree is built, changing to collection types as simple types are merged 
-	// TODO, change the distance algorithm to drive down to simple types first, test pip on poly/other cases, then test edges
+	/* TODO, roll geom_type *up* as tree is built, changing to collection types as simple types are merged 
+	 * TODO, change the distance algorithm to drive down to simple types first, test pip on poly/other cases, then test edges
+	 */
 
 	while( num_children > 1 )
 	{
@@ -433,15 +434,15 @@ circ_nodes_merge(CIRC_NODE** nodes, int num_nodes)
 int circ_tree_get_point(const CIRC_NODE* node, POINT2D* pt)
 {
 	if ( circ_node_is_leaf(node) )
-    {
-        pt->x = node->p1->x;
-        pt->y = node->p1->y;
-        return LW_SUCCESS;
-    }
-    else
-    {
-        return circ_tree_get_point(node->nodes[0], pt);
-    }
+	{
+		pt->x = node->p1->x;
+		pt->y = node->p1->y;
+		return LW_SUCCESS;
+	}
+	else
+	{
+		return circ_tree_get_point(node->nodes[0], pt);
+	}
 }
 
 
@@ -552,10 +553,13 @@ circ_node_max_distance(const CIRC_NODE *n1, const CIRC_NODE *n2)
 double
 circ_tree_distance_tree(const CIRC_NODE* n1, const CIRC_NODE* n2, const SPHEROID* spheroid, double threshold)
 {
-	double min_dist = MAXFLOAT;
-	double max_dist = MAXFLOAT;
+	double min_dist = FLT_MAX;
+	double max_dist = FLT_MAX;
 	GEOGRAPHIC_POINT closest1, closest2;
-	double threshold_radians = threshold / spheroid->radius;
+	/* Quietly decrease the threshold just a little to avoid cases where */
+	/* the actual spheroid distance is larger than the sphere distance */
+	/* causing the return value to be larger than the threshold value */
+	double threshold_radians = 0.95 * threshold / spheroid->radius;
 	
 	circ_tree_distance_tree_internal(n1, n2, threshold_radians, &min_dist, &max_dist, &closest1, &closest2);
 
@@ -578,18 +582,20 @@ circ_tree_distance_tree_internal(const CIRC_NODE* n1, const CIRC_NODE* n2, doubl
 	int i;
 	
 	LWDEBUGF(4, "entered, min_dist=%.8g max_dist=%.8g, type1=%d, type2=%d", *min_dist, *max_dist, n1->geom_type, n2->geom_type);
-//	circ_tree_print(n1, 0);
-//	circ_tree_print(n2, 0);
+/*
+	circ_tree_print(n1, 0);
+	circ_tree_print(n2, 0);
+*/
 	
 	/* Short circuit if we've already hit the minimum */
-	if( *min_dist <= threshold )
+	if( *min_dist < threshold || *min_dist == 0.0 )
 		return *min_dist;
 	
 	/* If your minimum is greater than anyone's maximum, you can't hold the winner */
 	if( circ_node_min_distance(n1, n2) > *max_dist )
 	{
 		LWDEBUGF(4, "pruning pair %p, %p", n1, n2);		
-		return MAXFLOAT;
+		return FLT_MAX;
 	}
 	
 	/* If your maximum is a new low, we'll use that as our new global tolerance */
@@ -707,7 +713,7 @@ circ_tree_distance_tree_internal(const CIRC_NODE* n1, const CIRC_NODE* n2, doubl
 	}
 	else
 	{	
-		d_min = MAXFLOAT;
+		d_min = FLT_MAX;
 		/* Drive the recursion into the COLLECTION types first so we end up with */
 		/* pairings of primitive geometries that can be forced into the point-in-polygon */
 		/* tests above. */
@@ -809,8 +815,8 @@ static CIRC_NODE*
 lwpoint_calculate_circ_tree(const LWPOINT* lwpoint)
 {
 	CIRC_NODE* node;
-    node = circ_tree_new(lwpoint->point);
-    node->geom_type = lwgeom_get_type((LWGEOM*)lwpoint);;
+	node = circ_tree_new(lwpoint->point);
+	node->geom_type = lwgeom_get_type((LWGEOM*)lwpoint);;
 	return node;
 }
 
@@ -818,8 +824,8 @@ static CIRC_NODE*
 lwline_calculate_circ_tree(const LWLINE* lwline)
 {
 	CIRC_NODE* node;
-    node = circ_tree_new(lwline->points);
-    node->geom_type = lwgeom_get_type((LWGEOM*)lwline);
+	node = circ_tree_new(lwline->points);
+	node->geom_type = lwgeom_get_type((LWGEOM*)lwline);
 	return node;
 }
 
@@ -855,7 +861,7 @@ lwpoly_calculate_circ_tree(const LWPOLY* lwpoly)
 
 	/* Metatdata about polygons, we need this to apply P-i-P tests */
 	/* selectively when doing distance calculations */
-    node->geom_type = lwgeom_get_type((LWGEOM*)lwpoly);
+	node->geom_type = lwgeom_get_type((LWGEOM*)lwpoly);
 	lwpoly_pt_outside(lwpoly, &(node->pt_outside));
 	
 	return node;
@@ -886,9 +892,7 @@ lwcollection_calculate_circ_tree(const LWCOLLECTION* lwcol)
 	node = circ_nodes_merge(nodes, j);
 	/* Don't need the working list any more */
 	lwfree(nodes);
-	
-    node->geom_type = lwgeom_get_type((LWGEOM*)lwcol);
-    
+	node->geom_type = lwgeom_get_type((LWGEOM*)lwcol);
 	return node;
 }
 
