@@ -1409,6 +1409,11 @@ extern LWGEOM* lwgeom_remove_repeated_points(LWGEOM *in, double tolerance)
 	LWDEBUGF(4, "lwgeom_remove_repeated_points got type %s",
 	         lwtype_name(in->type));
 
+	if(lwgeom_is_empty(in)) 
+	{
+		return lwgeom_clone_deep(in);
+	}
+
 	switch (in->type)
 	{
 	case MULTIPOINTTYPE:
@@ -1431,7 +1436,7 @@ extern LWGEOM* lwgeom_remove_repeated_points(LWGEOM *in, double tolerance)
 	case TRIANGLETYPE:
 	case TINTYPE:
 		/* No point is repeated for a single point, or for Triangle or TIN */
-		return in;
+		return lwgeom_clone_deep(in);
 
 	case CIRCSTRINGTYPE:
 	case COMPOUNDTYPE:
@@ -1439,12 +1444,12 @@ extern LWGEOM* lwgeom_remove_repeated_points(LWGEOM *in, double tolerance)
 	case CURVEPOLYTYPE:
 	case MULTISURFACETYPE:
 		/* Dunno how to handle these, will return untouched */
-		return in;
+		return lwgeom_clone_deep(in);
 
 	default:
 		lwnotice("%s: unsupported geometry type: %s",
 		         __func__, lwtype_name(in->type));
-		return in;
+		return lwgeom_clone_deep(in);
 		break;
 	}
 	return 0;
