@@ -14,16 +14,30 @@
 #include "liblwgeom_internal.h"
 #include "lwgeom_log.h"
 
-#include <float.h>
 #include <stdio.h>
 #include <errno.h>
 #include <assert.h>
+#include "../postgis_svn_revision.h"
 
 /*
  * Lower this to reduce integrity checks
  */
 #define PARANOIA_LEVEL 1
 
+const char *
+lwgeom_version()
+{
+  static char *ptr = NULL;
+  static char buf[256];
+  if ( ! ptr )
+  {
+    ptr = buf;
+    snprintf(ptr, 256, LIBLWGEOM_VERSION" r%d", POSTGIS_SVN_REVISION);
+  }
+
+  return ptr;
+}
+
 
 /**********************************************************************
  * BOX routines
@@ -489,6 +503,27 @@ getPoint3dz_cp(const POINTARRAY *pa, int n)
 }
 
 
+const POINT4D*
+getPoint4d_cp(const POINTARRAY *pa, int n)
+{
+	if ( ! pa ) return 0;
+	
+	if ( ! (FLAGS_GET_Z(pa->flags) && FLAGS_GET_Z(pa->flags)) )
+	{
+		lwerror("getPoint3dz_cp: no Z and M coordinates in point array");
+		return 0; /*error */
+	}
+
+	if ( (n<0) || (n>=pa->npoints))
+	{
+		lwerror("getPoint3dz_cp: point offset out of range");
+		return 0; /*error */
+	}
+
+	return (const POINT4D*)getPoint_internal(pa, n);
+}
+
+
 
 /*
  * set point N to the given value
@@ -784,4 +819,20 @@ interpolate_point4d(POINT4D *A, POINT4D *B, POINT4D *I, double F)
 }
 
 
+int _lwgeom_interrupt_requested = 0;
+void
+lwgeom_request_interrupt() {
+  _lwgeom_interrupt_requested = 1;
+}
+void
+lwgeom_cancel_interrupt() {
+  _lwgeom_interrupt_requested = 0;
+}
 
+lwinterrupt_callback *_lwgeom_interrupt_callback = 0;
+lwinterrupt_callback *
+lwgeom_register_interrupt_callback(lwinterrupt_callback *cb) {
+  lwinterrupt_callback *old = _lwgeom_interrupt_callback;
+  _lwgeom_interrupt_callback = cb;
+  return old;
+}
