@@ -3,7 +3,7 @@
  * PostGIS - Spatial Types for PostgreSQL
  * http://postgis.net
  *
- * Copyright 2011-2012 Sandro Santilli <strk@keybit.net>
+ * Copyright 2011-2014 Sandro Santilli <strk@keybit.net>
  *
  * This is free software; you can redistribute and/or modify it under
  * the terms of the GNU General Public Licence. See the COPYING file.
@@ -219,14 +219,14 @@ ptarray_to_GEOSCoordSeq(const POINTARRAY *pa)
 		{
 			p3d = getPoint3dz_cp(pa, i);
 			p2d = (const POINT2D *)p3d;
+			LWDEBUGF(4, "Point: %g,%g,%g", p3d->x, p3d->y, p3d->z);
 		}
 		else
 		{
 			p2d = getPoint2d_cp(pa, i);
+			LWDEBUGF(4, "Point: %g,%g", p2d->x, p2d->y);
 		}
 
-		LWDEBUGF(4, "Point: %g,%g,%g", p2d->x, p2d->y, p3d->z);
-
 #if POSTGIS_GEOS_VERSION < 33
 		/* Make sure we don't pass any infinite values down into GEOS */
 		/* GEOS 3.3+ is supposed to  handle this stuff OK */
@@ -245,11 +245,81 @@ ptarray_to_GEOSCoordSeq(const POINTARRAY *pa)
 	return sq;
 }
 
+static GEOSGeometry *
+ptarray_to_GEOSLinearRing(const POINTARRAY *pa, int autofix)
+{
+	GEOSCoordSeq sq;
+	GEOSGeom g;
+	POINTARRAY *npa = 0;
 
+	if ( autofix )
+	{
+		/* check ring for being closed and fix if not */
+		if ( ! ptarray_is_closed_2d(pa) ) 
+		{
+			npa = ptarray_addPoint(pa, getPoint_internal(pa, 0), FLAGS_NDIMS(pa->flags), pa->npoints);
+			pa = npa;
+		}
+		/* TODO: check ring for having at least 4 vertices */
+#if 0
+		while ( pa->npoints < 4 ) 
+		{
+			npa = ptarray_addPoint(npa, getPoint_internal(pa, 0), FLAGS_NDIMS(pa->flags), pa->npoints);
+		}
+#endif
+	}
 
+	sq = ptarray_to_GEOSCoordSeq(pa);
+	if ( npa ) ptarray_free(npa);
+	g = GEOSGeom_createLinearRing(sq);
+	return g;
+}
 
 GEOSGeometry *
-LWGEOM2GEOS(const LWGEOM *lwgeom)
+GBOX2GEOS(const GBOX *box)
+{
+	GEOSGeometry* envelope;
+	GEOSGeometry* ring;
+	GEOSCoordSequence* seq = GEOSCoordSeq_create(5, 2);
+	if (!seq) 
+	{
+		return NULL;
+	}
+
+	GEOSCoordSeq_setX(seq, 0, box->xmin);
+	GEOSCoordSeq_setY(seq, 0, box->ymin);
+
+	GEOSCoordSeq_setX(seq, 1, box->xmax);
+	GEOSCoordSeq_setY(seq, 1, box->ymin);
+
+	GEOSCoordSeq_setX(seq, 2, box->xmax);
+	GEOSCoordSeq_setY(seq, 2, box->ymax);
+
+	GEOSCoordSeq_setX(seq, 3, box->xmin);
+	GEOSCoordSeq_setY(seq, 3, box->ymax);
+
+	GEOSCoordSeq_setX(seq, 4, box->xmin);
+	GEOSCoordSeq_setY(seq, 4, box->ymin);
+
+	ring = GEOSGeom_createLinearRing(seq);
+	if (!ring) 
+	{
+		GEOSCoordSeq_destroy(seq);
+		return NULL;
+	}
+
+	envelope = GEOSGeom_createPolygon(ring, NULL, 0);
+	if (!envelope) 
+	{
+		GEOSGeom_destroy(ring);
+		return NULL;
+	}
+
+	return envelope;
+}
+
+GEOSGeometry *
+LWGEOM2GEOS(const LWGEOM *lwgeom, int autofix)
 {
 	GEOSCoordSeq sq;
 	GEOSGeom g, shell;
@@ -267,10 +337,10 @@ LWGEOM2GEOS(const LWGEOM *lwgeom)
 
 	if (lwgeom_has_arc(lwgeom))
 	{
-		LWDEBUG(3, "LWGEOM2GEOS: arced geometry found.");
-
-		lwerror("Exception in LWGEOM2GEOS: curved geometry not supported.");
-		return NULL;
+		LWGEOM *lwgeom_stroked = lwgeom_stroke(lwgeom, 32);
+		GEOSGeometry *g = LWGEOM2GEOS(lwgeom_stroked, autofix);
+		lwgeom_free(lwgeom_stroked);
+		return g;
 	}
 	
 	switch (lwgeom->type)
@@ -310,6 +380,7 @@ LWGEOM2GEOS(const LWGEOM *lwgeom)
 		break;
 	case LINETYPE:
 		lwl = (LWLINE *)lwgeom;
+		/* TODO: if (autofix) */
 		if ( lwl->points->npoints == 1 ) {
 			/* Duplicate point, to make geos-friendly */
 			lwl->points = ptarray_addPoint(lwl->points,
@@ -341,9 +412,7 @@ LWGEOM2GEOS(const LWGEOM *lwgeom)
 		}
 		else
 		{
-			sq = ptarray_to_GEOSCoordSeq(lwpoly->rings[0]);
-			/* TODO: check ring for being closed and fix if not */
-			shell = GEOSGeom_createLinearRing(sq);
+			shell = ptarray_to_GEOSLinearRing(lwpoly->rings[0], autofix);
 			if ( ! shell ) return NULL;
 			/*lwerror("LWGEOM2GEOS: exception during polygon shell conversion"); */
 			ngeoms = lwpoly->nrings-1;
@@ -352,8 +421,7 @@ LWGEOM2GEOS(const LWGEOM *lwgeom)
 
 			for (i=1; i<lwpoly->nrings; ++i)
 			{
-				sq = ptarray_to_GEOSCoordSeq(lwpoly->rings[i]);
-				geoms[i-1] = GEOSGeom_createLinearRing(sq);
+				geoms[i-1] = ptarray_to_GEOSLinearRing(lwpoly->rings[i], autofix);
 				if ( ! geoms[i-1] )
 				{
 					--i;
@@ -390,7 +458,7 @@ LWGEOM2GEOS(const LWGEOM *lwgeom)
 
 		for (i=0; i<ngeoms; ++i)
 		{
-			GEOSGeometry* g = LWGEOM2GEOS(lwc->geoms[i]);
+			GEOSGeometry* g = LWGEOM2GEOS(lwc->geoms[i], 0);
 			if ( ! g )
 			{
 				while (i) GEOSGeom_destroy(geoms[--i]);
@@ -440,7 +508,7 @@ lwgeom_normalize(const LWGEOM *geom1)
 
 	initGEOS(lwnotice, lwgeom_geos_error);
 
-	g1 = LWGEOM2GEOS(geom1);
+	g1 = LWGEOM2GEOS(geom1, 0);
 	if ( 0 == g1 )   /* exception thrown at construction */
 	{
 		lwerror("First argument geometry could not be converted to GEOS: %s", lwgeom_geos_errmsg);
@@ -477,11 +545,11 @@ lwgeom_intersection(const LWGEOM *geom1, const LWGEOM *geom2)
 
 	/* A.Intersection(Empty) == Empty */
 	if ( lwgeom_is_empty(geom2) )
-		return lwgeom_clone(geom2);
+		return lwgeom_clone_deep(geom2);
 
 	/* Empty.Intersection(A) == Empty */
 	if ( lwgeom_is_empty(geom1) )
-		return lwgeom_clone(geom1);
+		return lwgeom_clone_deep(geom1);
 
 	/* ensure srids are identical */
 	srid = (int)(geom1->srid);
@@ -493,14 +561,14 @@ lwgeom_intersection(const LWGEOM *geom1, const LWGEOM *geom2)
 
 	LWDEBUG(3, "intersection() START");
 
-	g1 = LWGEOM2GEOS(geom1);
+	g1 = LWGEOM2GEOS(geom1, 0);
 	if ( 0 == g1 )   /* exception thrown at construction */
 	{
 		lwerror("First argument geometry could not be converted to GEOS: %s", lwgeom_geos_errmsg);
 		return NULL ;
 	}
 
-	g2 = LWGEOM2GEOS(geom2);
+	g2 = LWGEOM2GEOS(geom2, 0);
 	if ( 0 == g2 )   /* exception thrown at construction */
 	{
 		lwerror("Second argument geometry could not be converted to GEOS.");
@@ -522,8 +590,8 @@ lwgeom_intersection(const LWGEOM *geom1, const LWGEOM *geom2)
 	{
 		GEOSGeom_destroy(g1);
 		GEOSGeom_destroy(g2);
-	        lwerror("Error performing intersection: %s",
-	                lwgeom_geos_errmsg);
+		lwerror("Error performing intersection: %s",
+		        lwgeom_geos_errmsg);
 		return NULL; /* never get here */
 	}
 
@@ -538,8 +606,8 @@ lwgeom_intersection(const LWGEOM *geom1, const LWGEOM *geom2)
 		GEOSGeom_destroy(g1);
 		GEOSGeom_destroy(g2);
 		GEOSGeom_destroy(g3);
-	        lwerror("Error performing intersection: GEOS2LWGEOM: %s",
-	                lwgeom_geos_errmsg);
+		lwerror("Error performing intersection: GEOS2LWGEOM: %s",
+		        lwgeom_geos_errmsg);
 		return NULL ; /* never get here */
 	}
 
@@ -550,6 +618,119 @@ lwgeom_intersection(const LWGEOM *geom1, const LWGEOM *geom2)
 	return result ;
 }
 
+LWGEOM *
+lwgeom_linemerge(const LWGEOM *geom1)
+{
+	LWGEOM *result ;
+	GEOSGeometry *g1, *g3 ;
+	int is3d = FLAGS_GET_Z(geom1->flags);
+	int srid = geom1->srid;
+
+	/* Empty.Linemerge() == Empty */
+	if ( lwgeom_is_empty(geom1) )
+		return (LWGEOM*)lwcollection_construct_empty( COLLECTIONTYPE, srid, is3d,
+                                         lwgeom_has_m(geom1) );
+
+	initGEOS(lwnotice, lwgeom_geos_error);
+
+	LWDEBUG(3, "linemerge() START");
+
+	g1 = LWGEOM2GEOS(geom1, 0);
+	if ( 0 == g1 )   /* exception thrown at construction */
+	{
+		lwerror("First argument geometry could not be converted to GEOS: %s", lwgeom_geos_errmsg);
+		return NULL ;
+	}
+
+	LWDEBUG(3, " constructed geometrys - calling geos");
+	LWDEBUGF(3, " g1 = %s", GEOSGeomToWKT(g1));
+	/*LWDEBUGF(3, "g1 is valid = %i",GEOSisvalid(g1)); */
+
+	g3 = GEOSLineMerge(g1);
+
+	LWDEBUG(3, " linemerge finished");
+
+	if (g3 == NULL)
+	{
+		GEOSGeom_destroy(g1);
+		lwerror("Error performing linemerge: %s",
+		        lwgeom_geos_errmsg);
+		return NULL; /* never get here */
+	}
+
+	LWDEBUGF(3, "result: %s", GEOSGeomToWKT(g3) ) ;
+
+	GEOSSetSRID(g3, srid);
+
+	result = GEOS2LWGEOM(g3, is3d);
+
+	if (result == NULL)
+	{
+		GEOSGeom_destroy(g1);
+		GEOSGeom_destroy(g3);
+		lwerror("Error performing linemerge: GEOS2LWGEOM: %s",
+		        lwgeom_geos_errmsg);
+		return NULL ; /* never get here */
+	}
+
+	GEOSGeom_destroy(g1);
+	GEOSGeom_destroy(g3);
+
+	return result ;
+}
+
+LWGEOM *
+lwgeom_unaryunion(const LWGEOM *geom1)
+{
+	LWGEOM *result ;
+	GEOSGeometry *g1, *g3 ;
+	int is3d = FLAGS_GET_Z(geom1->flags);
+	int srid = geom1->srid;
+
+	/* Empty.UnaryUnion() == Empty */
+	if ( lwgeom_is_empty(geom1) )
+		return lwgeom_clone_deep(geom1);
+
+	initGEOS(lwnotice, lwgeom_geos_error);
+
+	g1 = LWGEOM2GEOS(geom1, 0);
+	if ( 0 == g1 )   /* exception thrown at construction */
+	{
+		lwerror("First argument geometry could not be converted to GEOS: %s", lwgeom_geos_errmsg);
+		return NULL ;
+	}
+
+	g3 = GEOSUnaryUnion(g1);
+
+	if (g3 == NULL)
+	{
+		GEOSGeom_destroy(g1);
+		lwerror("Error performing unaryunion: %s",
+		        lwgeom_geos_errmsg);
+		return NULL; /* never get here */
+	}
+
+	LWDEBUGF(3, "result: %s", GEOSGeomToWKT(g3) ) ;
+
+	GEOSSetSRID(g3, srid);
+
+	result = GEOS2LWGEOM(g3, is3d);
+
+	if (result == NULL)
+	{
+		GEOSGeom_destroy(g1);
+		GEOSGeom_destroy(g3);
+		lwerror("Error performing unaryunion: GEOS2LWGEOM: %s",
+		        lwgeom_geos_errmsg);
+		return NULL ; /* never get here */
+	}
+
+	GEOSGeom_destroy(g1);
+	GEOSGeom_destroy(g3);
+
+	return result ;
+}
+
 LWGEOM *
 lwgeom_difference(const LWGEOM *geom1, const LWGEOM *geom2)
 {
@@ -560,11 +741,11 @@ lwgeom_difference(const LWGEOM *geom1, const LWGEOM *geom2)
 
 	/* A.Difference(Empty) == A */
 	if ( lwgeom_is_empty(geom2) )
-		return lwgeom_clone(geom1);
+		return lwgeom_clone_deep(geom1);
 
 	/* Empty.Intersection(A) == Empty */
 	if ( lwgeom_is_empty(geom1) )
-		return lwgeom_clone(geom1);
+		return lwgeom_clone_deep(geom1);
 
 	/* ensure srids are identical */
 	srid = (int)(geom1->srid);
@@ -574,14 +755,14 @@ lwgeom_difference(const LWGEOM *geom1, const LWGEOM *geom2)
 
 	initGEOS(lwnotice, lwgeom_geos_error);
 
-	g1 = LWGEOM2GEOS(geom1);
+	g1 = LWGEOM2GEOS(geom1, 0);
 	if ( 0 == g1 )   /* exception thrown at construction */
 	{
 		lwerror("First argument geometry could not be converted to GEOS: %s", lwgeom_geos_errmsg);
 		return NULL;
 	}
 
-	g2 = LWGEOM2GEOS(geom2);
+	g2 = LWGEOM2GEOS(geom2, 0);
 	if ( 0 == g2 )   /* exception thrown at construction */
 	{
 		GEOSGeom_destroy(g1);
@@ -610,8 +791,8 @@ lwgeom_difference(const LWGEOM *geom1, const LWGEOM *geom2)
 		GEOSGeom_destroy(g1);
 		GEOSGeom_destroy(g2);
 		GEOSGeom_destroy(g3);
-	        lwerror("Error performing difference: GEOS2LWGEOM: %s",
-	                lwgeom_geos_errmsg);
+		lwerror("Error performing difference: GEOS2LWGEOM: %s",
+		        lwgeom_geos_errmsg);
 		return NULL; /* never get here */
 	}
 
@@ -634,11 +815,11 @@ lwgeom_symdifference(const LWGEOM* geom1, const LWGEOM* geom2)
 
 	/* A.SymDifference(Empty) == A */
 	if ( lwgeom_is_empty(geom2) )
-		return lwgeom_clone(geom1);
+		return lwgeom_clone_deep(geom1);
 
 	/* Empty.DymDifference(B) == B */
 	if ( lwgeom_is_empty(geom1) )
-		return lwgeom_clone(geom2);
+		return lwgeom_clone_deep(geom2);
 
 	/* ensure srids are identical */
 	srid = (int)(geom1->srid);
@@ -648,7 +829,7 @@ lwgeom_symdifference(const LWGEOM* geom1, const LWGEOM* geom2)
 
 	initGEOS(lwnotice, lwgeom_geos_error);
 
-	g1 = LWGEOM2GEOS(geom1);
+	g1 = LWGEOM2GEOS(geom1, 0);
 
 	if ( 0 == g1 )   /* exception thrown at construction */
 	{
@@ -656,7 +837,7 @@ lwgeom_symdifference(const LWGEOM* geom1, const LWGEOM* geom2)
 		return NULL;
 	}
 
-	g2 = LWGEOM2GEOS(geom2);
+	g2 = LWGEOM2GEOS(geom2, 0);
 
 	if ( 0 == g2 )   /* exception thrown at construction */
 	{
@@ -709,11 +890,11 @@ lwgeom_union(const LWGEOM *geom1, const LWGEOM *geom2)
 
 	/* A.Union(empty) == A */
 	if ( lwgeom_is_empty(geom1) )
-		return lwgeom_clone(geom2);
+		return lwgeom_clone_deep(geom2);
 
 	/* B.Union(empty) == B */
 	if ( lwgeom_is_empty(geom2) )
-		return lwgeom_clone(geom1);
+		return lwgeom_clone_deep(geom1);
 
 
 	/* ensure srids are identical */
@@ -724,7 +905,7 @@ lwgeom_union(const LWGEOM *geom1, const LWGEOM *geom2)
 
 	initGEOS(lwnotice, lwgeom_geos_error);
 
-	g1 = LWGEOM2GEOS(geom1);
+	g1 = LWGEOM2GEOS(geom1, 0);
 
 	if ( 0 == g1 )   /* exception thrown at construction */
 	{
@@ -732,7 +913,7 @@ lwgeom_union(const LWGEOM *geom1, const LWGEOM *geom2)
 		return NULL;
 	}
 
-	g2 = LWGEOM2GEOS(geom2);
+	g2 = LWGEOM2GEOS(geom2, 0);
 
 	if ( 0 == g2 )   /* exception thrown at construction */
 	{
@@ -766,14 +947,78 @@ lwgeom_union(const LWGEOM *geom1, const LWGEOM *geom2)
 
 	if (result == NULL)
 	{
-	        lwerror("Error performing union: GEOS2LWGEOM: %s",
-	                lwgeom_geos_errmsg);
+		lwerror("Error performing union: GEOS2LWGEOM: %s",
+		        lwgeom_geos_errmsg);
 		return NULL; /*never get here */
 	}
 
 	return result;
 }
 
+LWGEOM *
+lwgeom_clip_by_rect(const LWGEOM *geom1, double x0, double y0, double x1, double y1)
+{
+#if POSTGIS_GEOS_VERSION < 35
+	lwerror("The GEOS version this postgis binary "
+	        "was compiled against (%d) doesn't support "
+	        "'GEOSClipByRect' function (3.3.5+ required)",
+	        POSTGIS_GEOS_VERSION);
+	return NULL;
+#else /* POSTGIS_GEOS_VERSION >= 35 */
+	LWGEOM *result ;
+	GEOSGeometry *g1, *g3 ;
+	int is3d ;
+
+	/* A.Intersection(Empty) == Empty */
+	if ( lwgeom_is_empty(geom1) )
+		return lwgeom_clone_deep(geom1);
+
+	is3d = FLAGS_GET_Z(geom1->flags);
+
+	initGEOS(lwnotice, lwgeom_geos_error);
+
+	LWDEBUG(3, "clip_by_rect() START");
+
+	g1 = LWGEOM2GEOS(geom1, 1); /* auto-fix structure */
+	if ( 0 == g1 )   /* exception thrown at construction */
+	{
+		lwerror("First argument geometry could not be converted to GEOS: %s", lwgeom_geos_errmsg);
+		return NULL ;
+	}
+
+	LWDEBUG(3, " constructed geometrys - calling geos");
+	LWDEBUGF(3, " g1 = %s", GEOSGeomToWKT(g1));
+	/*LWDEBUGF(3, "g1 is valid = %i",GEOSisvalid(g1)); */
+
+	g3 = GEOSClipByRect(g1,x0,y0,x1,y1);
+	GEOSGeom_destroy(g1);
+
+	LWDEBUG(3, " clip_by_rect finished");
+
+	if (g3 == NULL)
+	{
+		lwnotice("Error performing rectangular clipping: %s", lwgeom_geos_errmsg);
+		return NULL;
+	}
+
+	LWDEBUGF(3, "result: %s", GEOSGeomToWKT(g3) ) ;
+
+	result = GEOS2LWGEOM(g3, is3d);
+	GEOSGeom_destroy(g3);
+
+	if (result == NULL)
+	{
+		lwerror("Error performing intersection: GEOS2LWGEOM: %s", lwgeom_geos_errmsg);
+		return NULL ; /* never get here */
+	}
+
+	result->srid = geom1->srid;
+
+	return result ;
+#endif /* POSTGIS_GEOS_VERSION >= 35 */
+}
+
+
 /* ------------ BuildArea stuff ---------------------------------------------------------------------{ */
 
 typedef struct Face_t {
@@ -1068,7 +1313,7 @@ lwgeom_buildarea(const LWGEOM *geom)
 
 	initGEOS(lwnotice, lwgeom_geos_error);
 
-	geos_in = LWGEOM2GEOS(geom);
+	geos_in = LWGEOM2GEOS(geom, 0);
 	
 	if ( 0 == geos_in )   /* exception thrown at construction */
 	{
@@ -1106,6 +1351,38 @@ lwgeom_buildarea(const LWGEOM *geom)
 	return geom_out;
 }
 
+int
+lwgeom_is_simple(const LWGEOM *geom)
+{
+	GEOSGeometry* geos_in;
+	int simple;
+
+	/* Empty is always simple */
+	if ( lwgeom_is_empty(geom) )
+	{
+		return 1;
+	}
+
+	initGEOS(lwnotice, lwgeom_geos_error);
+
+	geos_in = LWGEOM2GEOS(geom, 0);
+	if ( 0 == geos_in )   /* exception thrown at construction */
+	{
+		lwerror("First argument geometry could not be converted to GEOS: %s", lwgeom_geos_errmsg);
+		return -1;
+	}
+	simple = GEOSisSimple(geos_in);
+	GEOSGeom_destroy(geos_in);
+
+	if ( simple == 2 ) /* exception thrown */
+	{
+		lwerror("lwgeom_is_simple: %s", lwgeom_geos_errmsg);
+		return -1;
+	}
+
+	return simple ? 1 : 0;
+}
+
 /* ------------ end of BuildArea stuff ---------------------------------------------------------------------} */
 
 LWGEOM*
@@ -1117,7 +1394,7 @@ lwgeom_geos_noop(const LWGEOM* geom_in)
 	int is3d = FLAGS_GET_Z(geom_in->flags);
 
 	initGEOS(lwnotice, lwgeom_geos_error);
-	geosgeom = LWGEOM2GEOS(geom_in);
+	geosgeom = LWGEOM2GEOS(geom_in, 0);
 	if ( ! geosgeom ) {
 		lwerror("Geometry could not be converted to GEOS: %s",
 			lwgeom_geos_errmsg);
@@ -1155,14 +1432,14 @@ lwgeom_snap(const LWGEOM* geom1, const LWGEOM* geom2, double tolerance)
 
 	initGEOS(lwnotice, lwgeom_geos_error);
 
-	g1 = (GEOSGeometry *)LWGEOM2GEOS(geom1);
+	g1 = (GEOSGeometry *)LWGEOM2GEOS(geom1, 0);
 	if ( 0 == g1 )   /* exception thrown at construction */
 	{
 		lwerror("First argument geometry could not be converted to GEOS: %s", lwgeom_geos_errmsg);
 		return NULL;
 	}
 
-	g2 = (GEOSGeometry *)LWGEOM2GEOS(geom2);
+	g2 = (GEOSGeometry *)LWGEOM2GEOS(geom2, 0);
 	if ( 0 == g2 )   /* exception thrown at construction */
 	{
 		lwerror("Second argument geometry could not be converted to GEOS: %s", lwgeom_geos_errmsg);
@@ -1218,14 +1495,14 @@ lwgeom_sharedpaths(const LWGEOM* geom1, const LWGEOM* geom2)
 
 	initGEOS(lwnotice, lwgeom_geos_error);
 
-	g1 = (GEOSGeometry *)LWGEOM2GEOS(geom1);
+	g1 = (GEOSGeometry *)LWGEOM2GEOS(geom1, 0);
 	if ( 0 == g1 )   /* exception thrown at construction */
 	{
 		lwerror("First argument geometry could not be converted to GEOS: %s", lwgeom_geos_errmsg);
 		return NULL;
 	}
 
-	g2 = (GEOSGeometry *)LWGEOM2GEOS(geom2);
+	g2 = (GEOSGeometry *)LWGEOM2GEOS(geom2, 0);
 	if ( 0 == g2 )   /* exception thrown at construction */
 	{
 		lwerror("Second argument geometry could not be converted to GEOS: %s", lwgeom_geos_errmsg);
@@ -1270,7 +1547,7 @@ lwgeom_offsetcurve(const LWLINE *lwline, double size, int quadsegs, int joinStyl
 
 	initGEOS(lwnotice, lwgeom_geos_error);
 
-	g1 = (GEOSGeometry *)LWGEOM2GEOS(lwgeom_in);
+	g1 = (GEOSGeometry *)LWGEOM2GEOS(lwgeom_in, 0);
 	if ( ! g1 ) 
 	{
 		lwerror("lwgeom_offsetcurve: Geometry could not be converted to GEOS: %s", lwgeom_geos_errmsg);
@@ -1391,7 +1668,7 @@ LWGEOM* lwgeom_delaunay_triangulation(const LWGEOM *lwgeom_in, double tolerance,
 
 	initGEOS(lwnotice, lwgeom_geos_error);
 
-	g1 = (GEOSGeometry *)LWGEOM2GEOS(lwgeom_in);
+	g1 = (GEOSGeometry *)LWGEOM2GEOS(lwgeom_in, 0);
 	if ( ! g1 ) 
 	{
 		lwerror("lwgeom_delaunay_triangulation: Geometry could not be converted to GEOS: %s", lwgeom_geos_errmsg);
