@@ -1,8 +1,7 @@
 /**********************************************************************
- * $Id: lwgeom_geos.c 5258 2010-02-17 21:02:49Z strk $
  *
  * PostGIS - Spatial Types for PostgreSQL
- * http://postgis.refractions.net
+ * http://postgis.net
  *
  * Copyright 2009-2010 Sandro Santilli <strk@keybit.net>
  *
@@ -206,12 +205,12 @@ POINTARRAY*
 ring_make_geos_friendly(POINTARRAY* ring)
 {
 	POINTARRAY* closedring;
+	POINTARRAY* ring_in = ring;
 
 	/* close the ring if not already closed (2d only) */
 	closedring = ptarray_close2d(ring);
 	if (closedring != ring )
 	{
-		ptarray_free(ring); /* should we do this ? */
 		ring = closedring;
 	}
 
@@ -219,12 +218,14 @@ ring_make_geos_friendly(POINTARRAY* ring)
 
 	while ( ring->npoints < 4 )
 	{
+		POINTARRAY *oring = ring;
 		LWDEBUGF(4, "ring has %d points, adding another", ring->npoints);
 		/* let's add another... */
 		ring = ptarray_addPoint(ring,
 		                        getPoint_internal(ring, 0),
 		                        FLAGS_NDIMS(ring->flags),
 		                        ring->npoints);
+		if ( oring != ring_in ) ptarray_free(oring);
 	}
 
 
@@ -256,10 +257,7 @@ lwpoly_make_geos_friendly(LWPOLY *poly)
 		if ( ring_in != ring_out )
 		{
 			LWDEBUGF(3, "lwpoly_make_geos_friendly: ring %d cleaned, now has %d points", i, ring_out->npoints);
-			/* this may come right from
-			 * the binary representation lands
-			 */
-			/*ptarray_free(ring_in); */
+			ptarray_free(ring_in);
 		}
 		else
 		{
@@ -997,7 +995,7 @@ lwgeom_make_valid(LWGEOM* lwgeom_in)
 	int is3d;
 	GEOSGeom geosgeom;
 	GEOSGeometry* geosout;
-	LWGEOM *lwgeom_out, *lwgeom_tmp;
+	LWGEOM *lwgeom_out;
 
 	is3d = FLAGS_GET_Z(lwgeom_in->flags);
 
@@ -1009,7 +1007,7 @@ lwgeom_make_valid(LWGEOM* lwgeom_in)
 	initGEOS(lwgeom_geos_error, lwgeom_geos_error);
 
 	lwgeom_out = lwgeom_in;
-	geosgeom = LWGEOM2GEOS(lwgeom_out);
+	geosgeom = LWGEOM2GEOS(lwgeom_out, 0);
 	if ( ! geosgeom )
 	{
 		LWDEBUGF(4,
@@ -1025,7 +1023,8 @@ lwgeom_make_valid(LWGEOM* lwgeom_in)
 		}
 
 		/* try again as we did cleanup now */
-		geosgeom = LWGEOM2GEOS(lwgeom_out);
+		/* TODO: invoke LWGEOM2GEOS directly with autoclean ? */
+		geosgeom = LWGEOM2GEOS(lwgeom_out, 0);
 		if ( ! geosgeom )
 		{
 			lwerror("Couldn't convert POSTGIS geom to GEOS: %s",
@@ -1051,12 +1050,19 @@ lwgeom_make_valid(LWGEOM* lwgeom_in)
 	GEOSGeom_destroy(geosout);
 
 	if ( lwgeom_is_collection(lwgeom_in) && ! lwgeom_is_collection(lwgeom_out) )
-	{
+	{{
+		LWGEOM **ogeoms = lwalloc(sizeof(LWGEOM*));
+		LWGEOM *ogeom;
 		LWDEBUG(3, "lwgeom_make_valid: forcing multi");
-		lwgeom_tmp = lwgeom_as_multi(lwgeom_out);
-		lwfree(lwgeom_out); /* note: only frees the wrapper, not the content */
-		lwgeom_out = lwgeom_tmp;
-	}
+		/* NOTE: this is safe because lwgeom_out is surely not lwgeom_in or
+		 * otherwise we couldn't have a collection and a non-collection */
+		assert(lwgeom_in != lwgeom_out);
+		ogeoms[0] = lwgeom_out;
+		ogeom = (LWGEOM *)lwcollection_construct(MULTITYPE[lwgeom_out->type],
+		                          lwgeom_out->srid, lwgeom_out->bbox, 1, ogeoms);
+		lwgeom_out->bbox = NULL;
+		lwgeom_out = ogeom;
+	}}
 
 	lwgeom_out->srid = lwgeom_in->srid;
 	return lwgeom_out;
