@@ -35,6 +35,8 @@
  *
  **********************************************************************/
 
+#include "../postgis_config.h"
+/*#define POSTGIS_DEBUG_LEVEL 4*/
 #include "lwgeom_geos.h"
 #include "liblwgeom_internal.h"
 
@@ -220,11 +222,13 @@ int
 lwline_split_by_point_to(const LWLINE* lwline_in, const LWPOINT* blade_in,
                          LWMLINE* v)
 {
-	double loc, dist;
+	double mindist = -1;
 	POINT4D pt, pt_projected;
+	POINT4D p1, p2;
+	POINTARRAY *ipa = lwline_in->points;
 	POINTARRAY* pa1;
 	POINTARRAY* pa2;
-	double vstol; /* vertex snap tolerance */
+	int i, nsegs, seg = -1;
 
 	/* Possible outcomes:
 	 *
@@ -240,30 +244,69 @@ lwline_split_by_point_to(const LWLINE* lwline_in, const LWPOINT* blade_in,
 	 */
 
 	getPoint4d_p(blade_in->point, 0, &pt);
-	loc = ptarray_locate_point(lwline_in->points, &pt, &dist, &pt_projected);
 
-	/* lwnotice("Location: %g -- Distance: %g", loc, dist); */
-
-	if ( dist > 0 )   /* TODO: accept a tolerance ? */
+	/* Find closest segment */
+	getPoint4d_p(ipa, 0, &p1);
+	nsegs = ipa->npoints - 1;
+	for ( i = 0; i < nsegs; i++ )
 	{
-		/* No intersection */
-		return 0;
+		getPoint4d_p(ipa, i+1, &p2);
+		double dist;
+		dist = distance2d_pt_seg((POINT2D*)&pt, (POINT2D*)&p1, (POINT2D*)&p2);
+		LWDEBUGF(4, " Distance of point %g %g to segment %g %g, %g %g: %g", pt.x, pt.y, p1.x, p1.y, p2.x, p2.y, dist);
+		if (i==0 || dist < mindist )
+		{
+			mindist = dist;
+			seg=i;
+		}
+		p1 = p2;
 	}
 
-	if ( loc == 0 || loc == 1 )
+	LWDEBUGF(3, "Closest segment: %d", seg);
+	LWDEBUGF(3, "mindist: %g", mindist);
+
+	/* No intersection */
+	if ( mindist > 0 ) return 0;
+
+	/* empty or single-point line, intersection on boundary */
+	if ( seg < 0 ) return 1;
+
+	/*
+	 * We need to project the
+	 * point on the closest segment.
+	 */
+	getPoint4d_p(ipa, seg, &p1);
+	getPoint4d_p(ipa, seg+1, &p2);
+	closest_point_on_segment(&pt, &p1, &p2, &pt_projected);
+
+	LWDEBUGF(3, "Projected point:(%g %g), seg:%d, p1:(%g %g), p2:(%g %g)", pt_projected.x, pt_projected.y, seg, p1.x, p1.y, p2.x, p2.y);
+
+	/* When closest point == an endpoint, this is a boundary intersection */
+	if ( ( (seg == nsegs-1) && p4d_same(&pt_projected, &p2) ) ||
+	     ( (seg == 0)       && p4d_same(&pt_projected, &p1) ) )
 	{
-		/* Intersection is on the boundary */
 		return 1;
 	}
 
-	/* There is a real intersection, let's get two substrings */
+	/* This is an internal intersection, let's build the two new pointarrays */
 
-	/* Compute vertex snap tolerance based on line length
-	 * TODO: take as parameter ? */
-	vstol = ptarray_length_2d(lwline_in->points) / 1e14;
+	pa1 = ptarray_construct_empty(FLAGS_GET_Z(ipa->flags), FLAGS_GET_M(ipa->flags), seg+2);
+	/* TODO: replace with a memcpy ? */
+	for (i=0; i<=seg; ++i)
+	{
+		getPoint4d_p(ipa, i, &p1);
+		ptarray_append_point(pa1, &p1, LW_FALSE);
+	}
+	ptarray_append_point(pa1, &pt_projected, LW_FALSE);
 
-	pa1 = ptarray_substring(lwline_in->points, 0, loc, vstol);
-	pa2 = ptarray_substring(lwline_in->points, loc, 1, vstol);
+	pa2 = ptarray_construct_empty(FLAGS_GET_Z(ipa->flags), FLAGS_GET_M(ipa->flags), ipa->npoints-seg);
+	ptarray_append_point(pa2, &pt_projected, LW_FALSE);
+	/* TODO: replace with a memcpy (if so need to check for duplicated point) ? */
+	for (i=seg+1; i<ipa->npoints; ++i)
+	{
+		getPoint4d_p(ipa, i, &p1);
+		ptarray_append_point(pa2, &p1, LW_FALSE);
+	}
 
 	/* NOTE: I've seen empty pointarrays with loc != 0 and loc != 1 */
 	if ( pa1->npoints == 0 || pa2->npoints == 0 ) {
