@@ -13,7 +13,7 @@
  **********************************************************************/
 
 
-#include "liblwgeom.h"
+#include "liblwgeom_internal.h"
 #include <SFCGAL/capi/sfcgal_c.h>
 
 
