@@ -9,8 +9,11 @@
  *
  **********************************************************************/
 
-#include "liblwgeom_internal.h"
+#include "../postgis_config.h"
+/*#define POSTGIS_DEBUG_LEVEL 4*/
+#include "liblwgeom_internal.h" /* NOTE: includes lwgeom_log.h */
 #include "lwgeom_log.h"
+#include <math.h>
 
 /**
 * Used for passing the parse state between the parsing functions.
@@ -318,6 +321,9 @@ static POINTARRAY* ptarray_from_wkb_state(wkb_parse_state *s)
 
 	/* Calculate the size of this point array. */
 	npoints = integer_from_wkb_state(s);
+
+	LWDEBUGF(4,"Pointarray has %d points", npoints);
+
 	if( s->has_z ) ndims++;
 	if( s->has_m ) ndims++;
 	pa_size = npoints * ndims * WKB_DOUBLE_SIZE;
@@ -366,6 +372,7 @@ static LWPOINT* lwpoint_from_wkb_state(wkb_parse_state *s)
 	POINTARRAY *pa = NULL;
 	size_t pa_size;
 	uint32_t ndims = 2;
+	const POINT2D *pt;
 
 	/* Count the dimensions. */
 	if( s->has_z ) ndims++;
@@ -394,7 +401,17 @@ static LWPOINT* lwpoint_from_wkb_state(wkb_parse_state *s)
 		}
 	}
 	
-	return lwpoint_construct(s->srid, NULL, pa);
+	/* Check for POINT(NaN NaN) ==> POINT EMPTY */
+	pt = getPoint2d_cp(pa, 0);
+	if ( isnan(pt->x) && isnan(pt->y) )
+	{
+		ptarray_free(pa);
+		return lwpoint_construct_empty(s->srid, s->has_z, s->has_m);
+	}
+	else
+	{
+		return lwpoint_construct(s->srid, NULL, pa);
+	}
 }
 
 /**
@@ -465,6 +482,8 @@ static LWPOLY* lwpoly_from_wkb_state(wkb_parse_state *s)
 	uint32_t nrings = integer_from_wkb_state(s);
 	int i = 0;
 	LWPOLY *poly = lwpoly_construct_empty(s->srid, s->has_z, s->has_m);
+
+	LWDEBUGF(4,"Polygon has %d rings", nrings);
 	
 	/* Empty polygon? */
 	if( nrings == 0 )
@@ -552,6 +571,10 @@ static LWTRIANGLE* lwtriangle_from_wkb_state(wkb_parse_state *s)
 		return NULL;
 	}
 
+	/* Empty TRIANGLE starts w/ empty POINTARRAY, free it first */
+	if (tri->points)
+		ptarray_free(tri->points);
+	
 	tri->points = pa;	
 	return tri;
 }
@@ -595,6 +618,8 @@ static LWCOLLECTION* lwcollection_from_wkb_state(wkb_parse_state *s)
 	LWCOLLECTION *col = lwcollection_construct_empty(s->lwtype, s->srid, s->has_z, s->has_m);
 	LWGEOM *geom = NULL;
 	int i;
+
+	LWDEBUGF(4,"Collection has %d components", ngeoms);
 	
 	/* Empty collection? */
 	if ( ngeoms == 0 )
