@@ -133,8 +133,9 @@ lwline_reverse(LWLINE *line)
 LWLINE *
 lwline_segmentize2d(LWLINE *line, double dist)
 {
-	return lwline_construct(line->srid, NULL,
-	                        ptarray_segmentize2d(line->points, dist));
+	POINTARRAY *segmentized = ptarray_segmentize2d(line->points, dist);
+	if ( ! segmentized ) return NULL;
+	return lwline_construct(line->srid, NULL, segmentized);
 }
 
 /* check coordinate equality  */
@@ -257,7 +258,7 @@ lwline_from_ptarray(int srid, uint32_t npoints, LWPOINT **points)
  * Construct a LWLINE from a LWMPOINT
  */
 LWLINE *
-lwline_from_lwmpoint(int srid, LWMPOINT *mpoint)
+lwline_from_lwmpoint(int srid, const LWMPOINT *mpoint)
 {
 	uint32_t i;
 	POINTARRAY *pa = NULL;
@@ -291,7 +292,7 @@ lwline_from_lwmpoint(int srid, LWMPOINT *mpoint)
 * Returns NULL if the geometry is empty or the index invalid.
 */
 LWPOINT*
-lwline_get_lwpoint(LWLINE *line, int where)
+lwline_get_lwpoint(const LWLINE *line, int where)
 {
 	POINT4D pt;
 	LWPOINT *lwpoint;
@@ -406,6 +407,9 @@ lwline_measured_from_lwline(const LWLINE *lwline, double m_start, double m_end)
 		length_so_far += distance2d_pt_pt(&a, &b);
 		if ( length > 0.0 )
 			m = m_start + m_range * length_so_far / length;
+		/* #3172, support (valid) zero-length inputs */
+		else if ( length == 0.0 && npoints > 1 )
+			m = m_start + m_range * i / (npoints-1);
 		else
 			m = 0.0;
 		q.x = p2.x;
@@ -420,11 +424,11 @@ lwline_measured_from_lwline(const LWLINE *lwline, double m_start, double m_end)
 }
 
 LWGEOM*
-lwline_remove_repeated_points(LWLINE *lwline)
+lwline_remove_repeated_points(LWLINE *lwline, double tolerance)
 {
-	POINTARRAY* npts = ptarray_remove_repeated_points(lwline->points);
+	POINTARRAY* npts = ptarray_remove_repeated_points_minpoints(lwline->points, tolerance, 2);
 
-	LWDEBUGF(3, "lwline_remove_repeated_points: npts %p", npts);
+	LWDEBUGF(3, "%s: npts %p", __func__, npts);
 
 	return (LWGEOM*)lwline_construct(lwline->srid,
 	                                 lwline->bbox ? gbox_copy(lwline->bbox) : 0,
@@ -440,6 +444,34 @@ lwline_is_closed(const LWLINE *line)
 	return ptarray_is_closed_2d(line->points);
 }
 
+int
+lwline_is_trajectory(const LWLINE *line)
+{
+  POINT3DM p;
+  int i, n;
+  double m = -1 * FLT_MAX;
+
+  if ( ! FLAGS_GET_M(line->flags) ) {
+    lwnotice("Line does not have M dimension");
+    return LW_FALSE;
+  }
+
+  n = line->points->npoints;
+  if ( n < 2 ) return LW_TRUE; /* empty or single-point are "good" */
+
+  for (i=0; i<n; ++i) {
+    getPoint3dm_p(line->points, i, &p);
+    if ( p.m <= m ) {
+      lwnotice("Measure of vertex %d (%g) not bigger than measure of vertex %d (%g)",
+        i, p.m, i-1, m);
+      return LW_FALSE;
+    }
+    m = p.m;
+  }
+
+  return LW_TRUE;
+}
+
 
 LWLINE*
 lwline_force_dims(const LWLINE *line, int hasz, int hasm)
@@ -477,18 +509,40 @@ int lwline_count_vertices(LWLINE *line)
 	return line->points->npoints;
 }
 
-LWLINE* lwline_simplify(const LWLINE *iline, double dist)
+LWLINE* lwline_simplify(const LWLINE *iline, double dist, int preserve_collapsed)
 {
+	static const int minvertices = 2; /* TODO: allow setting this */
 	LWLINE *oline;
+	POINTARRAY *pa;
 
 	LWDEBUG(2, "function called");
 
 	/* Skip empty case */
 	if( lwline_is_empty(iline) )
-		return lwline_clone(iline);
-		
-	static const int minvertices = 0; /* TODO: allow setting this */
-	oline = lwline_construct(iline->srid, NULL, ptarray_simplify(iline->points, dist, minvertices));
+		return NULL;
+
+	pa = ptarray_simplify(iline->points, dist, minvertices);
+	if ( ! pa ) return NULL;
+
+	/* Make sure single-point collapses have two points */
+	if ( pa->npoints == 1 )
+	{
+		/* Make sure single-point collapses have two points */
+		if ( preserve_collapsed )
+		{
+			POINT4D pt;
+			getPoint4d_p(pa, 0, &pt);		
+			ptarray_append_point(pa, &pt, LW_TRUE);
+		}
+		/* Return null for collapse */
+		else 
+		{
+			ptarray_free(pa);
+			return NULL;
+		}
+	}
+
+	oline = lwline_construct(iline->srid, NULL, pa);
 	oline->type = iline->type;
 	return oline;
 }
@@ -506,3 +560,22 @@ double lwline_length_2d(const LWLINE *line)
 		return 0.0;
 	return ptarray_length_2d(line->points);
 }
+
+
+
+LWLINE* lwline_grid(const LWLINE *line, const gridspec *grid)
+{
+	LWLINE *oline;
+	POINTARRAY *opa;
+
+	opa = ptarray_grid(line->points, grid);
+
+	/* Skip line3d with less then 2 points */
+	if ( opa->npoints < 2 ) return NULL;
+
+	/* TODO: grid bounding box... */
+	oline = lwline_construct(line->srid, NULL, opa);
+
+	return oline;
+}
+
