@@ -1,10 +1,9 @@
 /**********************************************************************
- * $Id: lwout_x3d.c 11511 2013-06-03 08:26:51Z strk $
  *
  * PostGIS - Spatial Types for PostgreSQL
- * http://www.postgis.org
+ * http://postgis.net
  * adapted from lwout_asgml.c
- * Copyright 2011 Arrival 3D
+ * Copyright 2011-2015 Arrival 3D
  * 				Regina Obe with input from Dave Arendash
  *
  * This is free software; you can redistribute and/or modify it under
@@ -28,7 +27,7 @@ static char *asx3d3_line(const LWLINE *line, char *srs, int precision, int opts,
 static size_t asx3d3_poly_size(const LWPOLY *poly, char *srs, int precision, int opts, const char *defid);
 static size_t asx3d3_triangle_size(const LWTRIANGLE *triangle, char *srs, int precision, int opts, const char *defid);
 static char *asx3d3_triangle(const LWTRIANGLE *triangle, char *srs, int precision, int opts, const char *defid);
-static size_t asx3d3_multi_size(const LWCOLLECTION *col, char *srs, int precision, int opts, const char *defid);
+static size_t asx3d3_multi_size(const LWCOLLECTION *col, char *srs, int precisioSn, int opts, const char *defid);
 static char *asx3d3_multi(const LWCOLLECTION *col, char *srs, int precision, int opts, const char *defid);
 static char *asx3d3_psurface(const LWPSURFACE *psur, char *srs, int precision, int opts, const char *defid);
 static char *asx3d3_tin(const LWTIN *tin, char *srs, int precision, int opts, const char *defid);
@@ -145,10 +144,17 @@ asx3d3_line_size(const LWLINE *line, char *srs, int precision, int opts, const c
 	size_t defidlen = strlen(defid);
 
 	size = pointArray_X3Dsize(line->points, precision)*2;
-
-	size += (
-	            sizeof("<LineSet vertexCount=''><Coordinate point='' /></LineSet>")  + defidlen
+	
+	if ( X3D_USE_GEOCOORDS(opts) ) {
+			size += (
+	            sizeof("<LineSet vertexCount=''><GeoCoordinate geoSystem='\"GD\" \"WE\" \"longitude_first\"' point='' /></LineSet>")  + defidlen
 	        ) * 2;
+	}
+	else {
+		size += (
+		            sizeof("<LineSet vertexCount=''><Coordinate point='' /></LineSet>")  + defidlen
+		        ) * 2;
+	}
 
 	/* if (srs)     size += strlen(srs) + sizeof(" srsName=.."); */
 	return size;
@@ -167,8 +173,9 @@ asx3d3_line_buf(const LWLINE *line, char *srs, char *output, int precision, int
 	pa = line->points;
 	ptr += sprintf(ptr, "<LineSet %s vertexCount='%d'>", defid, pa->npoints);
 
-
-	ptr += sprintf(ptr, "<Coordinate point='");
+	if ( X3D_USE_GEOCOORDS(opts) ) ptr += sprintf(ptr, "<GeoCoordinate geoSystem='\"GD\" \"WE\" \"%s\"' point='", ( (opts & LW_X3D_FLIP_XY) ? "latitude_first" : "longitude_first") );
+	else
+		ptr += sprintf(ptr, "<Coordinate point='");
 	ptr += pointArray_toX3D3(line->points, ptr, precision, opts, lwline_is_closed((LWLINE *) line));
 
 	ptr += sprintf(ptr, "' />");
@@ -371,7 +378,11 @@ asx3d3_multi_size(const LWCOLLECTION *col, char *srs, int precision, int opts, c
 	LWGEOM *subgeom;
 
 	/* the longest possible multi version needs to hold DEF=defid and coordinate breakout */
-	size = sizeof("<PointSet><Coordinate point='' /></PointSet>") + defidlen;
+	if ( X3D_USE_GEOCOORDS(opts) )
+		size = sizeof("<PointSet><GeoCoordinate geoSystem='\"GD\" \"WE\" \"longitude_first\"' point='' /></PointSet>");
+	else
+		size = sizeof("<PointSet><Coordinate point='' /></PointSet>") + defidlen;
+	
 
 	/* if ( srs ) size += strlen(srs) + sizeof(" srsName=.."); */
 
@@ -443,7 +454,10 @@ asx3d3_multi_buf(const LWCOLLECTION *col, char *srs, char *output, int precision
             return 0;
     }
     if (dimension == 3){
-        ptr += sprintf(ptr, "<Coordinate point='");
+		if ( X3D_USE_GEOCOORDS(opts) ) 
+			ptr += sprintf(ptr, "<GeoCoordinate geoSystem='\"GD\" \"WE\" \"%s\"' point='", ((opts & LW_X3D_FLIP_XY) ? "latitude_first" : "longitude_first") );
+		else
+        	ptr += sprintf(ptr, "<Coordinate point='");
     }
 
 	for (i=0; i<col->ngeoms; i++)
@@ -497,7 +511,9 @@ asx3d3_psurface_size(const LWPSURFACE *psur, char *srs, int precision, int opts,
 	size_t size;
 	size_t defidlen = strlen(defid);
 
-	size = sizeof("<IndexedFaceSet coordIndex=''><Coordinate point='' />") + defidlen;
+	if ( X3D_USE_GEOCOORDS(opts) ) size = sizeof("<IndexedFaceSet coordIndex=''><GeoCoordinate geoSystem='\"GD\" \"WE\" \"longitude_first\"' point='' />") + defidlen;
+	else size = sizeof("<IndexedFaceSet coordIndex=''><Coordinate point='' />") + defidlen;
+	
 
 	for (i=0; i<psur->ngeoms; i++)
 	{
@@ -546,7 +562,9 @@ asx3d3_psurface_buf(const LWPSURFACE *psur, char *srs, char *output, int precisi
 		j += k;
 	}
 
-	ptr += sprintf(ptr, "'><Coordinate point='");
+	if ( X3D_USE_GEOCOORDS(opts) ) 
+		ptr += sprintf(ptr, "'><GeoCoordinate geoSystem='\"GD\" \"WE\" \"%s\"' point='", ( (opts & LW_X3D_FLIP_XY) ? "latitude_first" : "longitude_first") );
+	else ptr += sprintf(ptr, "'><Coordinate point='");
 
 	for (i=0; i<psur->ngeoms; i++)
 	{
@@ -626,7 +644,9 @@ asx3d3_tin_buf(const LWTIN *tin, char *srs, char *output, int precision, int opt
 		k += 3;
 	}
 
-	ptr += sprintf(ptr, "'><Coordinate point='");
+	if ( X3D_USE_GEOCOORDS(opts) ) ptr += sprintf(ptr, "'><GeoCoordinate geoSystem='\"GD\" \"WE\" \"%s\"' point='", ( (opts & LW_X3D_FLIP_XY) ? "latitude_first" : "longitude_first") );
+	else ptr += sprintf(ptr, "'><Coordinate point='");
+	
 	for (i=0; i<tin->ngeoms; i++)
 	{
 		ptr += asx3d3_triangle_buf(tin->geoms[i], 0, ptr, precision,
@@ -667,15 +687,16 @@ asx3d3_collection_size(const LWCOLLECTION *col, char *srs, int precision, int op
 	size_t defidlen = strlen(defid);
 	LWGEOM *subgeom;
 
-	size = sizeof("<MultiGeometry></MultiGeometry>") + defidlen*2;
+	/* size = sizeof("<MultiGeometry></MultiGeometry>") + defidlen*2; */
+	size = defidlen*2;
 
-	if ( srs )
-		size += strlen(srs) + sizeof(" srsName=..");
+	/** if ( srs )
+		size += strlen(srs) + sizeof(" srsName=.."); **/
 
 	for (i=0; i<col->ngeoms; i++)
 	{
 		subgeom = col->geoms[i];
-		size += ( sizeof("<geometryMember>/") + defidlen ) * 2;
+		size += ( sizeof("<Shape />") + defidlen ) * 2; /** for collections we need to wrap each in a shape tag to make valid **/
 		if ( subgeom->type == POINTTYPE )
 		{
 			size += asx3d3_point_size((LWPOINT*)subgeom, 0, precision, opts, defid);
@@ -688,6 +709,14 @@ asx3d3_collection_size(const LWCOLLECTION *col, char *srs, int precision, int op
 		{
 			size += asx3d3_poly_size((LWPOLY*)subgeom, 0, precision, opts, defid);
 		}
+		else if ( subgeom->type == TINTYPE )
+		{
+			size += asx3d3_tin_size((LWTIN*)subgeom, 0, precision, opts, defid);
+		}
+		else if ( subgeom->type == POLYHEDRALSURFACETYPE )
+		{
+			size += asx3d3_psurface_size((LWPSURFACE*)subgeom, 0, precision, opts, defid);
+		}
 		else if ( lwgeom_is_collection(subgeom) )
 		{
 			size += asx3d3_multi_size((LWCOLLECTION*)subgeom, 0, precision, opts, defid);
@@ -709,6 +738,8 @@ asx3d3_collection_buf(const LWCOLLECTION *col, char *srs, char *output, int prec
 	ptr = output;
 
 	/* Open outmost tag */
+	/** @TODO: decide if we need outtermost tags, this one was just a copy from gml so is wrong **/
+#ifdef PGIS_X3D_OUTERMOST_TAGS
 	if ( srs )
 	{
 		ptr += sprintf(ptr, "<%sMultiGeometry srsName=\"%s\">", defid, srs);
@@ -717,11 +748,12 @@ asx3d3_collection_buf(const LWCOLLECTION *col, char *srs, char *output, int prec
 	{
 		ptr += sprintf(ptr, "<%sMultiGeometry>", defid);
 	}
+#endif
 
 	for (i=0; i<col->ngeoms; i++)
 	{
 		subgeom = col->geoms[i];
-		ptr += sprintf(ptr, "<%sgeometryMember>", defid);
+		ptr += sprintf(ptr, "<Shape%s>", defid);
 		if ( subgeom->type == POINTTYPE )
 		{
 			ptr += asx3d3_point_buf((LWPOINT*)subgeom, 0, ptr, precision, opts, defid);
@@ -734,6 +766,16 @@ asx3d3_collection_buf(const LWCOLLECTION *col, char *srs, char *output, int prec
 		{
 			ptr += asx3d3_poly_buf((LWPOLY*)subgeom, 0, ptr, precision, opts, 0, defid);
 		}
+		else if ( subgeom->type == TINTYPE )
+		{
+			ptr += asx3d3_tin_buf((LWTIN*)subgeom, srs, ptr, precision, opts,  defid);
+			
+		}
+		else if ( subgeom->type == POLYHEDRALSURFACETYPE )
+		{
+			ptr += asx3d3_psurface_buf((LWPSURFACE*)subgeom, srs, ptr, precision, opts,  defid);
+			
+		}
 		else if ( lwgeom_is_collection(subgeom) )
 		{
 			if ( subgeom->type == COLLECTIONTYPE )
@@ -744,11 +786,13 @@ asx3d3_collection_buf(const LWCOLLECTION *col, char *srs, char *output, int prec
 		else
 			lwerror("asx3d3_collection_buf: unknown geometry type");
 
-		ptr += sprintf(ptr, "</%sgeometryMember>", defid);
+		ptr += printf(ptr, "</Shape>");
 	}
 
 	/* Close outmost tag */
+#ifdef PGIS_X3D_OUTERMOST_TAGS
 	ptr += sprintf(ptr, "</%sMultiGeometry>", defid);
+#endif
 
 	return (ptr-output);
 }
@@ -806,7 +850,11 @@ pointArray_toX3D3(POINTARRAY *pa, char *output, int precision, int opts, int is_
 
 				if ( i )
 					ptr += sprintf(ptr, " ");
-				ptr += sprintf(ptr, "%s %s", x, y);
+					
+				if ( ( opts & LW_X3D_FLIP_XY) )
+					ptr += sprintf(ptr, "%s %s", y, x);
+				else
+					ptr += sprintf(ptr, "%s %s", x, y);
 			}
 		}
 	}
@@ -841,7 +889,10 @@ pointArray_toX3D3(POINTARRAY *pa, char *output, int precision, int opts, int is_
 				if ( i )
 					ptr += sprintf(ptr, " ");
 
-				ptr += sprintf(ptr, "%s %s %s", x, y, z);
+				if ( ( opts & LW_X3D_FLIP_XY) )
+					ptr += sprintf(ptr, "%s %s %s", y, x, z);
+				else
+					ptr += sprintf(ptr, "%s %s %s", x, y, z);
 			}
 		}
 	}
