@@ -216,6 +216,11 @@ lwpoly_segmentize2d(LWPOLY *poly, double dist)
 	for (i=0; i<poly->nrings; i++)
 	{
 		newrings[i] = ptarray_segmentize2d(poly->rings[i], dist);
+		if ( ! newrings[i] ) {
+			while (i--) ptarray_free(newrings[i]);
+			lwfree(newrings);
+			return NULL;
+		}
 	}
 	return lwpoly_construct(poly->srid, NULL,
 	                        poly->nrings, newrings);
@@ -281,7 +286,7 @@ lwpoly_from_lwlines(const LWLINE *shell,
 }
 
 LWGEOM*
-lwpoly_remove_repeated_points(LWPOLY *poly)
+lwpoly_remove_repeated_points(LWPOLY *poly, double tolerance)
 {
 	uint32_t i;
 	POINTARRAY **newrings;
@@ -289,7 +294,7 @@ lwpoly_remove_repeated_points(LWPOLY *poly)
 	newrings = lwalloc(sizeof(POINTARRAY *)*poly->nrings);
 	for (i=0; i<poly->nrings; i++)
 	{
-		newrings[i] = ptarray_remove_repeated_points(poly->rings[i]);
+		newrings[i] = ptarray_remove_repeated_points_minpoints(poly->rings[i], tolerance, 4);
 	}
 
 	return (LWGEOM*)lwpoly_construct(poly->srid,
@@ -343,20 +348,30 @@ int lwpoly_count_vertices(LWPOLY *poly)
 	return v;
 }
 
-LWPOLY* lwpoly_simplify(const LWPOLY *ipoly, double dist)
+LWPOLY* lwpoly_simplify(const LWPOLY *ipoly, double dist, int preserve_collapsed)
 {
 	int i;
 	LWPOLY *opoly = lwpoly_construct_empty(ipoly->srid, FLAGS_GET_Z(ipoly->flags), FLAGS_GET_M(ipoly->flags));
 
-	LWDEBUGF(2, "simplify_polygon3d: simplifying polygon with %d rings", ipoly->nrings);
+	LWDEBUGF(2, "%s: simplifying polygon with %d rings", __func__, ipoly->nrings);
 
-	if( lwpoly_is_empty(ipoly) )
-		return opoly; /* should we return NULL instead ? */
-
-	for (i = 0; i < ipoly->nrings; i++)
+	if ( lwpoly_is_empty(ipoly) )
 	{
-		static const int minvertices = 0; /* TODO: allow setting this */
-		POINTARRAY *opts = ptarray_simplify(ipoly->rings[i], dist, minvertices);
+		lwpoly_free(opoly);
+		return NULL;
+	}
+
+	for ( i = 0; i < ipoly->nrings; i++ )
+	{
+		POINTARRAY *opts;
+		int minvertices = 0;
+
+		/* We'll still let holes collapse, but if we're preserving */
+		/* and this is a shell, we ensure it is kept */
+		if ( preserve_collapsed && i == 0 )
+			minvertices = 4; 
+			
+		opts = ptarray_simplify(ipoly->rings[i], dist, minvertices);
 
 		LWDEBUGF(3, "ring%d simplified from %d to %d points", i, ipoly->rings[i]->npoints, opts->npoints);
 
@@ -371,14 +386,20 @@ LWPOLY* lwpoly_simplify(const LWPOLY *ipoly, double dist)
 
 		/* Add ring to simplified polygon */
 		if( lwpoly_add_ring(opoly, opts) == LW_FAILURE )
+		{
+			lwpoly_free(opoly);
 			return NULL;
+		}
 	}
 
 	LWDEBUGF(3, "simplified polygon with %d rings", ipoly->nrings);
 	opoly->type = ipoly->type;
 
 	if( lwpoly_is_empty(opoly) )
+	{
+		lwpoly_free(opoly);
 		return NULL;
+	}
 
 	return opoly;
 }
@@ -484,3 +505,78 @@ lwpoly_startpoint(const LWPOLY* poly, POINT4D* pt)
 	return ptarray_startpoint(poly->rings[0], pt);
 }
 
+int
+lwpoly_contains_point(const LWPOLY *poly, const POINT2D *pt)
+{
+	int i;
+	
+	if ( lwpoly_is_empty(poly) )
+		return LW_FALSE;
+	
+	if ( ptarray_contains_point(poly->rings[0], pt) == LW_OUTSIDE )
+		return LW_FALSE;
+	
+	for ( i = 1; i < poly->nrings; i++ )
+	{
+		if ( ptarray_contains_point(poly->rings[i], pt) == LW_INSIDE )
+			return LW_FALSE;
+	}
+	return LW_TRUE;
+}
+
+
+
+LWPOLY* lwpoly_grid(const LWPOLY *poly, const gridspec *grid)
+{
+	LWPOLY *opoly;
+	int ri;
+
+#if 0
+	/*
+	 * TODO: control this assertion
+	 * it is assumed that, since the grid size will be a pixel,
+	 * a visible ring should show at least a white pixel inside,
+	 * thus, for a square, that would be grid_xsize*grid_ysize
+	 */
+	double minvisiblearea = grid->xsize * grid->ysize;
+#endif
+
+	LWDEBUGF(3, "lwpoly_grid: applying grid to polygon with %d rings", poly->nrings);
+
+	opoly = lwpoly_construct_empty(poly->srid, lwgeom_has_z((LWGEOM*)poly), lwgeom_has_m((LWGEOM*)poly));
+
+	for (ri=0; ri<poly->nrings; ri++)
+	{
+		POINTARRAY *ring = poly->rings[ri];
+		POINTARRAY *newring;
+
+		newring = ptarray_grid(ring, grid);
+
+		/* Skip ring if not composed by at least 4 pts (3 segments) */
+		if ( newring->npoints < 4 )
+		{
+			ptarray_free(newring);
+
+			LWDEBUGF(3, "grid_polygon3d: ring%d skipped ( <4 pts )", ri);
+
+			if ( ri ) continue;
+			else break; /* this is the external ring, no need to work on holes */
+		}
+		
+		if ( ! lwpoly_add_ring(opoly, newring) )
+		{
+			lwerror("lwpoly_grid, memory error");
+			return NULL;
+		}
+	}
+
+	LWDEBUGF(3, "lwpoly_grid: simplified polygon with %d rings", opoly->nrings);
+
+	if ( ! opoly->nrings ) 
+	{
+		lwpoly_free(opoly);
+		return NULL;
+	}
+
+	return opoly;
+}
