@@ -1,5 +1,4 @@
 /**********************************************************************
- * $Id: lwprint.c 9324 2012-02-27 22:08:12Z pramsey $
  *
  * This is free software; you can redistribute and/or modify it under
  * the terms of the GNU General Public Licence. See the COPYING file.
@@ -57,8 +56,8 @@ static void lwprint_normalize_latlon(double *lat, double *lon)
 static char * lwdouble_to_dms(double val, const char *pos_dir_symbol, const char *neg_dir_symbol, const char * format)
 {
 	/* 3 numbers, 1 sign or compass dir, and 5 possible strings (degree signs, spaces, misc text, etc) between or around them.*/
-	const int NUM_PIECES = 9;
-	const int WORK_SIZE = 1024;
+	static int NUM_PIECES = 9;
+	static int WORK_SIZE = 1024;
 	char pieces[NUM_PIECES][WORK_SIZE];
 	int current_piece = 0;
 	int is_negative = 0;
@@ -322,7 +321,10 @@ static char * lwdouble_to_dms(double val, const char *pos_dir_symbol, const char
 	{
 		lwerror("Bad format, degrees (DD.DDD) number of digits was greater than our working limit.");
 	}
-	sprintf(pieces[deg_piece], "%*.*f", deg_digits, deg_dec_digits, degrees);
+	if(deg_piece >= 0) 
+	{
+		sprintf(pieces[deg_piece], "%*.*f", deg_digits, deg_dec_digits, degrees);
+	}
 
 	if (min_piece >= 0)
 	{
@@ -390,7 +392,7 @@ static char * lwdoubles_to_latlon(double lat, double lon, const char * format)
  */
 char* lwpoint_to_latlon(const LWPOINT * pt, const char *format)
 {
-	POINT2D p;
+	const POINT2D *p;
 	if (NULL == pt)
 	{
 		lwerror("Cannot convert a null point into formatted text.");
@@ -399,6 +401,6 @@ char* lwpoint_to_latlon(const LWPOINT * pt, const char *format)
 	{
 		lwerror("Cannot convert an empty point into formatted text.");
 	}
-	getPoint2d_p(pt->point, 0, &p);
-	return lwdoubles_to_latlon(p.y, p.x, format);
+	p = getPoint2d_cp(pt->point, 0);
+	return lwdoubles_to_latlon(p->y, p->x, format);
 }
