@@ -2,6 +2,7 @@
 #include <stdlib.h>
 #include <stdarg.h>
 #include <string.h>
+#include <ctype.h> /* for tolower */
 
 /* Global variables */
 #include "../postgis_config.h"
@@ -22,6 +23,12 @@ static void default_errorreporter(const char *fmt, va_list ap);
 lwreporter lwnotice_var = default_noticereporter;
 lwreporter lwerror_var = default_errorreporter;
 
+/* Default logger */
+static void default_debuglogger(int level, const char *fmt, va_list ap);
+lwdebuglogger lwdebug_var = default_debuglogger;
+
+#define LW_MSG_MAXLEN 256
+
 static char *lwgeomTypeName[] =
 {
 	"Unknown",
@@ -76,6 +83,19 @@ lwerror(const char *fmt, ...)
 	va_end(ap);
 }
 
+void
+lwdebug(int level, const char *fmt, ...)
+{
+	va_list ap;
+
+	va_start(ap, fmt);
+
+	/* Call the supplied function */
+	(*lwdebug_var)(level, fmt, ap);
+
+	va_end(ap);
+}
+
 /*
  * Default allocators
  *
@@ -107,37 +127,35 @@ default_reallocator(void *mem, size_t size)
 static void
 default_noticereporter(const char *fmt, va_list ap)
 {
-	char *msg;
-
-	/*
-	 * This is a GNU extension.
-	 * Dunno how to handle errors here.
-	 */
-	if (!lw_vasprintf (&msg, fmt, ap))
-	{
-		va_end (ap);
-		return;
-	}
+	char msg[LW_MSG_MAXLEN+1];
+	vsnprintf (msg, LW_MSG_MAXLEN, fmt, ap);
+	msg[LW_MSG_MAXLEN]='\0';
 	printf("%s\n", msg);
-	free(msg);
+}
+
+static void
+default_debuglogger(int level, const char *fmt, va_list ap)
+{
+	char msg[LW_MSG_MAXLEN+1];
+	if ( POSTGIS_DEBUG_LEVEL >= level )
+	{
+		/* Space pad the debug output */
+		int i;
+		for ( i = 0; i < level; i++ )
+			msg[i] = ' ';
+		vsnprintf(msg+i, LW_MSG_MAXLEN-i, fmt, ap);
+		msg[LW_MSG_MAXLEN]='\0';
+		printf("%s\n", msg);
+	}
 }
 
 static void
 default_errorreporter(const char *fmt, va_list ap)
 {
-	char *msg;
-
-	/*
-	 * This is a GNU extension.
-	 * Dunno how to handle errors here.
-	 */
-	if (!lw_vasprintf (&msg, fmt, ap))
-	{
-		va_end (ap);
-		return;
-	}
+	char msg[LW_MSG_MAXLEN+1];
+	vsnprintf (msg, LW_MSG_MAXLEN, fmt, ap);
+	msg[LW_MSG_MAXLEN]='\0';
 	fprintf(stderr, "%s\n", msg);
-	free(msg);
 	exit(1);
 }
 
@@ -160,6 +178,12 @@ lwgeom_set_handlers(lwallocator allocator, lwreallocator reallocator,
 	if ( noticereporter ) lwnotice_var = noticereporter;
 }
 
+void
+lwgeom_set_debuglogger(lwdebuglogger debuglogger) {
+
+	if ( debuglogger ) lwdebug_var = debuglogger;
+}
+
 const char* 
 lwtype_name(uint8_t type)
 {
