@@ -1,20 +1,21 @@
-
 /**********************************************************************
- * $Id: measures.h 4715 2009-11-01 17:58:42Z nicklas $
  *
  * PostGIS - Spatial Types for PostgreSQL
  * http://postgis.net
- * Copyright 2011 Nicklas Avn
+ * Copyright 2011 Nicklas Avén
  *
  * This is free software; you can redistribute and/or modify it under
  * the terms of the GNU General Public Licence. See the COPYING file.
  *
  **********************************************************************/
 
-#include "liblwgeom_internal.h"
+#ifndef _MEASURES3D_H
+#define _MEASURES3D_H 1
+#include <float.h>
+#include "measures.h"
 
-#define DOT(u,v)   (u.x * v.x + u.y * v.y + u.z * v.z)
-#define VECTORLENGTH(v)   sqrt((v.x * v.x) + (v.y * v.y) + (v.z * v.z))
+#define DOT(u,v)   ((u).x * (v).x + (u).y * (v).y + (u).z * (v).z)
+#define VECTORLENGTH(v)   sqrt(((v).x * (v).x) + ((v).y * (v).y) + ((v).z * (v).z))
 
 
 /**
@@ -45,12 +46,18 @@ typedef struct
 PLANE3D; 
 
 
+/*
+Geometry returning functions
+*/
+LWGEOM * lw_dist3d_distancepoint(const LWGEOM *lw1, const LWGEOM *lw2,int srid,int mode);
+LWGEOM * lw_dist3d_distanceline(const LWGEOM *lw1, const LWGEOM *lw2,int srid,int mode);
+
 /*
 Preprocessing functions
 */
-int lw_dist3d_distribute_bruteforce(LWGEOM *lwg1, LWGEOM *lwg2, DISTPTS3D *dl);
+int lw_dist3d_distribute_bruteforce(const LWGEOM *lwg1, const LWGEOM *lwg2, DISTPTS3D *dl);
 int lw_dist3d_recursive(const LWGEOM *lwg1,const LWGEOM *lwg2, DISTPTS3D *dl);
-int lw_dist3d_distribute_fast(LWGEOM *lwg1, LWGEOM *lwg2, DISTPTS3D *dl);
+int lw_dist3d_distribute_fast(const LWGEOM *lwg1, const LWGEOM *lwg2, DISTPTS3D *dl);
 
 /*
 Brute force functions
@@ -74,29 +81,10 @@ int lw_dist3d_ptarray_poly(POINTARRAY *pa, LWPOLY *poly, PLANE3D *plane, DISTPTS
 double project_point_on_plane(POINT3DZ *p,  PLANE3D *pl, POINT3DZ *p0);
 int define_plane(POINTARRAY *pa, PLANE3D *pl);
 int pt_in_ring_3d(const POINT3DZ *p, const POINTARRAY *ring,PLANE3D *plane);
+
 /*
 Helper functions
 */
-int get_3dvector_from_points(POINT3DZ *p1,POINT3DZ *p2, VECTOR3D *v);
-int get_3dcross_product(VECTOR3D *v1,VECTOR3D *v2, VECTOR3D *v);
 
 
-int
-get_3dvector_from_points(POINT3DZ *p1,POINT3DZ *p2, VECTOR3D *v)
-{
-	v->x=p2->x-p1->x;
-	v->y=p2->y-p1->y;
-	v->z=p2->z-p1->z;
-	
-	return LW_TRUE;
-}
-
-int
-get_3dcross_product(VECTOR3D *v1,VECTOR3D *v2, VECTOR3D *v)
-{
-	v->x=(v1->y*v2->z)-(v1->z*v2->y);
-	v->y=(v1->z*v2->x)-(v1->x*v2->z);
-	v->z=(v1->x*v2->y)-(v1->y*v2->x);
-
-	return LW_TRUE;
-}
\ No newline at end of file
+#endif /* !defined _MEASURES3D_H  */
