@@ -29,6 +29,8 @@
 
 #define PARANOIA_LEVEL 1
 
+#define PGC_ERRMSG_MAXLEN 256
+
 /**
 * Utility to convert cstrings to textp pointers 
 */
@@ -152,32 +154,48 @@ pg_free(void *ptr)
 static void
 pg_error(const char *fmt, va_list ap)
 {
-#define ERRMSG_MAXLEN 256
+	char errmsg[PGC_ERRMSG_MAXLEN+1];
 
-	char errmsg[ERRMSG_MAXLEN+1];
+	vsnprintf (errmsg, PGC_ERRMSG_MAXLEN, fmt, ap);
 
-	vsnprintf (errmsg, ERRMSG_MAXLEN, fmt, ap);
-
-	errmsg[ERRMSG_MAXLEN]='\0';
+	errmsg[PGC_ERRMSG_MAXLEN]='\0';
 	ereport(ERROR, (errmsg_internal("%s", errmsg)));
 }
 
+static void
+pg_warning(const char *fmt, va_list ap)
+{
+	char errmsg[PGC_ERRMSG_MAXLEN+1];
+
+	vsnprintf (errmsg, PGC_ERRMSG_MAXLEN, fmt, ap);
+
+	errmsg[PGC_ERRMSG_MAXLEN]='\0';
+	ereport(WARNING, (errmsg_internal("%s", errmsg)));
+}
+
 static void
 pg_notice(const char *fmt, va_list ap)
 {
-	char *msg;
+	char errmsg[PGC_ERRMSG_MAXLEN+1];
 
-	/*
-	 * This is a GNU extension.
-	 * Dunno how to handle errors here.
-	 */
-	if (!lw_vasprintf (&msg, fmt, ap))
-	{
-		va_end (ap);
-		return;
-	}
-	ereport(NOTICE, (errmsg_internal("%s", msg)));
-	free(msg);
+	vsnprintf (errmsg, PGC_ERRMSG_MAXLEN, fmt, ap);
+
+	errmsg[PGC_ERRMSG_MAXLEN]='\0';
+	ereport(NOTICE, (errmsg_internal("%s", errmsg)));
+}
+
+static void
+pg_debug(int level, const char *fmt, va_list ap)
+{
+	char errmsg[PGC_ERRMSG_MAXLEN+1];
+	vsnprintf (errmsg, PGC_ERRMSG_MAXLEN, fmt, ap);
+	errmsg[PGC_ERRMSG_MAXLEN]='\0';
+	int pglevel[6] = {NOTICE, DEBUG1, DEBUG2, DEBUG3, DEBUG4, DEBUG5};
+
+	if ( level >= 0 && level <= 5 )
+		ereport(pglevel[level], (errmsg_internal("%s", errmsg)));
+	else
+		ereport(DEBUG5, (errmsg_internal("%s", errmsg)));		
 }
 
 void
@@ -185,6 +203,7 @@ pg_install_lwgeom_handlers(void)
 {
 	/* install PostgreSQL handlers */
 	lwgeom_set_handlers(pg_alloc, pg_realloc, pg_free, pg_error, pg_notice);
+	lwgeom_set_debuglogger(pg_debug);
 }
 
 /**
@@ -203,7 +222,7 @@ GSERIALIZED* geography_serialize(LWGEOM *lwgeom)
 	GSERIALIZED *g = NULL;
 
 	g = gserialized_from_lwgeom(lwgeom, is_geodetic, &ret_size);
-	if ( ! g ) lwerror("Unable to serialize lwgeom.");
+	if ( ! g ) lwpgerror("Unable to serialize lwgeom.");
 	SET_VARSIZE(g, ret_size);
 	return g;
 }
@@ -220,7 +239,43 @@ GSERIALIZED* geometry_serialize(LWGEOM *lwgeom)
 	GSERIALIZED *g = NULL;
 
 	g = gserialized_from_lwgeom(lwgeom, is_geodetic, &ret_size);
-	if ( ! g ) lwerror("Unable to serialize lwgeom.");
+	if ( ! g ) lwpgerror("Unable to serialize lwgeom.");
 	SET_VARSIZE(g, ret_size);
 	return g;
 }
+
+void
+lwpgnotice(const char *fmt, ...)
+{
+	va_list ap;
+
+	va_start(ap, fmt);
+
+  pg_notice(fmt, ap);
+
+	va_end(ap);
+}
+
+void
+lwpgwarning(const char *fmt, ...)
+{
+	va_list ap;
+
+	va_start(ap, fmt);
+
+  pg_warning(fmt, ap);
+
+	va_end(ap);
+}
+
+void
+lwpgerror(const char *fmt, ...)
+{
+	va_list ap;
+
+	va_start(ap, fmt);
+
+  pg_error(fmt, ap);
+
+	va_end(ap);
+}
