@@ -1,5 +1,4 @@
 /******************************************************************************
- * $Id: dbfopen.c 9324 2012-02-27 22:08:12Z pramsey $
  *
  * Project:  Shapelib
  * Purpose:  Implementation of .dbf access API documented in dbf_api.html.
@@ -163,7 +162,7 @@
 #include <ctype.h>
 #include <string.h>
 
-SHP_CVSID("$Id: dbfopen.c 9324 2012-02-27 22:08:12Z pramsey $")
+SHP_CVSID("$Id: dbfopen.c 13497 2015-05-13 18:29:52Z pramsey $")
 
 #ifndef FALSE
 #  define FALSE		0
@@ -505,7 +504,7 @@ DBFOpenLL( const char * pszFilename, const char * pszAccess, SAHooks *psHooks )
         }
 		psDBF->sHooks.FClose( pfCPG );
     }
-    if( psDBF->pszCodePage == NULL && pabyBuf[29] != 0 )
+    if( (psDBF->pszCodePage == NULL) && (psDBF->iLanguageDriver != 0) )
     {
         sprintf( (char *) pabyBuf, "LDID/%d", psDBF->iLanguageDriver );
         psDBF->pszCodePage = (char *) malloc(strlen((char*)pabyBuf) + 1);
@@ -629,7 +628,7 @@ DBFHandle SHPAPI_CALL
 DBFCreate( const char * pszFilename )
 
 {
-    return DBFCreateEx( pszFilename, "LDID/87" ); // 0x57
+    return DBFCreateEx( pszFilename, "LDID/87" ); /* 0x57 */
 }
 
 /************************************************************************/
@@ -704,7 +703,7 @@ DBFCreateLL( const char * pszFilename, const char * pszCodePage, SAHooks *psHook
         {
             ldid = atoi( pszCodePage + 5 );
             if( ldid > 255 )
-                ldid = -1; // don't use 0 to indicate out of range as LDID/0 is a valid one
+                ldid = -1; /* don't use 0 to indicate out of range as LDID/0 is a valid one */
         }
         if( ldid < 0 )
         {
@@ -1133,14 +1132,14 @@ static int DBFIsValueNULL( char chType, const char* pszValue )
 
       case 'D':
         /* NULL date fields have value "00000000" or blank or empty */
-        if (pszValue[0] == '\0' ||                  // emtpy string
+        if (pszValue[0] == '\0' ||                  /* emtpy string */
             strncmp(pszValue,"00000000",8) == 0 || 
             strncmp(pszValue,"        ",8) == 0) {
             return 1;
         } else {
             return 0;
         }
-        // return strncmp(pszValue,"00000000",8) == 0;
+        /* return strncmp(pszValue,"00000000",8) == 0; */
 
       case 'L':
         /* NULL boolean fields have value "?" or empty */
