@@ -1,5 +1,4 @@
 /**********************************************************************
- * $Id: pgsql2shp-core.h 13134 2014-12-01 08:47:21Z strk $
  *
  * PostGIS - Spatial Types for PostgreSQL
  * http://postgis.net
@@ -23,14 +22,12 @@
 #include <sys/stat.h>
 #include <iconv.h>
 
-#include "../postgis_config.h"
 #include "libpq-fe.h"
 #include "shapefil.h"
 #include "shpcommon.h"
 #include "getopt.h"
 
-#define P2S_RCSID "$Id: pgsql2shp-core.h 13134 2014-12-01 08:47:21Z strk $"
-
+#define P2S_RCSID "$Id: pgsql2shp-core.h 14176 2015-10-03 06:12:01Z strk $"
 
 /*
  * Error message handling
@@ -184,14 +181,8 @@ typedef struct shp_dumper_state
 	/* Last (error) message */
 	char message[SHPDUMPERMSGLEN];
 
-	/* Column map pgfieldnames */
-	char **column_map_pgfieldnames;
-	
-	/* Column map dbffieldnames */
-	char **column_map_dbffieldnames;
-	
-	/* Number of entries within column map */
-	int column_map_size;
+	/* Column map */
+  colmap column_map;
 
 } SHPDUMPERSTATE;
 
