@@ -1,7 +1,7 @@
 /**********************************************************************
  *
  * PostGIS - Spatial Types for PostgreSQL
- * http://www.postgis.org
+ * http://postgis.net
  * 
  * Copyright (C) 2008 OpenGeo.org
  * Copyright (C) 2009 Mark Cave-Ayland <mark.cave-ayland@siriusit.co.uk>
@@ -16,10 +16,11 @@
 #include "../postgis_config.h"
 
 #include "shp2pgsql-core.h"
-#include "../liblwgeom/liblwgeom.h" /* for lw_vasprintf */
+#include "../liblwgeom/liblwgeom.h"
 #include "../liblwgeom/lwgeom_log.h" /* for LWDEBUG macros */
 
 
+
 /* Internal ring/point structures */
 typedef struct struct_point
 {
@@ -43,7 +44,6 @@ typedef struct struct_ring
 #define UTF8_BAD_RESULT 1
 #define UTF8_NO_RESULT 2
 
-int utf8(const char *fromcode, char *inputbuf, char **outputbuf);
 char *escape_copy_string(char *str);
 char *escape_insert_string(char *str);
 
@@ -54,30 +54,10 @@ int FindPolygons(SHPObject *obj, Ring ***Out);
 void ReleasePolygons(Ring **polys, int npolys);
 int GeneratePolygonGeometry(SHPLOADERSTATE *state, SHPObject *obj, char **geometry);
 
-/* Append variadic formatted string to a stringbuffer */
-void
-vasbappend(stringbuffer_t *sb, char *fmt, ... )
-{
-	va_list ap;
-	char *msg;
-
-	va_start(ap, fmt);
-
-	if (!lw_vasprintf (&msg, fmt, ap))
-	{
-		va_end (ap);
-		return;
-	}
-
-	/* Append to the stringbuffer */
-	stringbuffer_append(sb, msg);
-	free(msg);
-
-	va_end(ap);
-}
 
 /* Return allocated string containing UTF8 string converted from encoding fromcode */
-int utf8(const char *fromcode, char *inputbuf, char **outputbuf)
+static int 
+utf8(const char *fromcode, char *inputbuf, char **outputbuf)
 {
 	iconv_t cd;
 	char *outputptr;
@@ -544,7 +524,14 @@ FindPolygons(SHPObject *obj, Ring ***Out)
 		pt2.x = inner->list[1].x;
 		pt2.y = inner->list[1].y;
 
-		for (i = 0; i < out_index; i++)
+		/* 
+		* If we assume that the case of the "big polygon w/o hole 
+		* containing little polygon w/ hold" is ordered so that the 
+		* big polygon comes first, then checking the list in reverse
+		* will assign the little polygon's hole to the little polygon
+		* w/o a lot of extra fancy containment logic here
+		*/
+		for (i = out_index - 1; i >= 0; i--)
 		{
 			int in;
 
@@ -554,7 +541,6 @@ FindPolygons(SHPObject *obj, Ring ***Out)
 				outer = Outer[i];
 				break;
 			}
-			/*fprintf(stderr, "!PIP %s\nOUTE %s\n", dump_ring(inner), dump_ring(Outer[i])); */
 		}
 
 		if (outer)
@@ -577,7 +563,11 @@ FindPolygons(SHPObject *obj, Ring ***Out)
 	}
 
 	*Out = Outer;
-	free(Inner);
+	/* 
+	* Only free the containing Inner array, not the ring elements, because
+	* the rings are now owned by the linked lists in the Outer array elements.
+	*/
+	free(Inner); 
 
 	return out_index;
 }
@@ -771,6 +761,7 @@ set_loader_config_defaults(SHPLOADERCONFIG *config)
 	config->tablespace = NULL;
 	config->idxtablespace = NULL;
 	config->usetransaction = 1;
+	config->column_map_filename = NULL;
 }
 
 /* Create a new shapefile state object */
@@ -823,6 +814,8 @@ ShpLoaderCreate(SHPLOADERCONFIG *config)
 		state->geo_col = strdup(config->geography ? GEOGRAPHY_DEFAULT : GEOMETRY_DEFAULT);
 	}
 
+	colmap_init(&state->column_map);
+
 	return state;
 }
 
@@ -863,6 +856,31 @@ ShpLoaderOpenShape(SHPLOADERSTATE *state)
 
 		return SHPLOADERERR;
 	}
+	
+		
+	/* Open the column map if one was specified */
+	if (state->config->column_map_filename)
+	{
+		ret = colmap_read(state->config->column_map_filename,
+		                  &state->column_map, state->message, SHPLOADERMSGLEN);
+		if (!ret) return SHPLOADERERR;
+	}
+	
+	/* User hasn't altered the default encoding preference... */
+	if ( strcmp(state->config->encoding, ENCODING_DEFAULT) == 0 )
+	{
+	    /* But the file has a code page entry... */
+	    if ( state->hDBFHandle->pszCodePage )
+	    {
+	        /* And we figured out what iconv encoding it maps to, so use it! */
+            char *newencoding = NULL;
+	        if ( (newencoding = codepage2encoding(state->hDBFHandle->pszCodePage)) )
+	        {
+                lwfree(state->config->encoding);
+                state->config->encoding = newencoding;
+            }
+        }
+	}
 
 	/* If reading the whole shapefile (not just attributes)... */
 	if (state->config->readshape == 1)
@@ -1103,6 +1121,17 @@ ShpLoaderOpenShape(SHPLOADERSTATE *state)
 			strncpy(name, utf8str, MAXFIELDNAMELEN);
 			free(utf8str);
 		}
+		
+		/* If a column map file has been passed in, use this to create the postgresql field name from
+		   the dbf column name */
+		{
+		  const char *mapped = colmap_pg_by_dbf(&state->column_map, name);
+		  if (mapped)
+		  {
+			  strncpy(name, mapped, MAXFIELDNAMELEN);
+			  name[MAXFIELDNAMELEN-1] = '\0';
+			}
+		}
 
 		/*
 		 * Make field names lowercase unless asked to
@@ -1790,7 +1819,7 @@ ShpLoaderGetSQLFooter(SHPLOADERSTATE *state, char **strfooter)
 	stringbuffer_clear(sb);
 
 	/* Create gist index if specified and not in "prepare" mode */
-	if (state->config->createindex)
+	if (state->config->readshape && state->config->createindex)
 	{
 		stringbuffer_aprintf(sb, "CREATE INDEX ON ");
 		/* Schema is optional, include if present. */
@@ -1813,6 +1842,14 @@ ShpLoaderGetSQLFooter(SHPLOADERSTATE *state, char **strfooter)
 		stringbuffer_aprintf(sb, "COMMIT;\n");
 	}
 
+	/* Always ANALYZE the resulting table, for better stats */
+	stringbuffer_aprintf(sb, "ANALYZE ");
+	if (state->config->schema)
+	{
+		stringbuffer_aprintf(sb, "\"%s\".", state->config->schema);
+	}
+	stringbuffer_aprintf(sb, "\"%s\";\n", state->config->table);
+
 	/* Copy the string buffer into a new string, destroying the string buffer */
 	ret = (char *)malloc(strlen((char *)stringbuffer_getstring(sb)) + 1);
 	strcpy(ret, (char *)stringbuffer_getstring(sb));
@@ -1859,6 +1896,9 @@ ShpLoaderDestroy(SHPLOADERSTATE *state)
 		if (state->col_names)
 			free(state->col_names);
 
+		/* Free any column map fieldnames if specified */
+		colmap_clean(&state->column_map);
+		
 		/* Free the state itself */
 		free(state);
 	}
