@@ -1,5 +1,4 @@
 /**********************************************************************
- * $Id: shp2pgsql-core.h 13447 2015-04-26 04:33:54Z robe $
  *
  * PostGIS - Spatial Types for PostgreSQL
  * http://postgis.net
@@ -30,8 +29,7 @@
 
 #include "../liblwgeom/stringbuffer.h"
 
-#define S2P_RCSID "$Id: shp2pgsql-core.h 13447 2015-04-26 04:33:54Z robe $"
-
+#define S2P_RCSID "$Id: shp2pgsql-core.h 13497 2015-05-13 18:29:52Z pramsey $"
 
 /* Number of digits of precision in WKT produced. */
 #define WKT_PRECISION 15
@@ -145,6 +143,9 @@ typedef struct shp_loader_config
 
 	/* whether to do a single transaction or run each statement on its own */
 	int usetransaction;
+	
+	/* Name of the column map file if specified */
+	char *column_map_filename;
 
 } SHPLOADERCONFIG;
 
@@ -215,13 +216,15 @@ typedef struct shp_loader_state
 	/* geometry/geography column name to use.  Will be set to the default if the config did
 	   not specify a column name. */
 	char *geo_col; 
+	
+	/* Column map */
+  colmap column_map;
 
 } SHPLOADERSTATE;
 
 
 /* Externally accessible functions */
 void strtolower(char *s);
-void vasbappend(stringbuffer_t *sb, char *fmt, ... );
 void set_loader_config_defaults(SHPLOADERCONFIG *config);
 
 SHPLOADERSTATE *ShpLoaderCreate(SHPLOADERCONFIG *config);
