@@ -1,5 +1,4 @@
 /**********************************************************************
- * $Id: geography.h 9324 2012-02-27 22:08:12Z pramsey $
  *
  * PostGIS - Spatial Types for PostgreSQL
  * Copyright 2009 Paul Ramsey <pramsey@cleverelephant.ca>
@@ -16,7 +15,7 @@
 */
 
 /* Check that the typmod matches the flags on the lwgeom */
-void postgis_valid_typmod(const GSERIALIZED *gser, int32_t typmod);
+GSERIALIZED* postgis_valid_typmod(GSERIALIZED *gser, int32_t typmod);
 /* Check that the type is legal in geography (no curves please!) */
 void geography_valid_type(uint8_t type);
 
