@@ -1,5 +1,4 @@
 /**********************************************************************
- * $Id: geography_btree.c 13436 2015-04-23 18:19:47Z pramsey $
  *
  * PostGIS - Spatial Types for PostgreSQL
  * Copyright 2009 Paul Ramsey <pramsey@cleverelephant.ca>
@@ -178,8 +177,8 @@ PG_FUNCTION_INFO_V1(geography_eq);
 Datum geography_eq(PG_FUNCTION_ARGS)
 {
 	/* Perfect equals test based on hash */
-	GSERIALIZED *g1 = (GSERIALIZED*)PG_DETOAST_DATUM(PG_GETARG_DATUM(0));
-	GSERIALIZED *g2 = (GSERIALIZED*)PG_DETOAST_DATUM(PG_GETARG_DATUM(1));
+	GSERIALIZED *g1 = PG_GETARG_GSERIALIZED_P(0);
+	GSERIALIZED *g2 = PG_GETARG_GSERIALIZED_P(1);
 
 	uint32 h1 = geography_hash(g1);
 	uint32 h2 = geography_hash(g2);
@@ -210,7 +209,7 @@ Datum geography_eq(PG_FUNCTION_ARGS)
 
 	/* Must be able to build box for each argument (ie, not empty geometry) */
 	if ( ! gserialized_datum_get_gidx_p(PG_GETARG_DATUM(0), gbox1) ||
-	        ! gserialized_datum_get_gidx_p(PG_GETARG_DATUM(1), gbox2) )
+	     ! gserialized_datum_get_gidx_p(PG_GETARG_DATUM(1), gbox2) )
 	{
 		PG_RETURN_BOOL(FALSE);
 	}
@@ -241,7 +240,7 @@ Datum geography_cmp(PG_FUNCTION_ARGS)
 
 	/* Must be able to build box for each argument (ie, not empty geometry) */
 	if ( ! gserialized_datum_get_gidx_p(PG_GETARG_DATUM(0), gbox1) ||
-	     ! gserialized_datum_get_gidx_p(PG_GETARG_DATUM(1), gbox2) )
+	        ! gserialized_datum_get_gidx_p(PG_GETARG_DATUM(1), gbox2) )
 	{
 		PG_RETURN_BOOL(FALSE);
 	}
