@@ -1,5 +1,4 @@
 /**********************************************************************
- * $Id: geography_inout.c 13368 2015-03-20 14:06:44Z pramsey $
  *
  * PostGIS - Spatial Types for PostgreSQL
  * Copyright 2009-2011 Paul Ramsey <pramsey@cleverelephant.ca>
@@ -180,7 +179,7 @@ Datum geography_out(PG_FUNCTION_ARGS)
 	GSERIALIZED *g = NULL;
 	char *hexwkb;
 
-	g = (GSERIALIZED*)PG_DETOAST_DATUM(PG_GETARG_DATUM(0));
+	g = PG_GETARG_GSERIALIZED_P(0);
 	lwgeom = lwgeom_from_gserialized(g);
 	hexwkb = lwgeom_to_hexwkb(lwgeom, WKB_EXTENDED, 0);
 	lwgeom_free(lwgeom);
@@ -223,7 +222,7 @@ Datum geography_as_gml(PG_FUNCTION_ARGS)
 
 	/* Get the geography */
 	if ( PG_ARGISNULL(1) ) PG_RETURN_NULL();
-	g = (GSERIALIZED*)PG_DETOAST_DATUM(PG_GETARG_DATUM(1));
+	g = PG_GETARG_GSERIALIZED_P(1);
 
 	/* Convert to lwgeom so we can run the old functions */
 	lwgeom = lwgeom_from_gserialized(g);
@@ -341,7 +340,7 @@ Datum geography_as_kml(PG_FUNCTION_ARGS)
 
 	/* Get the geometry */
 	if ( PG_ARGISNULL(1) ) PG_RETURN_NULL();
-	g = (GSERIALIZED*)PG_DETOAST_DATUM(PG_GETARG_DATUM(1));
+	g = PG_GETARG_GSERIALIZED_P(1);
 
 	/* Convert to lwgeom so we can run the old functions */
 	lwgeom = lwgeom_from_gserialized(g);
@@ -407,7 +406,7 @@ Datum geography_as_svg(PG_FUNCTION_ARGS)
 
 	if ( PG_ARGISNULL(0) ) PG_RETURN_NULL();
 
-	g = (GSERIALIZED*)PG_DETOAST_DATUM(PG_GETARG_DATUM(0));
+	g = PG_GETARG_GSERIALIZED_P(0);
 
 	/* Convert to lwgeom so we can run the old functions */
 	lwgeom = lwgeom_from_gserialized(g);
@@ -463,7 +462,7 @@ Datum geography_as_geojson(PG_FUNCTION_ARGS)
 
 	/* Get the geography */
 	if (PG_ARGISNULL(1) ) PG_RETURN_NULL();
-	g = (GSERIALIZED*)PG_DETOAST_DATUM(PG_GETARG_DATUM(1));
+	g = PG_GETARG_GSERIALIZED_P(1);
 
 	/* Convert to lwgeom so we can run the old functions */
 	lwgeom = lwgeom_from_gserialized(g);
@@ -533,6 +532,9 @@ Datum geography_from_text(PG_FUNCTION_ARGS)
 	if ( lwgeom_parse_wkt(&lwg_parser_result, wkt, LW_PARSER_CHECK_ALL) == LW_FAILURE )
 		PG_PARSER_ERROR(lwg_parser_result);
 
+	/* Error on any SRID != default */
+	srid_is_latlong(fcinfo, lwg_parser_result.geom->srid);
+
 	/* Clean up string */
 	pfree(wkt);
 	g_ser = gserialized_geography_from_lwgeom(lwg_parser_result.geom, -1);
@@ -556,7 +558,10 @@ Datum geography_from_binary(PG_FUNCTION_ARGS)
 	LWGEOM *lwgeom = lwgeom_from_wkb(wkb, wkb_size, LW_PARSER_CHECK_NONE);
 	
 	if ( ! lwgeom )
-		lwerror("Unable to parse WKB");
+		lwpgerror("Unable to parse WKB");
+
+	/* Error on any SRID != default */
+	srid_is_latlong(fcinfo, lwgeom->srid);
  		
 	gser = gserialized_geography_from_lwgeom(lwgeom, -1);
 	lwgeom_free(lwgeom);
@@ -567,7 +572,7 @@ Datum geography_from_binary(PG_FUNCTION_ARGS)
 PG_FUNCTION_INFO_V1(geography_from_geometry);
 Datum geography_from_geometry(PG_FUNCTION_ARGS)
 {
-	GSERIALIZED *geom = (GSERIALIZED*)PG_DETOAST_DATUM_COPY(PG_GETARG_DATUM(0));
+	GSERIALIZED *geom = PG_GETARG_GSERIALIZED_P_COPY(0);
 	LWGEOM *lwgeom = NULL;
 	GSERIALIZED *g_ser = NULL;
 
@@ -598,9 +603,11 @@ Datum geography_from_geometry(PG_FUNCTION_ARGS)
 	** functions do the right thing.
 	*/
 	lwgeom_set_geodetic(lwgeom, true);
+	
 	/* Recalculate the boxes after re-setting the geodetic bit */
 	lwgeom_drop_bbox(lwgeom);
 	lwgeom_add_bbox(lwgeom);
+	
 	g_ser = geography_serialize(lwgeom);
 
 	/*
@@ -618,7 +625,7 @@ Datum geometry_from_geography(PG_FUNCTION_ARGS)
 {
 	LWGEOM *lwgeom = NULL;
 	GSERIALIZED *ret = NULL;
-	GSERIALIZED *g_ser = (GSERIALIZED*)PG_DETOAST_DATUM(PG_GETARG_DATUM(0));
+	GSERIALIZED *g_ser = PG_GETARG_GSERIALIZED_P(0);
 
 	lwgeom = lwgeom_from_gserialized(g_ser);
 
@@ -676,7 +683,7 @@ Datum geography_send(PG_FUNCTION_ARGS)
 	uint8_t *wkb;
 	bytea *result;
 
-	g = (GSERIALIZED*)PG_DETOAST_DATUM(PG_GETARG_DATUM(0));
+	g = PG_GETARG_GSERIALIZED_P(0);
 	lwgeom = lwgeom_from_gserialized(g);
 	wkb = lwgeom_to_wkb(lwgeom, WKB_EXTENDED, &size_result);
 	lwgeom_free(lwgeom);
