@@ -1,5 +1,4 @@
 /**********************************************************************
- * $Id: gserialized_gist_2d.c 6519 2010-12-28 00:54:19Z pramsey $
  *
  * PostGIS - Spatial Types for PostgreSQL
  * Copyright 2009 Paul Ramsey <pramsey@cleverelephant.ca>
@@ -34,16 +33,17 @@
 #include "liblwgeom.h"         /* For standard geometry types. */
 #include "lwgeom_pg.h"       /* For debugging macros. */
 #include "gserialized_gist.h"	     /* For utility functions. */
-#include "liblwgeom_internal.h"  /* For MAXFLOAT */
 
-/* Fall back to older finite() if necessary */ 
-#ifndef HAVE_ISFINITE 
-# ifdef HAVE_GNU_ISFINITE 
-#  define _GNU_SOURCE 
-# else 
-#  define isfinite finite 
-# endif 
-#endif 
+/* Fall back to older finite() if necessary */
+#ifndef HAVE_ISFINITE
+# ifdef HAVE_GNU_ISFINITE
+#  define _GNU_SOURCE
+# else
+#  define isfinite finite
+# endif
+#endif
+
+#include <float.h> /* For FLT_MAX */
 
 /*
 ** When is a node split not so good? If more than 90% of the entries
@@ -107,7 +107,6 @@ Datum gserialized_distance_centroid_2d(PG_FUNCTION_ARGS);
 typedef bool (*box2df_predicate)(const BOX2DF *a, const BOX2DF *b);
 
 
-#if POSTGIS_DEBUG_LEVEL > 0
 static char* box2df_to_string(const BOX2DF *a)
 {
 	char *rv = NULL;
@@ -119,8 +118,6 @@ static char* box2df_to_string(const BOX2DF *a)
 	sprintf(rv, "BOX2DF(%.12g %.12g, %.12g %.12g)", a->xmin, a->ymin, a->xmax, a->ymax);
 	return rv;
 }
-#endif
-
 
 /* Allocate a new copy of BOX2DF */
 static BOX2DF* box2df_copy(BOX2DF *b)
@@ -390,6 +387,7 @@ static double box2df_distance_leaf_centroid(const BOX2DF *a, const BOX2DF *b)
     return sqrt((a_x - b_x) * (a_x - b_x) + (a_y - b_y) * (a_y - b_y));
 }
 
+#if POSTGIS_PGSQL_VERSION < 95
 /**
 * Calculate the The node_box_edge->query_centroid distance 
 * between the boxes.
@@ -458,11 +456,13 @@ static double box2df_distance_node_centroid(const BOX2DF *node, const BOX2DF *qu
         else
         {
             /*ERROR*/
+			elog(ERROR, "%s: reached unreachable code", __func__);
         }
     }
     
     return sqrt(d);
 }
+#endif 
 
 /* Quick distance function */
 static inline double pt_distance(double ax, double ay, double bx, double by)
@@ -516,7 +516,7 @@ static double box2df_distance(const BOX2DF *a, const BOX2DF *b)
 			return b->ymin - a->ymax;
 	}
 	
-	return MAXFLOAT;
+	return FLT_MAX;
 }
 
 
@@ -539,14 +539,14 @@ gserialized_datum_get_box2df_p(Datum gsdatum, BOX2DF *box2df)
 	** The most info we need is the 8 bytes of serialized header plus the 
 	** of floats necessary to hold the bounding box.
 	*/
-	if (VARATT_IS_EXTENDED(gsdatum)) 
-	{ 
-		gpart = (GSERIALIZED*)PG_DETOAST_DATUM_SLICE(gsdatum, 0, 8 + sizeof(BOX2DF)); 
-	} 
-	else 
-	{ 
-		gpart = (GSERIALIZED*)PG_DETOAST_DATUM(gsdatum); 
-	} 
+	if (VARATT_IS_EXTENDED(gsdatum))
+	{
+		gpart = (GSERIALIZED*)PG_DETOAST_DATUM_SLICE(gsdatum, 0, 8 + sizeof(BOX2DF));
+	}
+	else
+	{
+		gpart = (GSERIALIZED*)PG_DETOAST_DATUM(gsdatum);
+	}
 
 	flags = gpart->flags;
 
@@ -630,7 +630,7 @@ Datum gserialized_distance_centroid_2d(PG_FUNCTION_ARGS)
 		POSTGIS_DEBUGF(3, "got boxes %s and %s", box2df_to_string(&b1), box2df_to_string(&b2));
 		PG_RETURN_FLOAT8(distance);
 	}
-	PG_RETURN_FLOAT8(MAXFLOAT);
+	PG_RETURN_FLOAT8(FLT_MAX);
 }
 
 PG_FUNCTION_INFO_V1(gserialized_distance_box_2d);
@@ -650,7 +650,7 @@ Datum gserialized_distance_box_2d(PG_FUNCTION_ARGS)
 		POSTGIS_DEBUGF(3, "got boxes %s and %s", box2df_to_string(&b1), box2df_to_string(&b2));
 		PG_RETURN_FLOAT8(distance);
 	}
-	PG_RETURN_FLOAT8(MAXFLOAT);
+	PG_RETURN_FLOAT8(FLT_MAX);
 }
 
 PG_FUNCTION_INFO_V1(gserialized_same_2d);
@@ -1058,8 +1058,8 @@ Datum gserialized_gist_consistent_2d(PG_FUNCTION_ARGS)
 ** represents the distance to the index entry; for an internal tree node, the
 ** result must be the smallest distance that any child entry could have.
 ** 
-** Strategy 13 = centroid-based distance tests
-** Strategy 14 = box-based distance tests
+** Strategy 13 = true distance tests <->
+** Strategy 14 = box-based distance tests <#>
 */
 PG_FUNCTION_INFO_V1(gserialized_gist_distance_2d);
 Datum gserialized_gist_distance_2d(PG_FUNCTION_ARGS)
@@ -1069,26 +1069,53 @@ Datum gserialized_gist_distance_2d(PG_FUNCTION_ARGS)
 	BOX2DF *entry_box;
 	StrategyNumber strategy = (StrategyNumber) PG_GETARG_UINT16(2);
 	double distance;
+#if POSTGIS_PGSQL_VERSION >= 95
+	bool *recheck = (bool *) PG_GETARG_POINTER(4);
+#endif
 
 	POSTGIS_DEBUG(4, "[GIST] 'distance' function called");
 
-    /* We are using '13' as the gist distance-betweeen-centroids strategy number 
-    *  and '14' as the gist distance-between-boxes strategy number */
-    if ( strategy != 13 && strategy != 14 ) {
-        elog(ERROR, "unrecognized strategy number: %d", strategy);
-		PG_RETURN_FLOAT8(MAXFLOAT);
+	/* We are using '13' as the gist true-distance <-> strategy number 
+	*  and '14' as the gist distance-between-boxes <#> strategy number */
+	if ( strategy != 13 && strategy != 14 ) {
+		elog(ERROR, "unrecognized strategy number: %d", strategy);
+		PG_RETURN_FLOAT8(FLT_MAX);
 	}
 
 	/* Null box should never make this far. */
 	if ( gserialized_datum_get_box2df_p(PG_GETARG_DATUM(1), &query_box) == LW_FAILURE )
 	{
 		POSTGIS_DEBUG(4, "[GIST] null query_gbox_index!");
-		PG_RETURN_FLOAT8(MAXFLOAT);
+		PG_RETURN_FLOAT8(FLT_MAX);
 	}
-
-	/* Get the entry box */
-    entry_box = (BOX2DF*)DatumGetPointer(entry->key);
 	
+	/* Get the entry box */
+	entry_box = (BOX2DF*)DatumGetPointer(entry->key);
+	
+#if POSTGIS_PGSQL_VERSION >= 95
+	
+	/* Box-style distance test */
+	if ( strategy == 14 ) /* operator <#> */
+	{
+		distance = (double)box2df_distance(entry_box, &query_box);
+	}
+	/* True distance test (formerly centroid distance) */
+	else if ( strategy == 13 ) /* operator <-> */
+	{
+		/* In all cases, since we only have keys (boxes) we'll return */
+		/* the minimum possible distance, which is the box2df_distance */
+		/* and let the recheck sort things out in the case of leaves */
+		distance = (double)box2df_distance(entry_box, &query_box);
+
+		if (GIST_LEAF(entry))
+			*recheck = true;
+	}
+	else
+	{
+		elog(ERROR, "%s: reached unreachable code", __func__);
+		PG_RETURN_NULL();
+	}
+#else
 	/* Box-style distance test */
 	if ( strategy == 14 )
 	{
@@ -1099,15 +1126,16 @@ Datum gserialized_gist_distance_2d(PG_FUNCTION_ARGS)
 	/* Treat leaf node tests different from internal nodes */
 	if (GIST_LEAF(entry))
 	{
-	    /* Calculate distance to leaves */
+		/* Calculate distance to leaves */
 		distance = (double)box2df_distance_leaf_centroid(entry_box, &query_box);
 	}
 	else
 	{
-	    /* Calculate distance for internal nodes */
+		/* Calculate distance for internal nodes */
 		distance = (double)box2df_distance_node_centroid(entry_box, &query_box);
 	}
-
+#endif	
+	
 	PG_RETURN_FLOAT8(distance);
 }
 
@@ -2206,7 +2234,7 @@ Datum box2df_in(PG_FUNCTION_ARGS)
 PG_FUNCTION_INFO_V1(box2df_out);
 Datum box2df_out(PG_FUNCTION_ARGS)
 {
-	ereport(ERROR,(errcode(ERRCODE_FEATURE_NOT_SUPPORTED),
-	               errmsg("function box2df_out not implemented")));
-	PG_RETURN_POINTER(NULL);
+  BOX2DF *box = (BOX2DF *) PG_GETARG_POINTER(0);
+  char *result = box2df_to_string(box);
+  PG_RETURN_CSTRING(result);
 }
