@@ -1,5 +1,4 @@
 /**********************************************************************
- * $Id: gserialized_gist_nd.c 6519 2010-12-28 00:54:19Z pramsey $
  *
  * PostGIS - Spatial Types for PostgreSQL
  * Copyright 2009 Paul Ramsey <pramsey@cleverelephant.ca>
@@ -36,14 +35,17 @@
 #include "gserialized_gist.h"	     /* For utility functions. */
 #include "geography.h"
 
-/* Fall back to older finite() if necessary */ 
-#ifndef HAVE_ISFINITE 
-# ifdef HAVE_GNU_ISFINITE 
-#  define _GNU_SOURCE 
-# else 
-#  define isfinite finite 
-# endif 
-#endif 
+#include <assert.h>
+
+
+/* Fall back to older finite() if necessary */
+#ifndef HAVE_ISFINITE
+# ifdef HAVE_GNU_ISFINITE
+#  define _GNU_SOURCE
+# else
+#  define isfinite finite
+# endif
+#endif
 
 /*
 ** When is a node split not so good? If more than 90% of the entries
@@ -75,6 +77,8 @@ Datum gserialized_gist_penalty(PG_FUNCTION_ARGS);
 Datum gserialized_gist_picksplit(PG_FUNCTION_ARGS);
 Datum gserialized_gist_union(PG_FUNCTION_ARGS);
 Datum gserialized_gist_same(PG_FUNCTION_ARGS);
+Datum gserialized_gist_distance(PG_FUNCTION_ARGS);
+Datum gserialized_gist_geog_distance(PG_FUNCTION_ARGS);
 
 /*
 ** ND Operator prototypes
@@ -82,6 +86,7 @@ Datum gserialized_gist_same(PG_FUNCTION_ARGS);
 Datum gserialized_overlaps(PG_FUNCTION_ARGS);
 Datum gserialized_contains(PG_FUNCTION_ARGS);
 Datum gserialized_within(PG_FUNCTION_ARGS);
+Datum gserialized_distance_nd(PG_FUNCTION_ARGS);
 
 /*
 ** GIDX true/false test function type
@@ -456,6 +461,178 @@ gserialized_datum_predicate(Datum gs1, Datum gs2, gidx_predicate predicate)
 	return LW_FALSE;
 }
 
+/**
+* Calculate the centroid->centroid distance between the boxes.
+*/
+#if POSTGIS_PGSQL_VERSION < 95
+static double gidx_distance_leaf_centroid(const GIDX *a, const GIDX *b)
+{
+  int ndims, i;
+  double sum = 0;
+
+  /* Base computation on least available dimensions */
+  ndims = Min(GIDX_NDIMS(b), GIDX_NDIMS(a));
+  for ( i = 0; i < ndims; ++i )
+  {
+    double ca, cb, d;
+    double amin = GIDX_GET_MIN(a,i);
+    double amax = GIDX_GET_MAX(a,i);
+    double bmin = GIDX_GET_MIN(b,i);
+    double bmax = GIDX_GET_MAX(b,i);
+    ca = amin + ( ( amax - amin ) / 2.0 );
+    cb = bmin + ( ( bmax - bmin ) / 2.0 );
+    d = ca - cb;
+    if ( ! isfinite(d) )
+    {
+      /* Can happen if a dimension was padded with FLT_MAX,
+       * effectively meaning "infinite range". In that case
+       * we take that dimension as adding 0 to the total
+       * distance.
+       */
+      continue;
+    }
+    sum += d * d;
+/*
+    POSTGIS_DEBUGF(3, " centroid of A for dimension %d is %g", i, ca);
+    POSTGIS_DEBUGF(3, " centroid of B for dimension %d is %g", i, cb);
+    POSTGIS_DEBUGF(3, " distance on dimension %d is %g, squared as %g, grows sum to %g", i, d, d*d, sum);
+*/
+  }
+  return sqrt(sum);
+}
+#endif
+
+/**
+* Calculate the box->box distance.
+*/
+static double gidx_distance(const GIDX *a, const GIDX *b, int m_is_time)
+{
+  int ndims, i;
+  double sum = 0;
+
+  /* Base computation on least available dimensions */
+  ndims = Min(GIDX_NDIMS(b), GIDX_NDIMS(a));
+  for ( i = 0; i < ndims; ++i )
+  {
+    double d;
+    double amin = GIDX_GET_MIN(a,i);
+    double amax = GIDX_GET_MAX(a,i);
+    double bmin = GIDX_GET_MIN(b,i);
+    double bmax = GIDX_GET_MAX(b,i);
+    POSTGIS_DEBUGF(3, "A %g - %g", amin, amax);
+    POSTGIS_DEBUGF(3, "B %g - %g", bmin, bmax);
+
+    if ( ( amin <= bmax && amax >= bmin ) )
+    {
+      /* overlaps */
+      d = 0;
+    }
+    else if ( i == 4 && m_is_time )
+    {
+      return FLT_MAX;
+    }
+    else if ( bmax < amin )
+    {
+      /* is "left" */
+      d = amin - bmax;
+    }
+    else
+    {
+      /* is "right" */
+      assert( bmin > amax );
+      d = bmin - amax;
+    }
+    if ( ! isfinite(d) )
+    {
+      /* Can happen if coordinates are corrupted/NaN */
+      continue;
+    }
+    sum += d * d;
+    POSTGIS_DEBUGF(3, "dist %g, squared %g, grows sum to %g", d, d*d, sum);
+  }
+  return sqrt(sum);
+}
+
+#if POSTGIS_PGSQL_VERSION < 95
+static double gidx_distance_node_centroid(const GIDX *node, const GIDX *query)
+{
+	int i;
+	double sum = 0;
+
+	/* Base computation on least available dimensions */
+	int ndims = Min(GIDX_NDIMS(node), GIDX_NDIMS(query));
+
+	for ( i = 0; i < ndims; ++i )
+	{
+		double d;
+		double amin = GIDX_GET_MIN(query,i);
+		double amax = GIDX_GET_MAX(query,i);
+		double bmin = GIDX_GET_MIN(node,i);
+		double bmax = GIDX_GET_MAX(node,i);
+		double ca = amin + ( ( amax - amin ) / 2.0 );
+
+		if ( ( ca <= bmax && ca >= bmin ) )
+		{
+			/* overlaps */
+			d = 0;
+		}
+		else if ( bmax < ca )
+		{
+			/* is "left" */
+			d = ca - bmax;
+		}
+		else
+		{
+			/* is "right" */
+			assert( bmin > ca );
+			d = bmin - ca;
+		}
+		if ( ! isfinite(d) )
+		{
+			/* Can happen if coordinates are corrupted/NaN */
+			continue;
+		}
+		sum += d * d;
+		POSTGIS_DEBUGF(3, "dist %g, squared %g, grows sum to %g", d, d*d, sum);
+	}
+	return sqrt(sum);
+}
+#else /* POSTGIS_PGSQL_VERSION >= 95 */
+static double gidx_distance_m(const GIDX *a, const GIDX *b)
+{
+	int mdim_a, mdim_b;
+	double d, amin, amax, bmin, bmax;
+
+	/* Base computation on least available dimensions */
+	mdim_a = GIDX_NDIMS(a) - 1;
+	mdim_b = GIDX_NDIMS(b) - 1;
+
+	amin = GIDX_GET_MIN(a,mdim_a);
+	amax = GIDX_GET_MAX(a,mdim_a);
+	bmin = GIDX_GET_MIN(b,mdim_b);
+	bmax = GIDX_GET_MAX(b,mdim_b);
+
+    if ( ( amin <= bmax && amax >= bmin ) )
+    {
+      /* overlaps */
+      d = 0;
+    }
+    else if ( bmax < amin )
+    {
+      /* is "left" */
+      d = amin - bmax;
+    }
+    else
+    {
+      /* is "right" */
+      assert( bmin > amax );
+      d = bmin - amax;
+    }
+	
+	return d;
+}
+#endif /* POSTGIS_PGSQL_VERSION >= 96 */
+
 /**
 * Return a #GSERIALIZED with an expanded bounding box.
 */
@@ -482,6 +659,123 @@ gserialized_expand(GSERIALIZED *g, double distance)
 * GiST N-D Index Operator Functions
 */
 
+/* 
+* Do centroid to centroid n-d distance if you don't have 
+* re-check available (PgSQL 9.5+), do "real" n-d distance
+* if you do
+*/
+PG_FUNCTION_INFO_V1(gserialized_distance_nd);
+Datum gserialized_distance_nd(PG_FUNCTION_ARGS)
+{
+	char b1mem[GIDX_MAX_SIZE];
+	GIDX *b1 = (GIDX*)b1mem;
+	char b2mem[GIDX_MAX_SIZE];
+	GIDX *b2 = (GIDX*)b2mem;
+
+#if POSTGIS_PGSQL_VERSION < 95
+
+	/* Centroid-to-centroid distance */
+	Datum gs1 = PG_GETARG_DATUM(0);
+	Datum gs2 = PG_GETARG_DATUM(1);
+	double box_distance = FLT_MAX;
+
+	/* Must be able to build box for each argument (ie, not empty geometry). */
+	if ( (gserialized_datum_get_gidx_p(gs1, b1) == LW_SUCCESS) &&
+	     (gserialized_datum_get_gidx_p(gs2, b2) == LW_SUCCESS) )
+	{
+		box_distance = gidx_distance_leaf_centroid(b1, b2);
+		POSTGIS_DEBUGF(3, "got boxes %s and %s", gidx_to_string(b1), gidx_to_string(b2));
+	}
+	PG_RETURN_FLOAT8(box_distance);
+
+#else /* POSTGIS_PGSQL_VERSION >= 96 */
+
+	/* Feature-to-feature distance */
+	GSERIALIZED *geom1 = PG_GETARG_GSERIALIZED_P(0);
+	GSERIALIZED *geom2 = PG_GETARG_GSERIALIZED_P(1);
+	LWGEOM *lw1 = lwgeom_from_gserialized(geom1);
+	LWGEOM *lw2 = lwgeom_from_gserialized(geom2);
+	LWGEOM *closest;
+	double distance;
+
+
+	/* Find an exact shortest line w/ the dimensions we support */
+	if ( lwgeom_has_z(lw1) && lwgeom_has_z(lw2) )
+	{
+		closest = lwgeom_closest_line_3d(lw1, lw2);
+		distance = lwgeom_length(closest);
+	}
+	else
+	{
+		closest = lwgeom_closest_line(lw1, lw2);
+		distance = lwgeom_length_2d(closest);
+	}
+
+	/* Un-sqrt the distance so we can add extra terms */
+	distance = distance*distance;
+
+	/* Can only add the M term if both objects have M */
+	if ( lwgeom_has_m(lw1) && lwgeom_has_m(lw2) )
+	{
+		double m1, m2;
+		int usebox = false;
+		
+		if ( lwgeom_get_type(lw1) == POINTTYPE )
+		{
+			POINT4D p;
+			lwpoint_getPoint4d_p((LWPOINT*)lw1, &p);
+			m1 = p.m;
+		}
+		else if ( lwgeom_get_type(lw1) == LINETYPE )
+		{
+			LWPOINT *lwp1 = lwline_get_lwpoint(lwgeom_as_lwline(closest), 0);
+			m1 = lwgeom_interpolate_point(lw1, lwp1);
+			lwpoint_free(lwp1);
+		}
+		else
+		{
+			usebox = true;
+		}
+		
+		if ( lwgeom_get_type(lw2) == POINTTYPE )
+		{
+			POINT4D p;
+			lwpoint_getPoint4d_p((LWPOINT*)lw2, &p);
+			m2 = p.m;
+		}
+		else if ( lwgeom_get_type(lw2) == LINETYPE )
+		{
+			LWPOINT *lwp2 = lwline_get_lwpoint(lwgeom_as_lwline(closest), 1);
+			m1 = lwgeom_interpolate_point(lw2, lwp2);
+			lwpoint_free(lwp2);
+		}
+		else 
+		{
+			usebox = true;
+		}
+
+		if ( usebox )
+		{
+			double d;
+			gserialized_get_gidx_p(geom1, b1);
+			gserialized_get_gidx_p(geom2, b2);
+			d = gidx_distance_m(b1, b2);
+			distance += d*d;
+		}
+		else
+		{
+			distance += (m2-m1)*(m2-m1);
+		}
+	}
+
+	lwgeom_free(closest);
+
+	PG_FREE_IF_COPY(geom1, 0);
+	PG_FREE_IF_COPY(geom2, 1);
+	PG_RETURN_FLOAT8(sqrt(distance));
+#endif /* POSTGIS_PGSQL_VERSION >= 96 */
+}
+
 /*
 ** '~' and operator function. Based on two serialized return true if
 ** the first is contained by the second.
@@ -596,8 +890,8 @@ Datum gserialized_gist_compress(PG_FUNCTION_ARGS)
 	 * If not, use the "unknown" GIDX as a key */
 	for ( i = 0; i < GIDX_NDIMS(bbox_out); i++ )
 	{
-		if ( ! isfinite(GIDX_GET_MAX(bbox_out, i)) ||
-		     ! isfinite(GIDX_GET_MIN(bbox_out, i)) )
+		if ( ! isfinite(GIDX_GET_MAX(bbox_out, i))
+		     || ! isfinite(GIDX_GET_MIN(bbox_out, i)) )
 		{
 			gidx_set_unknown(bbox_out);
 			gistentryinit(*entry_out, 
@@ -842,8 +1136,6 @@ Datum gserialized_gist_union(PG_FUNCTION_ARGS)
 
 }
 
-
-
 /*
 ** GiST support function. Test equality of keys.
 */
@@ -863,6 +1155,141 @@ Datum gserialized_gist_same(PG_FUNCTION_ARGS)
 
 
 
+
+PG_FUNCTION_INFO_V1(gserialized_gist_geog_distance);
+Datum gserialized_gist_geog_distance(PG_FUNCTION_ARGS)
+{
+	GISTENTRY *entry = (GISTENTRY*) PG_GETARG_POINTER(0);
+	Datum query_datum = PG_GETARG_DATUM(1);
+	StrategyNumber strategy = (StrategyNumber) PG_GETARG_UINT16(2);
+#if POSTGIS_PGSQL_VERSION >= 95
+	bool *recheck = (bool *) PG_GETARG_POINTER(4);
+#endif
+	char query_box_mem[GIDX_MAX_SIZE];
+	GIDX *query_box = (GIDX*)query_box_mem;
+	GIDX *entry_box;
+	double distance;
+
+	POSTGIS_DEBUGF(3, "[GIST] '%s' function called", __func__);
+ 
+	/* We are using '13' as the gist geography distance <-> strategy number */
+	if ( strategy != 13  ) 
+	{
+		elog(ERROR, "unrecognized strategy number: %d", strategy);
+		PG_RETURN_FLOAT8(FLT_MAX);
+	}
+
+	/* Null box should never make this far. */
+	if ( gserialized_datum_get_gidx_p(query_datum, query_box) == LW_FAILURE )
+	{
+		POSTGIS_DEBUG(2, "[GIST] null query_gbox_index!");
+		PG_RETURN_FLOAT8(FLT_MAX);
+	}
+
+#if POSTGIS_PGSQL_VERSION >= 95
+	/* When we hit leaf nodes, it's time to turn on recheck */
+	if (GIST_LEAF(entry))
+	{
+		*recheck = true;
+	}
+#endif
+			
+	/* Get the entry box */
+	entry_box = (GIDX*)DatumGetPointer(entry->key);
+
+	/* Return distances from key-based tests should always be */
+	/* the minimum possible distance, box-to-box */
+	/* We scale up to "world units" so that the box-to-box distances */
+	/* compare reasonably with the over-the-spheroid distances that */
+	/* the recheck process will turn up */
+	distance = WGS84_RADIUS * gidx_distance(entry_box, query_box, 0);
+	POSTGIS_DEBUGF(2, "[GIST] '%s' got distance %g", __func__, distance);
+
+	PG_RETURN_FLOAT8(distance);
+}
+
+
+/*
+** GiST support function.
+** Take in a query and an entry and return the "distance" between them.
+**
+** Given an index entry p and a query value q, this function determines the
+** index entry's "distance" from the query value. This function must be
+** supplied if the operator class contains any ordering operators. A query
+** using the ordering operator will be implemented by returning index entries
+** with the smallest "distance" values first, so the results must be consistent
+** with the operator's semantics. For a leaf index entry the result just
+** represents the distance to the index entry; for an internal tree node, the
+** result must be the smallest distance that any child entry could have.
+**
+** Strategy 13 is centroid-based distance tests <<->>
+** Strategy 20 is cpa based distance tests |=|
+*/
+PG_FUNCTION_INFO_V1(gserialized_gist_distance);
+Datum gserialized_gist_distance(PG_FUNCTION_ARGS)
+{
+	GISTENTRY *entry = (GISTENTRY*) PG_GETARG_POINTER(0);
+	StrategyNumber strategy = (StrategyNumber) PG_GETARG_UINT16(2);
+	char query_box_mem[GIDX_MAX_SIZE];
+	GIDX *query_box = (GIDX*)query_box_mem;
+	GIDX *entry_box;
+#if POSTGIS_PGSQL_VERSION >= 95
+	bool *recheck = (bool *) PG_GETARG_POINTER(4);
+#endif
+	
+	double distance;
+
+	POSTGIS_DEBUG(4, "[GIST] 'distance' function called");
+ 
+	/* Strategy 13 is <<->> */
+	/* Strategy 20 is |=| */
+	if ( strategy != 13 && strategy != 20 ) {
+		elog(ERROR, "unrecognized strategy number: %d", strategy);
+		PG_RETURN_FLOAT8(FLT_MAX);
+	}
+
+	/* Null box should never make this far. */
+	if ( gserialized_datum_get_gidx_p(PG_GETARG_DATUM(1), query_box) == LW_FAILURE )
+	{
+		POSTGIS_DEBUG(4, "[GIST] null query_gbox_index!");
+		PG_RETURN_FLOAT8(FLT_MAX);
+	}
+
+	/* Get the entry box */
+	entry_box = (GIDX*)DatumGetPointer(entry->key);
+
+#if POSTGIS_PGSQL_VERSION >= 95
+
+	/* Strategy 20 is |=| */
+	distance = gidx_distance(entry_box, query_box, strategy == 20);
+
+	/* Treat leaf node tests different from internal nodes */
+	if (GIST_LEAF(entry))
+		*recheck = true;
+#else
+
+	if ( strategy == 20 )
+	{
+		elog(ERROR, "You need PostgreSQL 9.5.0 or higher in order to use |=| with index");
+		PG_RETURN_FLOAT8(FLT_MAX);
+	}
+
+	/* Treat leaf node tests different from internal nodes */
+	if (GIST_LEAF(entry))
+	{
+		/* Calculate distance to leaves */
+		distance = (double)gidx_distance_leaf_centroid(entry_box, query_box);
+	}
+	else
+	{
+		/* Calculate distance for internal nodes */
+		distance = (double)gidx_distance_node_centroid(entry_box, query_box);
+	}
+#endif
+	PG_RETURN_FLOAT8(distance);
+}
+
+
 /*
 ** Utility function to add entries to the axis partition lists in the
 ** picksplit function.
@@ -1285,7 +1712,7 @@ Datum gidx_in(PG_FUNCTION_ARGS)
 PG_FUNCTION_INFO_V1(gidx_out); 
 Datum gidx_out(PG_FUNCTION_ARGS)
 {
-	ereport(ERROR,(errcode(ERRCODE_FEATURE_NOT_SUPPORTED),
-	               errmsg("function gidx_out not implemented")));
-	PG_RETURN_POINTER(NULL);
+  GIDX *box = (GIDX *) PG_GETARG_POINTER(0);
+  char *result = gidx_to_string(box);
+  PG_RETURN_CSTRING(result);
 }
