@@ -1,5 +1,4 @@
 /**********************************************************************
- * $Id: lwgeom_accum.c 13525 2015-05-19 17:38:43Z pramsey $
  *
  * PostGIS - Spatial Types for PostgreSQL
  * http://postgis.net
@@ -14,22 +13,28 @@
 #include "fmgr.h"
 #include "funcapi.h"
 #include "access/tupmacs.h"
+#include "utils/datum.h"
 #include "utils/array.h"
 #include "utils/lsyscache.h"
 
 #include "../postgis_config.h"
 
 #include "liblwgeom.h"
+#include "lwgeom_geos.h"
 #include "lwgeom_pg.h"
+#include "lwgeom_transform.h"
 
 /* Local prototypes */
 Datum PGISDirectFunctionCall1(PGFunction func, Datum arg1);
+Datum PGISDirectFunctionCall2(PGFunction func, Datum arg1, Datum arg2);
 Datum pgis_geometry_accum_transfn(PG_FUNCTION_ARGS);
 Datum pgis_geometry_accum_finalfn(PG_FUNCTION_ARGS);
 Datum pgis_geometry_union_finalfn(PG_FUNCTION_ARGS);
 Datum pgis_geometry_collect_finalfn(PG_FUNCTION_ARGS);
 Datum pgis_geometry_polygonize_finalfn(PG_FUNCTION_ARGS);
 Datum pgis_geometry_makeline_finalfn(PG_FUNCTION_ARGS);
+Datum pgis_geometry_clusterintersecting_finalfn(PG_FUNCTION_ARGS);
+Datum pgis_geometry_clusterwithin_finalfn(PG_FUNCTION_ARGS);
 Datum pgis_abs_in(PG_FUNCTION_ARGS);
 Datum pgis_abs_out(PG_FUNCTION_ARGS);
 
@@ -37,6 +42,8 @@ Datum pgis_abs_out(PG_FUNCTION_ARGS);
 Datum pgis_union_geometry_array(PG_FUNCTION_ARGS);
 Datum LWGEOM_collect_garray(PG_FUNCTION_ARGS);
 Datum polygonize_garray(PG_FUNCTION_ARGS);
+Datum clusterintersecting_garray(PG_FUNCTION_ARGS);
+Datum cluster_within_distance_garray(PG_FUNCTION_ARGS);
 Datum LWGEOM_makeline_garray(PG_FUNCTION_ARGS);
 
 
@@ -55,15 +62,19 @@ Datum LWGEOM_makeline_garray(PG_FUNCTION_ARGS);
 /**
 ** To pass the internal ArrayBuildState pointer between the
 ** transfn and finalfn we need to wrap it into a custom type first,
-** the pgis_abs type in our case.
+** the pgis_abs type in our case.  The extra "data" member can optionally
+** be used to pass an additional constant argument to a finalizer function.
 */
 
 typedef struct
 {
 	ArrayBuildState *a;
+	Datum data;
 }
 pgis_abs;
 
+
+
 /**
 ** We're never going to use this type externally so the in/out
 ** functions are dummies.
@@ -73,7 +84,7 @@ Datum
 pgis_abs_in(PG_FUNCTION_ARGS)
 {
 	ereport(ERROR,(errcode(ERRCODE_FEATURE_NOT_SUPPORTED),
-	               errmsg("function pgis_abs_in not implemented")));
+	               errmsg("function %s not implemented", __func__)));
 	PG_RETURN_POINTER(NULL);
 }
 PG_FUNCTION_INFO_V1(pgis_abs_out);
@@ -81,7 +92,7 @@ Datum
 pgis_abs_out(PG_FUNCTION_ARGS)
 {
 	ereport(ERROR,(errcode(ERRCODE_FEATURE_NOT_SUPPORTED),
-	               errmsg("function pgis_abs_out not implemented")));
+	               errmsg("function %s not implemented", __func__)));
 	PG_RETURN_POINTER(NULL);
 }
 
@@ -116,6 +127,18 @@ pgis_geometry_accum_transfn(PG_FUNCTION_ARGS)
 	{
 		p = (pgis_abs*) palloc(sizeof(pgis_abs));
 		p->a = NULL;
+		p->data = (Datum) NULL;
+
+		if (PG_NARGS() == 3)
+		{
+			Datum argument = PG_GETARG_DATUM(2);
+			Oid dataOid = get_fn_expr_argtype(fcinfo->flinfo, 2);
+			MemoryContext old = MemoryContextSwitchTo(aggcontext);
+
+			p->data = datumCopy(argument, get_typbyval(dataOid), get_typlen(dataOid));
+
+			MemoryContextSwitchTo(old);
+		}
 	}
 	else
 	{
@@ -133,6 +156,8 @@ pgis_geometry_accum_transfn(PG_FUNCTION_ARGS)
 	PG_RETURN_POINTER(p);
 }
 
+
+
 Datum pgis_accum_finalfn(pgis_abs *p, MemoryContext mctx, FunctionCallInfo fcinfo);
 
 /**
@@ -182,7 +207,7 @@ pgis_geometry_accum_finalfn(PG_FUNCTION_ARGS)
 }
 
 /**
-* The "accum" final function passes the geometry[] to a union
+* The "union" final function passes the geometry[] to a union
 * conversion before returning the result.
 */
 PG_FUNCTION_INFO_V1(pgis_geometry_union_finalfn);
@@ -231,6 +256,7 @@ pgis_geometry_collect_finalfn(PG_FUNCTION_ARGS)
 	PG_RETURN_DATUM(result);
 }
 
+
 /**
 * The "polygonize" final function passes the geometry[] to a polygonization
 * before returning the result.
@@ -279,6 +305,61 @@ pgis_geometry_makeline_finalfn(PG_FUNCTION_ARGS)
 	PG_RETURN_DATUM(result);
 }
 
+/**
+ * The "clusterintersecting" final function passes the geometry[] to a
+ * clustering function before returning the result.
+*/
+PG_FUNCTION_INFO_V1(pgis_geometry_clusterintersecting_finalfn);
+Datum
+pgis_geometry_clusterintersecting_finalfn(PG_FUNCTION_ARGS)
+{
+	pgis_abs *p;
+	Datum result = 0;
+	Datum geometry_array = 0;
+
+	if (PG_ARGISNULL(0))
+		PG_RETURN_NULL();
+
+	p = (pgis_abs*) PG_GETARG_POINTER(0);
+	geometry_array = pgis_accum_finalfn(p, CurrentMemoryContext, fcinfo);
+	result = PGISDirectFunctionCall1( clusterintersecting_garray, geometry_array );
+	if (!result)
+		PG_RETURN_NULL();
+
+	PG_RETURN_DATUM(result);
+}
+
+/**
+ * The "cluster_within_distance" final function passes the geometry[] to a
+ * clustering function before returning the result.
+*/
+PG_FUNCTION_INFO_V1(pgis_geometry_clusterwithin_finalfn);
+Datum
+pgis_geometry_clusterwithin_finalfn(PG_FUNCTION_ARGS)
+{
+	pgis_abs *p;
+	Datum result = 0;
+	Datum geometry_array = 0;
+
+	if (PG_ARGISNULL(0))
+		PG_RETURN_NULL();
+
+	p = (pgis_abs*) PG_GETARG_POINTER(0);
+
+	if (!p->data)
+	{
+		elog(ERROR, "Tolerance not defined");
+		PG_RETURN_NULL();
+	}
+
+	geometry_array = pgis_accum_finalfn(p, CurrentMemoryContext, fcinfo);
+	result = PGISDirectFunctionCall2( cluster_within_distance_garray, geometry_array, p->data);
+	if (!result)
+		PG_RETURN_NULL();
+
+	PG_RETURN_DATUM(result);
+}
+
 /**
 * A modified version of PostgreSQL's DirectFunctionCall1 which allows NULL results; this
 * is required for aggregates that return NULL.
@@ -308,3 +389,35 @@ PGISDirectFunctionCall1(PGFunction func, Datum arg1)
 
 	return result;
 }
+
+/**
+* A modified version of PostgreSQL's DirectFunctionCall2 which allows NULL results; this
+* is required for aggregates that return NULL.
+*/
+Datum
+PGISDirectFunctionCall2(PGFunction func, Datum arg1, Datum arg2)
+{
+	FunctionCallInfoData fcinfo;
+	Datum           result;
+
+#if POSTGIS_PGSQL_VERSION > 90
+
+	InitFunctionCallInfoData(fcinfo, NULL, 1, InvalidOid, NULL, NULL);
+#else
+
+	InitFunctionCallInfoData(fcinfo, NULL, 1, NULL, NULL);
+#endif
+
+	fcinfo.arg[0] = arg1;
+	fcinfo.arg[1] = arg2;
+	fcinfo.argnull[0] = false;
+	fcinfo.argnull[1] = false;
+
+	result = (*func) (&fcinfo);
+
+	/* Check for null result, returning a "NULL" Datum if indicated */
+	if (fcinfo.isnull)
+		return (Datum) 0;
+
+	return result;
+}
