@@ -1,5 +1,4 @@
 /**********************************************************************
- * $Id:$
  *
  * PostGIS - Export functions for PostgreSQL/PostGIS
  * Copyright 2009-2011 Olivier Courtin <olivier.courtin@oslandia.com>
@@ -26,8 +25,10 @@
 Datum LWGEOM_asGML(PG_FUNCTION_ARGS);
 Datum LWGEOM_asKML(PG_FUNCTION_ARGS);
 Datum LWGEOM_asGeoJson(PG_FUNCTION_ARGS);
+Datum LWGEOM_asGeoJson_old(PG_FUNCTION_ARGS);
 Datum LWGEOM_asSVG(PG_FUNCTION_ARGS);
 Datum LWGEOM_asX3D(PG_FUNCTION_ARGS);
+Datum LWGEOM_asEncodedPolyline(PG_FUNCTION_ARGS);
 
 /*
  * Retrieve an SRS from a given SRID
@@ -112,7 +113,6 @@ int getSRIDbySRS(const char* srs)
 		SPI_finish();
 		return 0;
 	}
-
 	sprintf(query, 
 		"SELECT srid "
 		"FROM spatial_ref_sys, "
@@ -197,7 +197,7 @@ Datum LWGEOM_asGML(PG_FUNCTION_ARGS)
 
 	/* Get the geometry */
 	if ( PG_ARGISNULL(1) ) PG_RETURN_NULL();
-	geom = (GSERIALIZED *)PG_DETOAST_DATUM(PG_GETARG_DATUM(1));
+	geom = PG_GETARG_GSERIALIZED_P(1);
 
 	/* Retrieve precision if any (default is max) */
 	if (PG_NARGS() >2 && !PG_ARGISNULL(2))
@@ -312,7 +312,7 @@ Datum LWGEOM_asKML(PG_FUNCTION_ARGS)
 
 	/* Get the geometry */
 	if ( PG_ARGISNULL(1) ) PG_RETURN_NULL();
-	geom = (GSERIALIZED *)PG_DETOAST_DATUM(PG_GETARG_DATUM(1));
+	geom = PG_GETARG_GSERIALIZED_P(1);
 
 	/* Retrieve precision if any (default is max) */
 	if (PG_NARGS() >2 && !PG_ARGISNULL(2))
@@ -360,6 +360,29 @@ Datum LWGEOM_asKML(PG_FUNCTION_ARGS)
 }
 
 
+/**
+ * Encode Feature in GeoJson (Old C Signature)
+ * ST_AsGeoJSON(version, geom, precision, options)
+ * why was this written with a version param when there 
+ * is only one version?
+ */
+PG_FUNCTION_INFO_V1(LWGEOM_asGeoJson_old);
+Datum LWGEOM_asGeoJson_old(PG_FUNCTION_ARGS)
+{
+	switch( PG_NARGS() )
+	{
+	case 2: 
+		return DirectFunctionCall1(LWGEOM_asGeoJson, PG_GETARG_DATUM(1));
+	case 3:
+		return DirectFunctionCall2(LWGEOM_asGeoJson, PG_GETARG_DATUM(1), PG_GETARG_DATUM(2));
+	case 4:
+		return DirectFunctionCall3(LWGEOM_asGeoJson, PG_GETARG_DATUM(1), PG_GETARG_DATUM(2), PG_GETARG_DATUM(3));
+	default:
+		elog(ERROR, "bad call in %s", __func__);
+	}
+	PG_RETURN_NULL();
+}
+
 /**
  * Encode Feature in GeoJson
  */
@@ -370,32 +393,24 @@ Datum LWGEOM_asGeoJson(PG_FUNCTION_ARGS)
 	LWGEOM *lwgeom;
 	char *geojson;
 	text *result;
-	int srid;
-	int version;
-	int option = 0;
 	int has_bbox = 0;
 	int precision = DBL_DIG;
-	char * srs = NULL;
-
-	/* Get the version */
-	version = PG_GETARG_INT32(0);
-	if ( version != 1)
-	{
-		elog(ERROR, "Only GeoJSON 1 is supported");
-		PG_RETURN_NULL();
-	}
+	char *srs = NULL;
 
 	/* Get the geometry */
-	if (PG_ARGISNULL(1) ) PG_RETURN_NULL();
-	geom = (GSERIALIZED *)PG_DETOAST_DATUM(PG_GETARG_DATUM(1));
+	if ( PG_ARGISNULL(0) ) 
+		PG_RETURN_NULL();
+	
+	geom = PG_GETARG_GSERIALIZED_P(0);
 
 	/* Retrieve precision if any (default is max) */
-	if (PG_NARGS() >2 && !PG_ARGISNULL(2))
+	if ( PG_NARGS() > 1 && !PG_ARGISNULL(1) )
 	{
-		precision = PG_GETARG_INT32(2);
+		precision = PG_GETARG_INT32(1);
 		if ( precision > DBL_DIG )
 			precision = DBL_DIG;
-		else if ( precision < 0 ) precision = 0;
+		else if ( precision < 0 ) 
+			precision = 0;
 	}
 
 	/* Retrieve output option
@@ -404,39 +419,45 @@ Datum LWGEOM_asGeoJson(PG_FUNCTION_ARGS)
 	 * 2 = short crs
 	 * 4 = long crs
 	 */
-	if (PG_NARGS() >3 && !PG_ARGISNULL(3))
-		option = PG_GETARG_INT32(3);
-
-	if (option & 2 || option & 4)
+	if ( PG_NARGS() > 2 && !PG_ARGISNULL(2) )
 	{
-		srid = gserialized_get_srid(geom);
-		if ( srid != SRID_UNKNOWN )
+		int option = PG_GETARG_INT32(2);
+
+		if ( option & 2 || option & 4 )
 		{
-			if (option & 2) srs = getSRSbySRID(srid, true);
-			if (option & 4) srs = getSRSbySRID(srid, false);
-			if (!srs)
+			int srid = gserialized_get_srid(geom);
+			if ( srid != SRID_UNKNOWN )
 			{
-				elog(	ERROR,
-				      "SRID %i unknown in spatial_ref_sys table",
-				      srid);
-				PG_RETURN_NULL();
+				if ( option & 2 )
+					srs = getSRSbySRID(srid, true);
+			
+				if ( option & 4 )
+					srs = getSRSbySRID(srid, false);
+			
+				if ( !srs )
+				{
+					elog(ERROR,
+					      "SRID %i unknown in spatial_ref_sys table",
+					      srid);
+					PG_RETURN_NULL();
+				}
 			}
 		}
-	}
 
-	if (option & 1) has_bbox = 1;
+		if (option & 1) 
+			has_bbox = 1;
+	}
 
 	lwgeom = lwgeom_from_gserialized(geom);
 	geojson = lwgeom_to_geojson(lwgeom, srs, precision, has_bbox);
 	lwgeom_free(lwgeom);
 
-	PG_FREE_IF_COPY(geom, 1);
 	if (srs) pfree(srs);
 
 	result = cstring2text(geojson);
-
 	lwfree(geojson);
 
+	PG_FREE_IF_COPY(geom, 0);
 	PG_RETURN_TEXT_P(result);
 }
 
@@ -456,7 +477,7 @@ Datum LWGEOM_asSVG(PG_FUNCTION_ARGS)
 
 	if ( PG_ARGISNULL(0) ) PG_RETURN_NULL();
 
-	geom = (GSERIALIZED *)PG_DETOAST_DATUM(PG_GETARG_DATUM(0));
+	geom = PG_GETARG_GSERIALIZED_P(0);
 
 	/* check for relative path notation */
 	if ( PG_NARGS() > 1 && ! PG_ARGISNULL(1) )
@@ -511,7 +532,7 @@ Datum LWGEOM_asX3D(PG_FUNCTION_ARGS)
 
 	/* Get the geometry */
 	if ( PG_ARGISNULL(1) ) PG_RETURN_NULL();
-	geom = (GSERIALIZED *)PG_DETOAST_DATUM(PG_GETARG_DATUM(1));
+	geom = PG_GETARG_GSERIALIZED_P(1);
 
 	/* Retrieve precision if any (default is max) */
 	if (PG_NARGS() >2 && !PG_ARGISNULL(2))
@@ -526,6 +547,8 @@ Datum LWGEOM_asX3D(PG_FUNCTION_ARGS)
 	/* retrieve option */
 	if (PG_NARGS() >3 && !PG_ARGISNULL(3))
 		option = PG_GETARG_INT32(3);
+		
+	
 
 	/* retrieve defid */
 	if (PG_NARGS() >4 && !PG_ARGISNULL(4))
@@ -548,12 +571,22 @@ Datum LWGEOM_asX3D(PG_FUNCTION_ARGS)
 		}
 	}
 
+	lwgeom = lwgeom_from_gserialized(geom);
 	srid = gserialized_get_srid(geom);
 	if (srid == SRID_UNKNOWN)      srs = NULL;
 	else if (option & 1) srs = getSRSbySRID(srid, false);
 	else                 srs = getSRSbySRID(srid, true);
-
-	lwgeom = lwgeom_from_gserialized(geom);
+	
+	if (option & LW_X3D_USE_GEOCOORDS) {
+		if (srid != 4326) {
+			PG_FREE_IF_COPY(geom, 0);
+			/** TODO: we need to support UTM and other coordinate systems supported by X3D eventually 
+			http://www.web3d.org/documents/specifications/19775-1/V3.2/Part01/components/geodata.html#t-earthgeoids **/
+			elog(ERROR, "Only SRID 4326 is supported for geocoordinates.");
+			PG_RETURN_NULL();
+		}
+	}
+	
 
 	x3d = lwgeom_to_x3d3(lwgeom, srs, precision,option, defid);
 
@@ -565,3 +598,41 @@ Datum LWGEOM_asX3D(PG_FUNCTION_ARGS)
 
 	PG_RETURN_TEXT_P(result);
 }
+
+/**
+ * Encode feature as Encoded Polyline
+ */
+PG_FUNCTION_INFO_V1(LWGEOM_asEncodedPolyline);
+Datum LWGEOM_asEncodedPolyline(PG_FUNCTION_ARGS)
+{
+	GSERIALIZED *geom;
+	LWGEOM *lwgeom;
+	char *encodedpolyline;
+	int precision = 5;
+	text *result;
+
+	if ( PG_ARGISNULL(0) ) PG_RETURN_NULL();
+
+	geom = PG_GETARG_GSERIALIZED_P(0);
+	if (gserialized_get_srid(geom) != 4326) {
+		PG_FREE_IF_COPY(geom, 0);
+		elog(ERROR, "Only SRID 4326 is supported.");
+		PG_RETURN_NULL();
+	}
+	lwgeom = lwgeom_from_gserialized(geom);
+	PG_FREE_IF_COPY(geom, 0);
+	
+	if (PG_NARGS() >2 && !PG_ARGISNULL(2))
+	{
+		precision = PG_GETARG_INT32(2);
+		if ( precision < 0 ) precision = 5;
+	}
+
+	encodedpolyline = lwgeom_to_encoded_polyline(lwgeom, precision);
+	lwgeom_free(lwgeom);
+
+  result = cstring2text(encodedpolyline);
+	lwfree(encodedpolyline);
+
+	PG_RETURN_TEXT_P(result);
+}
