@@ -1,5 +1,4 @@
 /**********************************************************************
- * $Id: lwgeom_geos.c 5258 2010-02-17 21:02:49Z strk $
  *
  * PostGIS - Spatial Types for PostgreSQL
  * http://postgis.net
@@ -42,7 +41,6 @@
 #include "../postgis_config.h"
 #include "lwgeom_geos.h"
 #include "liblwgeom.h"
-#include "liblwgeom_internal.h"
 #include "lwgeom_pg.h"
 
 #include <string.h>
@@ -62,7 +60,7 @@ Datum ST_MakeValid(PG_FUNCTION_ARGS)
 	GSERIALIZED *in, *out;
 	LWGEOM *lwgeom_in, *lwgeom_out;
 
-	in = (GSERIALIZED *)PG_DETOAST_DATUM(PG_GETARG_DATUM(0));
+	in = PG_GETARG_GSERIALIZED_P(0);
 	lwgeom_in = lwgeom_from_gserialized(in);
 
 	switch ( lwgeom_in->type )
@@ -77,7 +75,7 @@ Datum ST_MakeValid(PG_FUNCTION_ARGS)
 		break;
 
 	default:
-		lwerror("ST_MakeValid: unsupported geometry type %s",
+		lwpgerror("ST_MakeValid: unsupported geometry type %s",
 		        lwtype_name(lwgeom_in->type));
 		PG_RETURN_NULL();
 		break;
@@ -114,7 +112,7 @@ lwgeom_clean(LWGEOM* lwgeom_in)
 	/* Check dimensionality is the same as input */
 	if ( lwgeom_dimensionality(lwgeom_in) != lwgeom_dimensionality(lwgeom_out) )
 	{
-		lwnotice("lwgeom_clean: dimensional collapse (%d to %d)",
+		lwpgnotice("lwgeom_clean: dimensional collapse (%d to %d)",
 		         lwgeom_dimensionality(lwgeom_in), lwgeom_dimensionality(lwgeom_out));
 
 		return NULL;
@@ -124,7 +122,7 @@ lwgeom_clean(LWGEOM* lwgeom_in)
 	if ( lwgeom_out->type == COLLECTIONTYPE &&
 	        lwgeom_in->type != COLLECTIONTYPE )
 	{
-		lwnotice("lwgeom_clean: mixed-type output (%s) "
+		lwpgnotice("lwgeom_clean: mixed-type output (%s) "
 		         "from single-type input (%s)",
 		         lwtype_name(lwgeom_out->type),
 		         lwtype_name(lwgeom_in->type));
@@ -155,7 +153,7 @@ Datum ST_CleanGeometry(PG_FUNCTION_ARGS)
 	GSERIALIZED *in, *out;
 	LWGEOM *lwgeom_in, *lwgeom_out;
 
-	in = (GSERIALIZED *)PG_DETOAST_DATUM(PG_GETARG_DATUM(0));
+	in = PG_GETARG_GSERIALIZED_P(0);
 	lwgeom_in = lwgeom_from_gserialized(in);
 
 	/* Short-circuit: empty geometry are the cleanest ! */
