@@ -16,10 +16,9 @@
 #include "fmgr.h"
 #include "../liblwgeom/liblwgeom.h"
 
-#include "../postgis_config.h"
-
 #include "lwgeom_pg.h"
 #include "lwgeom_sfcgal.h"
+#include "../postgis_config.h"
 
 
 Datum postgis_sfcgal_version(PG_FUNCTION_ARGS);
@@ -33,14 +32,22 @@ Datum sfcgal_intersects(PG_FUNCTION_ARGS);
 Datum sfcgal_intersects3D(PG_FUNCTION_ARGS);
 Datum sfcgal_intersection(PG_FUNCTION_ARGS);
 Datum sfcgal_intersection3D(PG_FUNCTION_ARGS);
+Datum sfcgal_difference(PG_FUNCTION_ARGS);
+Datum sfcgal_difference3D(PG_FUNCTION_ARGS);
+Datum sfcgal_union(PG_FUNCTION_ARGS);
+Datum sfcgal_union3D(PG_FUNCTION_ARGS);
+Datum sfcgal_volume(PG_FUNCTION_ARGS);
 Datum sfcgal_extrude(PG_FUNCTION_ARGS);
 Datum sfcgal_straight_skeleton(PG_FUNCTION_ARGS);
+Datum sfcgal_approximate_medial_axis(PG_FUNCTION_ARGS);
 Datum sfcgal_is_planar(PG_FUNCTION_ARGS);
 Datum sfcgal_orientation(PG_FUNCTION_ARGS);
 Datum sfcgal_force_lhr(PG_FUNCTION_ARGS);
 Datum sfcgal_triangulate(PG_FUNCTION_ARGS);
 Datum sfcgal_tesselate(PG_FUNCTION_ARGS);
 Datum sfcgal_minkowski_sum(PG_FUNCTION_ARGS);
+Datum sfcgal_make_solid(PG_FUNCTION_ARGS);
+Datum sfcgal_is_solid(PG_FUNCTION_ARGS);
 
 
 GSERIALIZED *geometry_serialize(LWGEOM *lwgeom);
@@ -52,7 +59,7 @@ void sfcgal_postgis_init(void)
 {
     if ( ! __sfcgal_init ) {
 	sfcgal_init();
-	sfcgal_set_error_handlers((sfcgal_error_handler_t) lwnotice, (sfcgal_error_handler_t) lwerror);
+	sfcgal_set_error_handlers((sfcgal_error_handler_t) lwpgnotice, (sfcgal_error_handler_t) lwpgerror);
 	sfcgal_set_alloc_handlers(lwalloc, lwfree);
 	__sfcgal_init = 1;
     }
@@ -67,7 +74,7 @@ sfcgal_geometry_t* POSTGIS2SFCGALGeometry(GSERIALIZED *pglwgeom)
 
 	if (! lwgeom)
 	{
-		lwerror("POSTGIS2SFCGALGeometry: Unable to deserialize input");
+		lwpgerror("POSTGIS2SFCGALGeometry: Unable to deserialize input");
 	}
 	g = LWGEOM2SFCGAL(lwgeom);
 	lwgeom_free(lwgeom);
@@ -84,7 +91,7 @@ sfcgal_prepared_geometry_t* POSTGIS2SFCGALPreparedGeometry(GSERIALIZED *pglwgeom
 
 	if (!lwgeom)
 	{
-		lwerror("POSTGIS2SFCGALPreparedGeometry: Unable to deserialize input");
+		lwpgerror("POSTGIS2SFCGALPreparedGeometry: Unable to deserialize input");
 	}
 	g = LWGEOM2SFCGAL(lwgeom);
 
@@ -146,7 +153,7 @@ Datum sfcgal_area(PG_FUNCTION_ARGS)
 
 	sfcgal_postgis_init();
 
-	input = (GSERIALIZED*) PG_DETOAST_DATUM(PG_GETARG_DATUM(0));
+	input = PG_GETARG_GSERIALIZED_P(0);
 	geom = POSTGIS2SFCGALGeometry(input);
 
 	result = sfcgal_geometry_area(geom);
@@ -167,7 +174,7 @@ Datum sfcgal_area3D(PG_FUNCTION_ARGS)
 
 	sfcgal_postgis_init();
 
-	input = (GSERIALIZED*) PG_DETOAST_DATUM(PG_GETARG_DATUM(0));
+	input = PG_GETARG_GSERIALIZED_P(0);
 	geom = POSTGIS2SFCGALGeometry(input);
 
 	result = sfcgal_geometry_area_3d(geom);
@@ -188,7 +195,7 @@ Datum sfcgal_is_planar(PG_FUNCTION_ARGS)
 
 	sfcgal_postgis_init();
 
-	input = (GSERIALIZED*) PG_DETOAST_DATUM(PG_GETARG_DATUM(0));
+	input = PG_GETARG_GSERIALIZED_P(0);
 	geom = POSTGIS2SFCGALGeometry(input);
 
 	result = sfcgal_geometry_is_planar(geom);
@@ -209,7 +216,7 @@ Datum sfcgal_orientation(PG_FUNCTION_ARGS)
 
 	sfcgal_postgis_init();
 
-	input = (GSERIALIZED*) PG_DETOAST_DATUM(PG_GETARG_DATUM(0));
+	input = PG_GETARG_GSERIALIZED_P(0);
 	geom = POSTGIS2SFCGALGeometry(input);
 
 	result = sfcgal_geometry_orientation(geom);
@@ -230,8 +237,8 @@ Datum sfcgal_intersects(PG_FUNCTION_ARGS)
 
 	sfcgal_postgis_init();
 
-	input0 = (GSERIALIZED*) PG_DETOAST_DATUM(PG_GETARG_DATUM(0));
-	input1 = (GSERIALIZED*) PG_DETOAST_DATUM(PG_GETARG_DATUM(1));
+	input0 = PG_GETARG_GSERIALIZED_P(0);
+	input1 = PG_GETARG_GSERIALIZED_P(1);
 	geom0 = POSTGIS2SFCGALGeometry(input0);
 	PG_FREE_IF_COPY(input0, 0);
 	geom1 = POSTGIS2SFCGALGeometry(input1);
@@ -254,8 +261,8 @@ Datum sfcgal_intersects3D(PG_FUNCTION_ARGS)
 
 	sfcgal_postgis_init();
 
-	input0 = (GSERIALIZED*) PG_DETOAST_DATUM(PG_GETARG_DATUM(0));
-	input1 = (GSERIALIZED*) PG_DETOAST_DATUM(PG_GETARG_DATUM(1));
+	input0 = PG_GETARG_GSERIALIZED_P(0);
+	input1 = PG_GETARG_GSERIALIZED_P(1);
 	geom0 = POSTGIS2SFCGALGeometry(input0);
 	PG_FREE_IF_COPY(input0, 0);
 	geom1 = POSTGIS2SFCGALGeometry(input1);
@@ -278,8 +285,8 @@ Datum sfcgal_distance(PG_FUNCTION_ARGS)
 
 	sfcgal_postgis_init();
 
-	input0 = (GSERIALIZED*) PG_DETOAST_DATUM(PG_GETARG_DATUM(0));
-	input1 = (GSERIALIZED*) PG_DETOAST_DATUM(PG_GETARG_DATUM(1));
+	input0 = PG_GETARG_GSERIALIZED_P(0);
+	input1 = PG_GETARG_GSERIALIZED_P(1);
 	geom0 = POSTGIS2SFCGALGeometry(input0);
 	PG_FREE_IF_COPY(input0, 0);
 	geom1 = POSTGIS2SFCGALGeometry(input1);
@@ -302,8 +309,8 @@ Datum sfcgal_distance3D(PG_FUNCTION_ARGS)
 
 	sfcgal_postgis_init();
 
-	input0 = (GSERIALIZED*) PG_DETOAST_DATUM(PG_GETARG_DATUM(0));
-	input1 = (GSERIALIZED*) PG_DETOAST_DATUM(PG_GETARG_DATUM(1));
+	input0 = PG_GETARG_GSERIALIZED_P(0);
+	input1 = PG_GETARG_GSERIALIZED_P(1);
 	geom0 = POSTGIS2SFCGALGeometry(input0);
 	PG_FREE_IF_COPY(input0, 0);
 	geom1 = POSTGIS2SFCGALGeometry(input1);
@@ -327,7 +334,7 @@ Datum sfcgal_tesselate(PG_FUNCTION_ARGS)
 
 	sfcgal_postgis_init();
 
-	input = (GSERIALIZED*) PG_DETOAST_DATUM(PG_GETARG_DATUM(0));
+	input = PG_GETARG_GSERIALIZED_P(0);
 	srid = gserialized_get_srid(input);
 	geom = POSTGIS2SFCGALGeometry(input);
 	PG_FREE_IF_COPY(input, 0);
@@ -352,7 +359,7 @@ Datum sfcgal_triangulate(PG_FUNCTION_ARGS)
 
 	sfcgal_postgis_init();
 
-	input = (GSERIALIZED*) PG_DETOAST_DATUM(PG_GETARG_DATUM(0));
+	input = PG_GETARG_GSERIALIZED_P(0);
 	srid = gserialized_get_srid(input);
 	geom = POSTGIS2SFCGALGeometry(input);
 	PG_FREE_IF_COPY(input, 0);
@@ -377,7 +384,7 @@ Datum sfcgal_force_lhr(PG_FUNCTION_ARGS)
 
 	sfcgal_postgis_init();
 
-	input = (GSERIALIZED*) PG_DETOAST_DATUM(PG_GETARG_DATUM(0));
+	input = PG_GETARG_GSERIALIZED_P(0);
 	srid = gserialized_get_srid(input);
 	geom = POSTGIS2SFCGALGeometry(input);
 	PG_FREE_IF_COPY(input, 0);
@@ -402,7 +409,7 @@ Datum sfcgal_straight_skeleton(PG_FUNCTION_ARGS)
 
 	sfcgal_postgis_init();
 
-	input = (GSERIALIZED*) PG_DETOAST_DATUM(PG_GETARG_DATUM(0));
+	input = PG_GETARG_GSERIALIZED_P(0);
 	srid = gserialized_get_srid(input);
 	geom = POSTGIS2SFCGALGeometry(input);
 	PG_FREE_IF_COPY(input, 0);
@@ -416,6 +423,38 @@ Datum sfcgal_straight_skeleton(PG_FUNCTION_ARGS)
 	PG_RETURN_POINTER(output);
 }
 
+PG_FUNCTION_INFO_V1(sfcgal_approximate_medial_axis);
+Datum sfcgal_approximate_medial_axis(PG_FUNCTION_ARGS)
+{
+#if POSTGIS_SFCGAL_VERSION < 12
+	lwpgerror("The SFCGAL version this PostGIS binary "
+	        "was compiled against (%d) doesn't support "
+	        "'sfcgal_geometry_approximate_medial_axis' function (1.2.0+ required)",
+	        POSTGIS_SFCGAL_VERSION);
+	PG_RETURN_NULL();
+#else /* POSTGIS_SFCGAL_VERSION >= 12 */
+	GSERIALIZED *input, *output;
+	sfcgal_geometry_t *geom;
+	sfcgal_geometry_t *result;
+	srid_t srid;
+
+	sfcgal_postgis_init();
+
+	input = PG_GETARG_GSERIALIZED_P(0);
+	srid = gserialized_get_srid(input);
+	geom = POSTGIS2SFCGALGeometry(input);
+	PG_FREE_IF_COPY(input, 0);
+
+	result = sfcgal_geometry_approximate_medial_axis(geom);
+	sfcgal_geometry_delete(geom);
+
+	output = SFCGALGeometry2POSTGIS(result, 0, srid);
+	sfcgal_geometry_delete(result);
+
+	PG_RETURN_POINTER(output);
+#endif /* POSTGIS_SFCGAL_VERSION >= 12 */
+}
+
 
 PG_FUNCTION_INFO_V1(sfcgal_intersection);	
 Datum sfcgal_intersection(PG_FUNCTION_ARGS)
@@ -427,9 +466,9 @@ Datum sfcgal_intersection(PG_FUNCTION_ARGS)
 
 	sfcgal_postgis_init();
 
-	input0 = (GSERIALIZED*) PG_DETOAST_DATUM(PG_GETARG_DATUM(0));
+	input0 = PG_GETARG_GSERIALIZED_P(0);
 	srid = gserialized_get_srid(input0);
-	input1 = (GSERIALIZED*) PG_DETOAST_DATUM(PG_GETARG_DATUM(1));
+	input1 = PG_GETARG_GSERIALIZED_P(1);
 	geom0 = POSTGIS2SFCGALGeometry(input0);
 	PG_FREE_IF_COPY(input0, 0);
 	geom1 = POSTGIS2SFCGALGeometry(input1);
@@ -456,9 +495,9 @@ Datum sfcgal_intersection3D(PG_FUNCTION_ARGS)
 
 	sfcgal_postgis_init();
 
-	input0 = (GSERIALIZED*) PG_DETOAST_DATUM(PG_GETARG_DATUM(0));
+	input0 = PG_GETARG_GSERIALIZED_P(0);
 	srid = gserialized_get_srid(input0);
-	input1 = (GSERIALIZED*) PG_DETOAST_DATUM(PG_GETARG_DATUM(1));
+	input1 = PG_GETARG_GSERIALIZED_P(1);
 	geom0 = POSTGIS2SFCGALGeometry(input0);
 	PG_FREE_IF_COPY(input0, 0);
 	geom1 = POSTGIS2SFCGALGeometry(input1);
@@ -474,9 +513,8 @@ Datum sfcgal_intersection3D(PG_FUNCTION_ARGS)
 	PG_RETURN_POINTER(output);
 }
 
-
-PG_FUNCTION_INFO_V1(sfcgal_minkowski_sum);	
-Datum sfcgal_minkowski_sum(PG_FUNCTION_ARGS)
+PG_FUNCTION_INFO_V1(sfcgal_difference);	
+Datum sfcgal_difference(PG_FUNCTION_ARGS)
 {
 	GSERIALIZED *input0, *input1, *output;
 	sfcgal_geometry_t *geom0, *geom1;
@@ -493,6 +531,140 @@ Datum sfcgal_minkowski_sum(PG_FUNCTION_ARGS)
 	geom1 = POSTGIS2SFCGALGeometry(input1);
 	PG_FREE_IF_COPY(input1, 1);
 
+	result = sfcgal_geometry_difference(geom0, geom1);
+	sfcgal_geometry_delete(geom0);
+	sfcgal_geometry_delete(geom1);
+
+	output = SFCGALGeometry2POSTGIS(result, 0, srid);
+	sfcgal_geometry_delete(result);
+
+	PG_RETURN_POINTER(output);
+}
+
+
+PG_FUNCTION_INFO_V1(sfcgal_difference3D);	
+Datum sfcgal_difference3D(PG_FUNCTION_ARGS)
+{
+	GSERIALIZED *input0, *input1, *output;
+	sfcgal_geometry_t *geom0, *geom1;
+	sfcgal_geometry_t *result;
+	srid_t srid;
+
+	sfcgal_postgis_init();
+
+	input0 = (GSERIALIZED*) PG_DETOAST_DATUM(PG_GETARG_DATUM(0));
+	srid = gserialized_get_srid(input0);
+	input1 = (GSERIALIZED*) PG_DETOAST_DATUM(PG_GETARG_DATUM(1));
+	geom0 = POSTGIS2SFCGALGeometry(input0);
+	PG_FREE_IF_COPY(input0, 0);
+	geom1 = POSTGIS2SFCGALGeometry(input1);
+	PG_FREE_IF_COPY(input1, 1);
+
+	result = sfcgal_geometry_difference_3d(geom0, geom1);
+	sfcgal_geometry_delete(geom0);
+	sfcgal_geometry_delete(geom1);
+
+	output = SFCGALGeometry2POSTGIS(result, 0, srid);
+	sfcgal_geometry_delete(result);
+
+	PG_RETURN_POINTER(output);
+}
+
+PG_FUNCTION_INFO_V1(sfcgal_union);	
+Datum sfcgal_union(PG_FUNCTION_ARGS)
+{
+	GSERIALIZED *input0, *input1, *output;
+	sfcgal_geometry_t *geom0, *geom1;
+	sfcgal_geometry_t *result;
+	srid_t srid;
+
+	sfcgal_postgis_init();
+
+	input0 = (GSERIALIZED*) PG_DETOAST_DATUM(PG_GETARG_DATUM(0));
+	srid = gserialized_get_srid(input0);
+	input1 = (GSERIALIZED*) PG_DETOAST_DATUM(PG_GETARG_DATUM(1));
+	geom0 = POSTGIS2SFCGALGeometry(input0);
+	PG_FREE_IF_COPY(input0, 0);
+	geom1 = POSTGIS2SFCGALGeometry(input1);
+	PG_FREE_IF_COPY(input1, 1);
+
+	result = sfcgal_geometry_union(geom0, geom1);
+	sfcgal_geometry_delete(geom0);
+	sfcgal_geometry_delete(geom1);
+
+	output = SFCGALGeometry2POSTGIS(result, 0, srid);
+	sfcgal_geometry_delete(result);
+
+	PG_RETURN_POINTER(output);
+}
+
+
+PG_FUNCTION_INFO_V1(sfcgal_union3D);	
+Datum sfcgal_union3D(PG_FUNCTION_ARGS)
+{
+	GSERIALIZED *input0, *input1, *output;
+	sfcgal_geometry_t *geom0, *geom1;
+	sfcgal_geometry_t *result;
+	srid_t srid;
+
+	sfcgal_postgis_init();
+
+	input0 = (GSERIALIZED*) PG_DETOAST_DATUM(PG_GETARG_DATUM(0));
+	srid = gserialized_get_srid(input0);
+	input1 = (GSERIALIZED*) PG_DETOAST_DATUM(PG_GETARG_DATUM(1));
+	geom0 = POSTGIS2SFCGALGeometry(input0);
+	PG_FREE_IF_COPY(input0, 0);
+	geom1 = POSTGIS2SFCGALGeometry(input1);
+	PG_FREE_IF_COPY(input1, 1);
+
+	result = sfcgal_geometry_union_3d(geom0, geom1);
+	sfcgal_geometry_delete(geom0);
+	sfcgal_geometry_delete(geom1);
+
+	output = SFCGALGeometry2POSTGIS(result, 0, srid);
+	sfcgal_geometry_delete(result);
+
+	PG_RETURN_POINTER(output);
+}
+
+PG_FUNCTION_INFO_V1(sfcgal_volume);
+Datum sfcgal_volume(PG_FUNCTION_ARGS)
+{
+	GSERIALIZED *input;
+	sfcgal_geometry_t *geom;
+	double result;
+
+	sfcgal_postgis_init();
+
+	input = (GSERIALIZED*) PG_DETOAST_DATUM(PG_GETARG_DATUM(0));
+	geom = POSTGIS2SFCGALGeometry(input);
+
+	result = sfcgal_geometry_volume(geom);
+	sfcgal_geometry_delete(geom);
+
+	PG_FREE_IF_COPY(input, 0);
+
+	PG_RETURN_FLOAT8(result);
+}
+
+PG_FUNCTION_INFO_V1(sfcgal_minkowski_sum);	
+Datum sfcgal_minkowski_sum(PG_FUNCTION_ARGS)
+{
+	GSERIALIZED *input0, *input1, *output;
+	sfcgal_geometry_t *geom0, *geom1;
+	sfcgal_geometry_t *result;
+	srid_t srid;
+
+	sfcgal_postgis_init();
+
+	input0 = PG_GETARG_GSERIALIZED_P(0);
+	srid = gserialized_get_srid(input0);
+	input1 = PG_GETARG_GSERIALIZED_P(1);
+	geom0 = POSTGIS2SFCGALGeometry(input0);
+	PG_FREE_IF_COPY(input0, 0);
+	geom1 = POSTGIS2SFCGALGeometry(input1);
+	PG_FREE_IF_COPY(input1, 1);
+
 	result = sfcgal_geometry_minkowski_sum(geom0, geom1);
 	sfcgal_geometry_delete(geom0);
 	sfcgal_geometry_delete(geom1);
@@ -515,7 +687,7 @@ Datum sfcgal_extrude(PG_FUNCTION_ARGS)
 
     sfcgal_postgis_init();
     
-    input = (GSERIALIZED*) PG_DETOAST_DATUM(PG_GETARG_DATUM(0));
+    input = PG_GETARG_GSERIALIZED_P(0);
     srid = gserialized_get_srid(input);
 
     geom = POSTGIS2SFCGALGeometry(input);
@@ -542,3 +714,41 @@ Datum postgis_sfcgal_version(PG_FUNCTION_ARGS)
         PG_RETURN_POINTER(result);
 }
 
+PG_FUNCTION_INFO_V1(sfcgal_is_solid);	
+Datum sfcgal_is_solid(PG_FUNCTION_ARGS)
+{
+	int result;
+	GSERIALIZED *input = PG_GETARG_GSERIALIZED_P(0);
+        LWGEOM *lwgeom = lwgeom_from_gserialized(input);
+	PG_FREE_IF_COPY(input, 0);
+	if (! lwgeom)
+	{
+		lwerror("sfcgal_is_solid: Unable to deserialize input");
+	}
+        result = FLAGS_GET_SOLID( lwgeom->flags );
+
+	lwgeom_free(lwgeom);
+
+	PG_RETURN_BOOL(result);
+}
+
+PG_FUNCTION_INFO_V1(sfcgal_make_solid);	
+Datum sfcgal_make_solid(PG_FUNCTION_ARGS)
+{
+        GSERIALIZED *output;
+	GSERIALIZED *input = PG_GETARG_GSERIALIZED_P(0);
+        LWGEOM *lwgeom = lwgeom_from_gserialized(input);
+	PG_FREE_IF_COPY(input, 0);
+	if (! lwgeom)
+	{
+		lwerror("sfcgal_make_solid: Unable to deserialize input");
+	}
+
+        FLAGS_SET_SOLID( lwgeom->flags, 1);
+
+	output = geometry_serialize( lwgeom );
+	lwgeom_free(lwgeom);
+
+	PG_RETURN_POINTER(output);
+}
+
