@@ -1,5 +1,4 @@
 /**********************************************************************
- * $Id: lwgeom_sqlmm.c 13134 2014-12-01 08:47:21Z strk $
  *
  * PostGIS - Spatial Types for PostgreSQL
  * http://postgis.net
@@ -37,7 +36,7 @@ Datum LWGEOM_line_desegmentize(PG_FUNCTION_ARGS);
 PG_FUNCTION_INFO_V1(LWGEOM_has_arc);
 Datum LWGEOM_has_arc(PG_FUNCTION_ARGS)
 {
-	GSERIALIZED *geom = (GSERIALIZED *)PG_DETOAST_DATUM(PG_GETARG_DATUM(0));
+	GSERIALIZED *geom = PG_GETARG_GSERIALIZED_P(0);
 	LWGEOM *lwgeom = lwgeom_from_gserialized(geom);
 	uint32 result = lwgeom_has_arc(lwgeom);
 	lwgeom_free(lwgeom);
@@ -53,7 +52,7 @@ Datum LWGEOM_has_arc(PG_FUNCTION_ARGS)
 PG_FUNCTION_INFO_V1(LWGEOM_curve_segmentize);
 Datum LWGEOM_curve_segmentize(PG_FUNCTION_ARGS)
 {
-	GSERIALIZED *geom = (GSERIALIZED *)PG_DETOAST_DATUM(PG_GETARG_DATUM(0));
+	GSERIALIZED *geom = PG_GETARG_GSERIALIZED_P(0);
 	int32 perQuad = PG_GETARG_INT32(1);
 	GSERIALIZED *ret;
 	LWGEOM *igeom = NULL, *ogeom = NULL;
@@ -69,7 +68,7 @@ Datum LWGEOM_curve_segmentize(PG_FUNCTION_ARGS)
 	POSTGIS_DEBUGF(3, "perQuad = %d", perQuad);
 
 	igeom = lwgeom_from_gserialized(geom);
-	ogeom = lwgeom_segmentize(igeom, perQuad);
+	ogeom = lwgeom_stroke(igeom, perQuad);
 	lwgeom_free(igeom);
 	
 	if (ogeom == NULL) 
@@ -84,14 +83,14 @@ Datum LWGEOM_curve_segmentize(PG_FUNCTION_ARGS)
 PG_FUNCTION_INFO_V1(LWGEOM_line_desegmentize);
 Datum LWGEOM_line_desegmentize(PG_FUNCTION_ARGS)
 {
-	GSERIALIZED *geom = (GSERIALIZED *)PG_DETOAST_DATUM(PG_GETARG_DATUM(0));
+	GSERIALIZED *geom = PG_GETARG_GSERIALIZED_P(0);
 	GSERIALIZED *ret;
 	LWGEOM *igeom = NULL, *ogeom = NULL;
 
 	POSTGIS_DEBUG(2, "LWGEOM_line_desegmentize.");
 
 	igeom = lwgeom_from_gserialized(geom);
-	ogeom = lwgeom_desegmentize(igeom);
+	ogeom = lwgeom_unstroke(igeom);
 	lwgeom_free(igeom);
 
 	if (ogeom == NULL)
