@@ -1,5 +1,4 @@
 /*
- * $Id: raster2pgsql.c 12748 2014-07-07 08:57:57Z strk $
  *
  * PostGIS raster loader
  * http://trac.osgeo.org/postgis/wiki/WKTRaster
@@ -22,7 +21,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software Foundation,
- * Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
+ * Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
  */
 
@@ -329,10 +328,18 @@ usage() {
 		"\n"
 		"OPTIONS:\n"
 	));
+	/*
 	printf(_(
-		"  -s <srid> Set the raster's SRID. Defaults to %d. If SRID not\n"
-		"      provided or is %d, raster's metadata will be checked to\n"
-		"      determine an appropriate SRID.\n"
+		"  -s [<from>:]<srid> Set the SRID field. Defaults to %d.\n"
+		"     Optionally reprojects from given SRID (cannot be used with -Y).\n"
+		"     Raster's metadata will be checked to determine an appropriate SRID.\n"
+		"     If a srid of %d is provided (either as from or as target).\n"
+	), SRID_UNKNOWN, SRID_UNKNOWN);
+	*/
+	printf(_(
+		"  -s <srid> Set the SRID field. Defaults to %d. If SRID not\n"
+		"     provided or is %d, raster's metadata will be checked to\n"
+		"     determine an appropriate SRID.\n"
 	), SRID_UNKNOWN, SRID_UNKNOWN);
 	printf(_(
 		"  -b <band> Index (1-based) of band to extract from raster. For more\n"
@@ -626,7 +633,7 @@ diff_rastinfo(RASTERINFO *x, RASTERINFO *ref) {
 	if (!msg[3]) {
 		for (i = 0; i < ref->nband_count; i++) {
 			if (!x->hasnodata[i] && !ref->hasnodata[i]) continue;
-			if (FLT_NEQ(x->hasnodata[i], ref->hasnodata[i])) {
+			if (x->hasnodata[i] != ref->hasnodata[i]) {
 				rtwarn(_("Different NODATA values found for band %d in the set of rasters being converted to PostGIS raster"), ref->nband[i]);
 				msg[3]++;
 			}
@@ -670,7 +677,7 @@ diff_rastinfo(RASTERINFO *x, RASTERINFO *ref) {
 	/* tile size */
 	if (!msg[5]) {
 		for (i = 0; i < 2; i++) {
-			if (FLT_NEQ(x->tile_size[i], ref->tile_size[i])) {
+			if (x->tile_size[i] != ref->tile_size[i]) {
 				rtwarn(_("Different tile sizes found in the set of rasters being converted to PostGIS raster"));
 				msg[5]++;
 				break;
@@ -693,7 +700,7 @@ init_config(RTLOADERCFG *config) {
 	config->overview = NULL;
 	config->overview_table = NULL;
 	config->quoteident = 0;
-	config->srid = SRID_UNKNOWN;
+	config->srid = config->out_srid = SRID_UNKNOWN;
 	config->nband = NULL;
 	config->nband_count = 0;
 	memset(config->tile_size, 0, sizeof(int) * 2);
@@ -794,6 +801,7 @@ flush_stringbuffer(STRINGBUFFER *buffer) {
 	rtdealloc_stringbuffer(buffer, 0);
 }
 
+/* Takes ownership of the passed string */
 static int
 append_stringbuffer(STRINGBUFFER *buffer, const char *str) {
 	buffer->length++;
@@ -804,13 +812,7 @@ append_stringbuffer(STRINGBUFFER *buffer, const char *str) {
 		return 0;
 	}
 
-	buffer->line[buffer->length - 1] = NULL;
-	buffer->line[buffer->length - 1] = rtalloc(sizeof(char) * (strlen(str) + 1));
-	if (buffer->line[buffer->length - 1] == NULL) {
-		rterror(_("append_stringbuffer: Could not allocate memory for appending string to buffer"));
-		return 0;
-	}
-	strcpy(buffer->line[buffer->length - 1], str);
+	buffer->line[buffer->length - 1] = (char *) str;
 
 	return 1;
 }
@@ -823,11 +825,58 @@ append_sql_to_buffer(STRINGBUFFER *buffer, const char *str) {
 	return append_stringbuffer(buffer, str);
 }
 
+static int
+copy_from(const char *schema, const char *table, const char *column,
+          const char *filename, const char *file_column_name,
+          STRINGBUFFER *buffer)
+{
+	char *sql = NULL;
+	uint32_t len = 0;
+
+	assert(table != NULL);
+	assert(column != NULL);
+
+	len = strlen("COPY  () FROM stdin;") + 1;
+	if (schema != NULL)
+		len += strlen(schema);
+	len += strlen(table);
+	len += strlen(column);
+	if (filename != NULL)
+		len += strlen(",") + strlen(file_column_name);
+
+	sql = rtalloc(sizeof(char) * len);
+	if (sql == NULL) {
+		rterror(_("copy_from: Could not allocate memory for COPY statement"));
+		return 0;
+	}
+	sprintf(sql, "COPY %s%s (%s%s%s) FROM stdin;",
+		(schema != NULL ? schema : ""),
+		table,
+		column,
+		(filename != NULL ? "," : ""),
+		(filename != NULL ? file_column_name : "")
+	);
+
+	append_sql_to_buffer(buffer, sql);
+	sql = NULL;
+
+	return 1;
+}
+
+static int
+copy_from_end(STRINGBUFFER *buffer)
+{
+	/* end of data */
+	append_sql_to_buffer(buffer, strdup("\\."));
+
+	return 1;
+}
+
 static int
 insert_records(
 	const char *schema, const char *table, const char *column,
 	const char *filename, const char *file_column_name,
-	int copy_statements,
+	int copy_statements, int out_srid,
 	STRINGBUFFER *tileset, STRINGBUFFER *buffer
 ) {
 	char *fn = NULL;
@@ -841,6 +890,16 @@ insert_records(
 	/* COPY statements */
 	if (copy_statements) {
 
+    if (!copy_from(
+      schema, table, column,
+      (file_column_name ? filename : NULL), file_column_name,
+      buffer
+    )) {
+      rterror(_("insert_records: Could not add COPY statement to string buffer"));
+      return 0;
+    }
+
+
 		/* escape tabs in filename */
 		if (filename != NULL)
 			fn = strreplace(filename, "\t", "\\t", NULL);
@@ -864,14 +923,18 @@ insert_records(
 			);
 
 			append_sql_to_buffer(buffer, sql);
-			rtdealloc(sql);
 			sql = NULL;
 		}
 
+    if (!copy_from_end(buffer)) {
+      rterror(_("process_rasters: Could not add COPY end statement to string buffer"));
+      return 0;
+    }
+
 	}
 	/* INSERT statements */
 	else {
-		len = strlen("INSERT INTO  () VALUES (''::raster);") + 1;
+		len = strlen("INSERT INTO  () VALUES (ST_Transform(''::raster,xxxxxxxxx));") + 1;
 		if (schema != NULL)
 			len += strlen(schema);
 		len += strlen(table);
@@ -884,6 +947,7 @@ insert_records(
 			fn = strreplace(filename, "'", "''", NULL);
 
 		for (x = 0; x < tileset->length; x++) {
+			char *ptr;
 			int sqllen = len;
 
 			sqllen += strlen(tileset->line[x]);
@@ -895,20 +959,29 @@ insert_records(
 				rterror(_("insert_records: Could not allocate memory for INSERT statement"));
 				return 0;
 			}
-			sprintf(sql, "INSERT INTO %s%s (%s%s%s) VALUES ('%s'::raster%s%s%s);",
-				(schema != NULL ? schema : ""),
-				table,
-				column,
-				(filename != NULL ? "," : ""),
-				(filename != NULL ? file_column_name : ""),
-				tileset->line[x],
-				(filename != NULL ? ",'" : ""),
-				(filename != NULL ? fn : ""),
-				(filename != NULL ? "'" : "")
-			);
+			ptr = sql;
+			ptr += sprintf(sql, "INSERT INTO %s%s (%s%s%s) VALUES (",
+					(schema != NULL ? schema : ""),
+					table,
+					column,
+					(filename != NULL ? "," : ""),
+					(filename != NULL ? file_column_name : "")
+				);
+			if (out_srid != SRID_UNKNOWN) {
+				ptr += sprintf(ptr, "ST_Transform(");
+			}
+			ptr += sprintf(ptr, "'%s'::raster",
+					tileset->line[x]
+				);
+			if (out_srid != SRID_UNKNOWN) {
+				ptr += sprintf(ptr, ", %d)", out_srid);
+			}
+			if (filename != NULL) {
+				ptr += sprintf(ptr, ",'%s'", fn);
+			}
+			ptr += sprintf(ptr, ");");
 
 			append_sql_to_buffer(buffer, sql);
-			rtdealloc(sql);
 			sql = NULL;
 		}
 	}
@@ -938,7 +1011,6 @@ drop_table(const char *schema, const char *table, STRINGBUFFER *buffer) {
 	);
 
 	append_sql_to_buffer(buffer, sql);
-	rtdealloc(sql);
 
 	return 1;
 }
@@ -987,55 +1059,6 @@ create_table(
 	);
 
 	append_sql_to_buffer(buffer, sql);
-	rtdealloc(sql);
-
-	return 1;
-}
-
-static int
-copy_from(
-	const char *schema, const char *table, const char *column,
-	const char *filename, const char *file_column_name,
-	STRINGBUFFER *buffer
-) {
-	char *sql = NULL;
-	uint32_t len = 0;
-
-	assert(table != NULL);
-	assert(column != NULL);
-
-	len = strlen("COPY  () FROM stdin;") + 1;
-	if (schema != NULL)
-		len += strlen(schema);
-	len += strlen(table);
-	len += strlen(column);
-	if (filename != NULL)
-		len += strlen(",") + strlen(file_column_name);
-
-	sql = rtalloc(sizeof(char) * len);
-	if (sql == NULL) {
-		rterror(_("copy_from: Could not allocate memory for COPY statement"));
-		return 0;
-	}
-	sprintf(sql, "COPY %s%s (%s%s%s) FROM stdin;",
-		(schema != NULL ? schema : ""),
-		table,
-		column,
-		(filename != NULL ? "," : ""),
-		(filename != NULL ? file_column_name : "")
-	);
-
-	append_sql_to_buffer(buffer, sql);
-	rtdealloc(sql);
-	sql = NULL;
-
-	return 1;
-}
-
-static int
-copy_from_end(STRINGBUFFER *buffer) {
-	/* end of data */
-	append_sql_to_buffer(buffer, "\\.");
 
 	return 1;
 }
@@ -1086,7 +1109,6 @@ create_index(
 	rtdealloc(_column);
 
 	append_sql_to_buffer(buffer, sql);
-	rtdealloc(sql);
 
 	return 1;
 }
@@ -1117,7 +1139,6 @@ analyze_table(
 	);
 
 	append_sql_to_buffer(buffer, sql);
-	rtdealloc(sql);
 
 	return 1;
 }
@@ -1148,7 +1169,6 @@ vacuum_table(
 	);
 
 	append_sql_to_buffer(buffer, sql);
-	rtdealloc(sql);
 
 	return 1;
 }
@@ -1215,7 +1235,6 @@ add_raster_constraints(
 	rtdealloc(_column);
 
 	append_sql_to_buffer(buffer, sql);
-	rtdealloc(sql);
 
 	return 1;
 }
@@ -1322,7 +1341,6 @@ add_overview_constraints(
 	rtdealloc(_column);
 
 	append_sql_to_buffer(buffer, sql);
-	rtdealloc(sql);
 
 	return 1;
 }
@@ -1515,7 +1533,6 @@ build_overview(int idx, RTLOADERCFG *config, RASTERINFO *info, int ovx, STRINGBU
 			/* add hexwkb to tileset */
 			append_stringbuffer(tileset, hex);
 
-			rtdealloc(hex);
 			GDALClose(hdsDst);
 
 			/* flush if tileset gets too big */
@@ -1523,7 +1540,7 @@ build_overview(int idx, RTLOADERCFG *config, RASTERINFO *info, int ovx, STRINGBU
 				if (!insert_records(
 					config->schema, ovtable, config->raster_column,
 					(config->file_column ? config->rt_filename[idx] : NULL), config->file_column_name,
-					config->copy_statements,
+					config->copy_statements, config->out_srid,
 					tileset, buffer
 				)) {
 					rterror(_("build_overview: Could not convert raster tiles into INSERT or COPY statements"));
@@ -1613,6 +1630,12 @@ convert_raster(int idx, RTLOADERCFG *config, RASTERINFO *info, STRINGBUFFER *til
 		}
 	}
 
+	if ( info->srid == SRID_UNKNOWN && config->out_srid != SRID_UNKNOWN ) {
+		  rterror(_("convert_raster: could not determine source srid, cannot transform to target srid %d"), config->out_srid);
+		  GDALClose(hdsSrc);
+		  return 0;
+	}
+
 	/* record geotransform matrix */
 	if (GDALGetGeoTransform(hdsSrc, info->gt) != CE_None) {
 		rtinfo(_("Using default geotransform matrix (0, 1, 0, 0, 0, -1) for raster: %s"), config->rt_file[idx]);
@@ -1832,14 +1855,12 @@ convert_raster(int idx, RTLOADERCFG *config, RASTERINFO *info, STRINGBUFFER *til
 				/* add hexwkb to tileset */
 				append_stringbuffer(tileset, hex);
 
-				rtdealloc(hex);
-
 				/* flush if tileset gets too big */
 				if (tileset->length > 10) {
 					if (!insert_records(
 						config->schema, config->table, config->raster_column,
 						(config->file_column ? config->rt_filename[idx] : NULL), config->file_column_name,
-						config->copy_statements,
+						config->copy_statements, config->out_srid,
 						tileset, buffer
 					)) {
 						rterror(_("convert_raster: Could not convert raster tiles into INSERT or COPY statements"));
@@ -1951,7 +1972,6 @@ convert_raster(int idx, RTLOADERCFG *config, RASTERINFO *info, STRINGBUFFER *til
 				/* add hexwkb to tileset */
 				append_stringbuffer(tileset, hex);
 
-				rtdealloc(hex);
 				GDALClose(hdsDst);
 
 				/* flush if tileset gets too big */
@@ -1959,7 +1979,7 @@ convert_raster(int idx, RTLOADERCFG *config, RASTERINFO *info, STRINGBUFFER *til
 					if (!insert_records(
 						config->schema, config->table, config->raster_column,
 						(config->file_column ? config->rt_filename[idx] : NULL), config->file_column_name,
-						config->copy_statements,
+						config->copy_statements, config->out_srid,
 						tileset, buffer
 					)) {
 						rterror(_("convert_raster: Could not convert raster tiles into INSERT or COPY statements"));
@@ -1987,7 +2007,7 @@ process_rasters(RTLOADERCFG *config, STRINGBUFFER *buffer) {
 	assert(config->raster_column != NULL);
 
 	if (config->transaction) {
-		if (!append_sql_to_buffer(buffer, "BEGIN;")) {
+		if (!append_sql_to_buffer(buffer, strdup("BEGIN;"))) {
 			rterror(_("process_rasters: Could not add BEGIN statement to string buffer"));
 			return 0;
 		}
@@ -2052,17 +2072,6 @@ process_rasters(RTLOADERCFG *config, STRINGBUFFER *buffer) {
 			init_rastinfo(&rastinfo);
 			init_stringbuffer(&tileset);
 
-			if (config->copy_statements && !copy_from(
-				config->schema, config->table, config->raster_column,
-				(config->file_column ? config->rt_filename[i] : NULL), config->file_column_name,
-				buffer
-			)) {
-				rterror(_("process_rasters: Could not add COPY statement to string buffer"));
-				rtdealloc_rastinfo(&rastinfo);
-				rtdealloc_stringbuffer(&tileset, 0);
-				return 0;
-			}
-
 			/* convert raster */
 			if (!convert_raster(i, config, &rastinfo, &tileset, buffer)) {
 				rterror(_("process_rasters: Could not process raster: %s"), config->rt_file[i]);
@@ -2074,8 +2083,9 @@ process_rasters(RTLOADERCFG *config, STRINGBUFFER *buffer) {
 			/* process raster tiles into COPY or INSERT statements */
 			if (tileset.length && !insert_records(
 				config->schema, config->table, config->raster_column,
-				(config->file_column ? config->rt_filename[i] : NULL), config->file_column_name,
-				config->copy_statements,
+				(config->file_column ? config->rt_filename[i] : NULL),
+        config->file_column_name,
+				config->copy_statements, config->out_srid,
 				&tileset, buffer
 			)) {
 				rterror(_("process_rasters: Could not convert raster tiles into INSERT or COPY statements"));
@@ -2086,12 +2096,6 @@ process_rasters(RTLOADERCFG *config, STRINGBUFFER *buffer) {
 
 			rtdealloc_stringbuffer(&tileset, 0);
 
-			if (config->copy_statements && !copy_from_end(buffer)) {
-				rterror(_("process_rasters: Could not add COPY end statement to string buffer"));
-				rtdealloc_rastinfo(&rastinfo);
-				return 0;
-			}
-
 			/* flush buffer after every raster */
 			flush_stringbuffer(buffer);
 
@@ -2101,17 +2105,6 @@ process_rasters(RTLOADERCFG *config, STRINGBUFFER *buffer) {
 
 				for (j = 0; j < config->overview_count; j++) {
 
-					if (config->copy_statements && !copy_from(
-							config->schema, config->overview_table[j], config->raster_column,
-							(config->file_column ? config->rt_filename[i] : NULL), config->file_column_name,
-							buffer
-					)) {
-						rterror(_("process_rasters: Could not add COPY statement to string buffer"));
-						rtdealloc_rastinfo(&rastinfo);
-						rtdealloc_stringbuffer(&tileset, 0);
-						return 0;
-					}
-
 					if (!build_overview(i, config, &rastinfo, j, &tileset, buffer)) {
 						rterror(_("process_rasters: Could not create overview of factor %d for raster %s"), config->overview[j], config->rt_file[i]);
 						rtdealloc_rastinfo(&rastinfo);
@@ -2122,7 +2115,7 @@ process_rasters(RTLOADERCFG *config, STRINGBUFFER *buffer) {
 					if (tileset.length && !insert_records(
 						config->schema, config->overview_table[j], config->raster_column,
 						(config->file_column ? config->rt_filename[i] : NULL), config->file_column_name,
-						config->copy_statements,
+						config->copy_statements, config->out_srid,
 						&tileset, buffer
 					)) {
 						rterror(_("process_rasters: Could not convert overview tiles into INSERT or COPY statements"));
@@ -2135,14 +2128,6 @@ process_rasters(RTLOADERCFG *config, STRINGBUFFER *buffer) {
 
 					/* flush buffer after every raster */
 					flush_stringbuffer(buffer);
-
-					if (config->copy_statements) {
-						if (!copy_from_end(buffer)) {
-							rterror(_("process_rasters: Could not add COPY end statement to string buffer"));
-							rtdealloc_rastinfo(&rastinfo);
-							return 0;
-						}
-					}
 				}
 			}
 
@@ -2250,7 +2235,7 @@ process_rasters(RTLOADERCFG *config, STRINGBUFFER *buffer) {
 	}
 
 	if (config->transaction) {
-		if (!append_sql_to_buffer(buffer, "END;")) {
+		if (!append_sql_to_buffer(buffer, strdup("END;"))) {
 			rterror(_("process_rasters: Could not add END statement to string buffer"));
 			return 0;
 		}
@@ -2319,9 +2304,18 @@ main(int argc, char **argv) {
 	****************************************************************************/
 
 	for (i = 1; i < argc; i++) {
+		char *optarg, *ptr;
 		/* srid */
 		if (CSEQUAL(argv[i], "-s") && i < argc - 1) {
-			config->srid = atoi(argv[++i]);
+			optarg = argv[++i];
+			ptr = strchr(optarg, ':');
+			if (ptr) {
+				*ptr++ = '\0';
+				sscanf(optarg, "%d", &config->srid);
+				sscanf(ptr, "%d", &config->out_srid);
+			} else {
+				config->srid = config->out_srid = atoi(optarg);
+			}
 		}
 		/* band index */
 		else if (CSEQUAL(argv[i], "-b") && i < argc - 1) {
@@ -2651,6 +2645,17 @@ main(int argc, char **argv) {
 		}
 	}
 
+	if (config->srid != config->out_srid) {
+		if (config->copy_statements) {
+			rterror(_("Invalid argument combination - cannot use -Y with -s FROM_SRID:TO_SRID"));
+			exit(1);
+		}
+		if (config->out_srid == SRID_UNKNOWN) {
+			rterror(_("Unknown target SRID is invalid when source SRID is given"));
+			exit(1);
+		}
+	}
+
 	/* register GDAL drivers */
 	GDALAllRegister();
 
