@@ -17,7 +17,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software Foundation,
- * Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
+ * Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
  */
 
@@ -570,6 +570,7 @@ static void test_raster_iterator() {
 		PT_32BUI,
 		1, 0,
 		0, 0,
+		NULL,
 		userargs,
 		testRasterIterator1_callback,
 		&rtn
@@ -599,6 +600,7 @@ static void test_raster_iterator() {
 		PT_32BUI,
 		1, 0,
 		0, 0,
+		NULL,
 		userargs,
 		testRasterIterator1_callback,
 		&rtn
@@ -628,6 +630,7 @@ static void test_raster_iterator() {
 		PT_32BUI,
 		1, 0,
 		0, 0,
+		NULL,
 		userargs,
 		testRasterIterator2_callback,
 		&rtn
@@ -653,6 +656,7 @@ static void test_raster_iterator() {
 		PT_32BUI,
 		1, 0,
 		0, 0,
+		NULL,
 		userargs,
 		testRasterIterator3_callback,
 		&rtn
@@ -678,6 +682,7 @@ static void test_raster_iterator() {
 		PT_32BUI,
 		1, 0,
 		0, 0,
+		NULL,
 		userargs,
 		testRasterIterator4_callback,
 		&rtn
@@ -703,6 +708,7 @@ static void test_raster_iterator() {
 		PT_32BUI,
 		1, 0,
 		0, 0,
+		NULL,
 		userargs,
 		testRasterIterator5_callback,
 		&rtn
@@ -728,6 +734,7 @@ static void test_raster_iterator() {
 		PT_32BUI,
 		1, 0,
 		0, 0,
+		NULL,
 		userargs,
 		testRasterIterator6_callback,
 		&rtn
@@ -757,6 +764,7 @@ static void test_raster_iterator() {
 		PT_32BUI,
 		1, 0,
 		1, 1,
+		NULL,
 		userargs,
 		testRasterIterator7_callback,
 		&rtn
@@ -1176,10 +1184,12 @@ static void test_raster_colormap() {
 }
 
 /* register tests */
-CU_TestInfo mapalgebra_tests[] = {
-	PG_TEST(test_raster_iterator),
-	PG_TEST(test_band_reclass),
-	PG_TEST(test_raster_colormap),
-	CU_TEST_INFO_NULL
-};
-CU_SuiteInfo mapalgebra_suite = {"mapalgebra",  NULL,  NULL, mapalgebra_tests};
+void mapalgebra_suite_setup(void);
+void mapalgebra_suite_setup(void)
+{
+	CU_pSuite suite = CU_add_suite("mapalgebra", NULL, NULL);
+	PG_ADD_TEST(suite, test_raster_iterator);
+	PG_ADD_TEST(suite, test_band_reclass);
+	PG_ADD_TEST(suite, test_raster_colormap);
+}
+
