@@ -1,5 +1,4 @@
 -----------------------------------------------------------------------
--- $Id$
 --
 -- Copyright (c) 2009 Mateusz Loskot <mateusz@loskot.net>
 --
@@ -15,7 +14,7 @@
 --
 -- You should have received a copy of the GNU General Public License
 -- along with this program; if not, write to the Free Software Foundation,
--- Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
+-- Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 -----------------------------------------------------------------------
 
 CREATE TABLE rt_bytea_test (
@@ -182,10 +181,8 @@ DROP TABLE rt_bytea_test;
 -----------------------------------------------------------------------
 --- Test out-db as in-db
 -----------------------------------------------------------------------
-/*
-cannot test as by default, out-db is disabled and can only be
-overridden by an ENV variable
-
+SET postgis.gdal_enabled_drivers = 'GTiff';
+SET postgis.enable_outdb_rasters = TRUE;
 WITH foo AS (
 	SELECT
 		rid,
@@ -219,4 +216,3 @@ SELECT
 	rid
 FROM foo
 WHERE encode(outbytea, 'base64') = encode(outin, 'base64');
-*/
