@@ -1,12 +1,21 @@
--- #1485
+/******************************************************************************
+ #1485
+******************************************************************************/
+
 SELECT '#1485', count(*) FROM geometry_columns
 WHERE f_table_name = 'raster_columns';
 
--- #2532
+/******************************************************************************
+ #2532
+******************************************************************************/
+
 SELECT '#2532.1', NULL::raster @ null::geometry;
 SELECT '#2532.2', NULL::geometry @ null::raster;
 
--- #2911
+/******************************************************************************
+ #2911
+******************************************************************************/
+
 WITH data AS ( SELECT '#2911' l, ST_Metadata(ST_Rescale(
  ST_AddBand(
   ST_MakeEmptyRaster(10, 10, 0, 0, 1, -1, 0, 0, 0),
@@ -17,30 +26,33 @@ WITH data AS ( SELECT '#2911' l, ST_Metadata(ST_Rescale(
  )) m
 ) SELECT l, (m).* FROM data;
 
--- #3006
+/******************************************************************************
+ #3006
+******************************************************************************/
+
 SET client_min_messages TO ERROR;
 
 DROP TABLE IF EXISTS test_raster_scale_regular;
-CREATE TABLE test_raster_scale_regular (
-	rid integer,
-	rast raster
-);
-
 DROP TABLE IF EXISTS test_raster_scale_big;
-CREATE TABLE test_raster_scale_big (
-	rid integer,
-	rast raster
-);
-
 DROP TABLE IF EXISTS test_raster_scale_small;
-CREATE TABLE test_raster_scale_small (
-	rid integer,
-	rast raster
+
+CREATE TABLE test_raster_scale_regular (
+  rid integer,
+ 	rast raster
 );
 
+CREATE TABLE test_raster_scale_big (
+  rid integer,
+ 	rast raster
+);
+
+CREATE TABLE test_raster_scale_small (
+ 	rid integer,
+ 	rast raster
+);
 
 CREATE OR REPLACE FUNCTION make_test_raster(
-	table_suffix text,
+  table_suffix text,
 	rid integer,
   scale_x double precision,
   scale_y double precision DEFAULT 1.0
@@ -49,19 +61,19 @@ RETURNS void
 AS $$
 DECLARE
 	rast raster;
-	width integer := 2;
-	height integer := 2;
-	ul_x double precision := 0;
-	ul_y double precision := 0;
-	skew_x double precision := 0;
-	skew_y double precision := 0;
-	initvalue double precision := 1;
-	nodataval double precision := 0;
+ 	width integer := 2;
+ 	height integer := 2;
+ 	ul_x double precision := 0;
+ 	ul_y double precision := 0;
+ 	skew_x double precision := 0;
+ 	skew_y double precision := 0;
+ 	initvalue double precision := 1;
+ 	nodataval double precision := 0;
 BEGIN
 	rast := ST_MakeEmptyRaster(width, height, ul_x, ul_y, scale_x, scale_y, skew_x, skew_y, 0);
 	rast := ST_AddBand(rast, 1, '8BUI', initvalue, nodataval);
 
-	EXECUTE 'INSERT INTO test_raster_scale_' || table_suffix || ' VALUES ($1, $2)' USING rid, rast;
+	EXECUTE format('INSERT INTO test_raster_scale_%s VALUES (%L, %L)', table_suffix, rid, rast);
 	RETURN;
 END;
 $$ LANGUAGE 'plpgsql';
@@ -71,18 +83,18 @@ SELECT make_test_raster('regular', 1, 1.0000001);
 SELECT make_test_raster('regular', 2, 0.9999999);
 SELECT AddRasterConstraints('test_raster_scale_regular'::name, 'rast'::name, 'scale_x', 'scale_y');
 SELECT r_table_name, r_raster_column, scale_x, scale_y FROM raster_columns
-	WHERE  r_raster_column = 'rast' AND r_table_name = 'test_raster_scale_regular';
+  WHERE  r_raster_column = 'rast' AND r_table_name = 'test_raster_scale_regular';
 
--- Issues enforce_scalex_rast constraint violation
+-- Issues enforce_scaley_rast constraint violation
 SELECT make_test_raster('regular', 3, 1.001, 0.9999999);
 
-SELECT make_test_raster('big', 0, -12345678901234.0);
+SELECT make_test_raster('big', 0, -1234567890123456789.0);
 SELECT AddRasterConstraints('test_raster_scale_big'::name, 'rast'::name, 'scale_x', 'scale_y');
 SELECT r_table_name, r_raster_column, scale_x, scale_y FROM raster_columns
   WHERE  r_raster_column = 'rast' AND r_table_name = 'test_raster_scale_big';
 
 -- Issues enforce_scalex_rast constraint violation
-SELECT make_test_raster('big', 1, -123456789012345.0);
+SELECT make_test_raster('big', 1, -12345678901234567890.0);
 
 SELECT make_test_raster('small', 0, 0.00001);
 SELECT make_test_raster('small', 1, 0.000011);
