@@ -276,3 +276,49 @@ SELECT 't3280', 'L1b' || topology.TopoGeo_AddLinestring('bug3280', geom)
  ORDER BY 1;
 SELECT 't3280.end', topology.DropTopology('bug3280');
 
+-- See http://trac.osgeo.org/postgis/ticket/3380
+SELECT 't3380.start', CreateTopology( 'bug3380', 0, 0.01) > 0;
+SELECT 't3380.L1', TopoGeo_AddLinestring('bug3380', '
+LINESTRING(
+1612829.90652844007126987 4841274.48807844985276461,
+1612830.1566380700096488 4841287.23833953030407429,
+1612883.15799825009889901 4841277.73794914968311787)
+', 0);
+SELECT 't3380.L2', TopoGeo_AddLinestring('bug3380', '
+LINESTRING(
+1612790.88055733009241521 4841286.88526585046201944,
+1612830.15823523001745343 4841287.12674008030444384,
+1612829.98813172010704875 4841274.56198261026293039)
+', 0);
+SELECT 't3380.L3', TopoGeo_AddLinestring('bug3380', '
+ LINESTRING(
+1612830.15823523 4841287.12674008,
+1612881.64990281 4841274.56198261)
+', 0);
+SELECT 't3380.end', DropTopology( 'bug3380' );
+
+-- See http://trac.osgeo.org/postgis/ticket/3402
+
+SELECT 't3402.start', CreateTopology('bug3402') > 1;
+SELECT 't3402.L1', TopoGeo_addLinestring('bug3402',
+'010200000003000000C1AABC2B192739418E7DE0E6AB9652411F85EB5119283941F6285CEF2D9652411F85EB5128283941F6285CCF2C965241'
+, 0);
+SELECT 't3402.L2', TopoGeo_addLinestring('bug3402',
+'010200000003000000BCAABC2B192739418F7DE0E6AB96524185EB51382828394115AE47D12C96524187EB51382828394115AE47D12C965241'
+, 0);
+SELECT 't3402.end', DropTopology('bug3402');
+
+-- See http://trac.osgeo.org/postgis/ticket/3412
+SELECT 't3412.start', CreateTopology('bug3412', 0, 0.001) > 0;
+SELECT 't3412.L1', TopoGeo_AddLinestring('bug3412',
+'LINESTRING(
+599671.37 4889664.32,
+599665.52 4889680.18,
+599671.37 4889683.4,
+599671.37 4889781.87
+)'
+::geometry, 0);
+SELECT 't3412.L2', TopoGeo_AddLinestring('bug3412',
+'0102000000020000003AB42BBFEE4C22410010C5A997A6524167BB5DBDEE4C224117FE3DA85FA75241'
+::geometry, 0);
+SELECT 't3412.end', DropTopology('bug3412');
